@@ -21,7 +21,7 @@
 #include "gui_http.h"
 #include "error_numbers.h"
 
-/// represents the contents of project_info.xml
+// represents the contents of project_info.xml
 
 struct PROJECT_INIT {
     char url[256];
@@ -37,10 +37,10 @@ struct PROJECT_INIT {
 
 struct ACCOUNT_IN {
     std::string url;
-        /// the account identifier (user name or email addr)
     std::string email_addr;
-        /// the suggested friendly name for the user during account creation.
+        // the account identifier (user name or email addr)
     std::string user_name;
+        // the suggested friendly name for the user during account creation.
     std::string team_name;
     std::string passwd_hash;
 
@@ -56,7 +56,7 @@ struct GET_PROJECT_CONFIG_OP: public GUI_HTTP_OP {
         gui_http = p;
     }
     virtual ~GET_PROJECT_CONFIG_OP(){}
-    int do_rpc(string url);
+    int do_rpc(std::string url);
     virtual void handle_reply(int http_op_retval);
 };
 
@@ -86,18 +86,6 @@ struct CREATE_ACCOUNT_OP: public GUI_HTTP_OP {
     virtual void handle_reply(int http_op_retval);
 };
 
-struct GET_CURRENT_VERSION_OP: public GUI_HTTP_OP {
-    int error_num;
-
-    GET_CURRENT_VERSION_OP(GUI_HTTP* p){
-        error_num = BOINC_SUCCESS;
-        gui_http = p;
-    }
-    virtual ~GET_CURRENT_VERSION_OP(){}
-    int do_rpc();
-    virtual void handle_reply(int http_op_retval);
-};
-
 struct GET_PROJECT_LIST_OP: public GUI_HTTP_OP {
     int error_num;
 
