@@ -15,8 +15,8 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
-#ifndef _TASK_
-#define _TASK_
+#ifndef _APP_H_
+#define _APP_H_
 
 #ifndef _WIN32
 #include <cstdio>
@@ -28,6 +28,13 @@
 #include "app_ipc.h"
 #include "procinfo.h"
 
+#define ABORT_TIMEOUT   60
+    // if we send app <abort> request, wait this long before killing it.
+    // This gives it time to download symbol files (which can be several MB)
+    // and write stack trace to stderr
+#define QUIT_TIMEOUT    10
+    // Same, for <quit>.  Shorter because no stack trace is generated
+
 // values for preempt_type
 //
 #define REMOVE_NEVER        0
@@ -35,7 +42,7 @@
 #define REMOVE_MAYBE_SCHED  2
 #define REMOVE_ALWAYS       3
 
-class CLIENT_STATE;
+struct CLIENT_STATE;
 typedef int PROCESS_ID;
 
 #define MAX_STDERR_LEN  65536
@@ -43,16 +50,15 @@ typedef int PROCESS_ID;
     // before sending to server,
     // to protect against apps that write unbounded amounts.
 
-/// Represents a job in progress.
+// Represents a job in progress.
 
-/// When an active task is created, it is assigned a "slot"
-/// which determines the directory it runs in.
-/// This doesn't change over the life of the active task;
-/// thus the task can use the slot directory for temp files
-/// that BOINC doesn't know about.
+// When an active task is created, it is assigned a "slot"
+// which determines the directory it runs in.
+// This doesn't change over the life of the active task;
+// thus the task can use the slot directory for temp files
+// that BOINC doesn't know about.
 
-class ACTIVE_TASK {
-public:
+struct ACTIVE_TASK {
 #ifdef _WIN32
     HANDLE process_handle, shm_handle;
     bool kill_all_children();
@@ -64,73 +70,76 @@ public:
     PROCESS_ID pid;
 	PROCINFO procinfo;
 
-    //// START OF ITEMS SAVED IN STATE FILE
+    // START OF ITEMS SAVED IN STATE FILE
     int _task_state;
-        /// subdirectory of slots/ where this runs
     int slot;
-        /// CPU at the last checkpoint
-        /// Note: "CPU time" refers to the sum over all episodes.
-        /// (not counting the "lost" time after the last checkpoint
-        /// in episodes before the current one)
-        /// TODO: debt should be based on FLOPs, not CPU time
-        /// CPU time at the start of current episode
+        // subdirectory of slots/ where this runs
     double checkpoint_cpu_time;
-        /// elapsed time at last checkpoint
+        // CPU at the last checkpoint
+        // Note: "CPU time" refers to the sum over all episodes.
+        // (not counting the "lost" time after the last checkpoint
+        // in episodes before the current one)
     double checkpoint_elapsed_time;
-        /// App's estimate of how much of the work unit is done.
-        /// Passed from the application via an API call;
-        /// will be zero if the app doesn't use this call
-    double fraction_done;
-        /// most recent CPU time reported by app
+        // elapsed time at last checkpoint
+    double checkpoint_fraction_done;
+        // fraction done at last checkpoint
+    double checkpoint_fraction_done_elapsed_time;
+        // fraction done elapsed time at last checkpoint
     double current_cpu_time;
+        // most recent CPU time reported by app
     bool once_ran_edf;
 
-    //// END OF ITEMS SAVED IN STATE FILE
+    // END OF ITEMS SAVED IN STATE FILE
 
+    double fraction_done;
+        // App's estimate of how much of the work unit is done.
+        // Passed from the application via an API call;
+        // will be zero if the app doesn't use this call
+    double fraction_done_elapsed_time;
+        // elapsed time when fraction done was last reported
     int scheduler_state;
     int next_scheduler_state; // temp
     int signal;
-        /// Wall time at the start of the current run interval
     double run_interval_start_wall_time;
-        /// wall time at the last checkpoint
+        // Wall time at the start of the current run interval
     double checkpoint_wall_time;
-        /// current total elapsed (running) time
+        // wall time at the last checkpoint
     double elapsed_time;
-        /// disk used by output files and temp files of this task
-    int current_disk_usage(double&);
-        /// directory where process runs (relative)
+        // current total elapsed (running) time
     char slot_dir[256];
-        /// same, absolute
-
-        /// This is used only to run graphics apps
-        /// (that way don't have to worry about top-level dirs
-        /// being non-readable, etc).
+        // directory where process runs (relative)
     char slot_path[512];
-        /// abort if elapsed time exceeds this
+        // same, absolute
+        // This is used only to run graphics apps
+        // (that way don't have to worry about top-level dirs
+        // being non-readable, etc).
     double max_elapsed_time;
-        /// abort if disk usage (in+out+temp) exceeds this
+        // abort if elapsed time exceeds this
     double max_disk_usage;
-        /// abort if memory usage exceeds this
+        // abort if disk usage (in+out+temp) exceeds this
     double max_mem_usage;
+        // abort if memory usage exceeds this
     bool have_trickle_down;
     bool send_upload_file_status;
-        /// working set too large to run now
     bool too_large;
+        // working set too large to run now
     bool needs_shmem;               // waiting for a free shared memory segment
     int want_network;
-        /// This task wants to do network comm (for F@h)
-
-        /// this is passed via share-memory message (app_status channel)
+        // This task wants to do network comm (for F@h)
+        // this is passed via share-memory message (app_status channel)
     double abort_time;
-        /// when we sent an abort message to this app
-
-        /// kill it 5 seconds later if it doesn't exit
+        // when we sent an abort message to this app
+        // kill it 5 seconds later if it doesn't exit
     double quit_time;
-        /// when we sent a quit message; kill if still there after 10 sec
     int premature_exit_count;
+        // when we sent a quit message; kill if still there after 10 sec
+    bool overdue_checkpoint;
+        // running past end of time slice because not checkpointed;
+        // when we do checkpoint, reschedule
+    double last_deadline_miss_time;
 
-        /// core/app shared mem segment
     APP_CLIENT_SHM app_client_shm;
+        // core/app shared mem segment
     MSG_QUEUE graphics_request_queue;
     MSG_QUEUE process_control_queue;
 
@@ -141,14 +150,14 @@ public:
 
     // info related to app's graphics mode (win, screensaver, etc.)
     //
-        /// mode acked by app
     int graphics_mode_acked;
-        /// mode before last screensaver request
+        // mode acked by app
     int graphics_mode_before_ss;
+        // mode before last screensaver request
     double graphics_mode_ack_timeout;
 
 #ifdef SIM
-    double cpu_time_left;
+    double flops_left;
 #endif
 
 #if (defined (__APPLE__) && (defined(__i386__) || defined(__x86_64__)))
@@ -174,6 +183,8 @@ public:
     int init(RESULT*);
     void cleanup_task();
 
+    int current_disk_usage(double&);
+        // disk used by output files and temp files of this task
     void get_free_slot(RESULT*);
     int start(bool first_time);         // start a process
     int request_exit();
@@ -181,24 +192,24 @@ public:
         // i.e. by sending a <quit> message
     int request_abort();                // send "abort" message
     bool process_exists();
-        /// Kill process forcibly,
 
-        /// Unix: send a SIGKILL signal, Windows: TerminateProcess()
-		/// if restart is true, arrange for resulted to get restarted;
-		/// otherwise it ends with an error
     int kill_task(bool restart);
-        /// tell a process to stop executing (but stay in mem)
+        // Kill process forcibly,
+		// otherwise it ends with an error
+        // Unix: send a SIGKILL signal, Windows: TerminateProcess()
+		// if restart is true, arrange for resulted to get restarted;
 
-        /// Done by sending it a <suspend> message
     int suspend();
-        /// Undo a suspend: send a <resume> message
+        // tell a process to stop executing (but stay in mem)
+        // Done by sending it a <suspend> message
     int unsuspend();
-        /// can be called whether or not process exists
+        // Undo a suspend: send a <resume> message
     int abort_task(int exit_status, const char*);
-        /// return true if this task has exited
+        // can be called whether or not process exists
     bool has_task_exited();
-        /// preempt (via suspend or quit) a running task
+        // return true if this task has exited
     int preempt(int preempt_type);
+        // preempt (via suspend or quit) a running task
     int resume_or_start(bool);
     void send_network_available();
 #ifdef _WIN32
@@ -212,7 +223,7 @@ public:
 
     bool get_app_status_msg();
     bool get_trickle_up_msg();
-    double est_dur(bool for_work_fetch);
+    double est_dur();
     bool read_stderr_file();
     bool finish_file_present();
     bool temporary_exit_file_present(double&);
@@ -230,7 +241,7 @@ public:
     int parse(MIOFILE&);
 };
 
-/// Represents the set of all jobs in progress
+// Represents the set of all jobs in progress
 
 class ACTIVE_TASK_SET {
 public:
@@ -275,9 +286,9 @@ public:
     int parse(MIOFILE&);
 };
 
-extern bool exclusive_app_running;
-extern bool exclusive_gpu_app_running;
-extern bool gpu_suspended;
+extern double exclusive_app_running;    // last time an exclusive app was running
+extern double exclusive_gpu_app_running;
+extern int gpu_suspend_reason;
 extern double non_boinc_cpu_usage;
 
 #endif
