@@ -22,6 +22,9 @@
 #ifdef _WIN32
 #include "boinc_win.h"
 #include "win_util.h"
+#ifdef _MSC_VER
+#define snprintf _snprintf
+#endif
 #ifndef STATUS_SUCCESS
 #define STATUS_SUCCESS 0x0                 // may be in ntstatus.h
 #endif
@@ -71,6 +74,50 @@ using std::vector;
 
 #include "app.h"
 
+// Do periodic checks on running apps:
+// - get latest CPU time and % done info
+// - check if any has exited, and clean up
+// - see if any has exceeded its CPU or disk space limits, and abort it
+//
+bool ACTIVE_TASK_SET::poll() {
+    bool action;
+    unsigned int i;
+    static double last_time = 0;
+    if (gstate.now - last_time < TASK_POLL_PERIOD) return false;
+    last_time = gstate.now;
+
+    action = check_app_exited();
+    send_heartbeats();
+    send_trickle_downs();
+    graphics_poll();
+    process_control_poll();
+    action |= check_rsc_limits_exceeded();
+    get_msgs();
+    for (i=0; i<active_tasks.size(); i++) {
+        ACTIVE_TASK* atp = active_tasks[i];
+        if (atp->task_state() == PROCESS_ABORT_PENDING) {
+            if (gstate.now > atp->abort_time + ABORT_TIMEOUT) {
+                atp->kill_task(false);
+            }
+        }
+        if (atp->task_state() == PROCESS_QUIT_PENDING) {
+            if (gstate.now > atp->quit_time + QUIT_TIMEOUT) {
+                atp->kill_task(true);
+            }
+        }
+    }
+
+    if (action) {
+        gstate.set_client_state_dirty("ACTIVE_TASK_SET::poll");
+    }
+
+    return action;
+}
+
+#if 0
+// deprecated; TerminateProcessById() doesn't work if
+// the process is running as a different user
+//
 #ifdef _WIN32
 bool ACTIVE_TASK::kill_all_children() {
 	unsigned int i,j;
@@ -97,6 +144,7 @@ bool ACTIVE_TASK::kill_all_children() {
     return true;
 }
 #endif
+#endif
 
 // Send a quit message.
 //
@@ -121,25 +169,43 @@ int ACTIVE_TASK::request_abort() {
     return 0;
 }
 
-// Kill the task by OS-specific means.
-//
-int ACTIVE_TASK::kill_task(bool restart) {
+static void kill_app_process(int pid) {
 #ifdef _WIN32
-    TerminateProcess(process_handle, 1);
+    HANDLE h = OpenProcess(READ_CONTROL | PROCESS_TERMINATE, false, pid);
+    if (h == NULL) return;
+    TerminateProcess(h, 1);
+    CloseHandle(h);
 #else
 #ifdef SANDBOX
     kill_via_switcher(pid);
-    // Also kill app directly, just to be safe
-    //
 #endif
     kill(pid, SIGKILL);
 #endif
+}
+
+// Kill the task (and descendants) by OS-specific means.
+//
+int ACTIVE_TASK::kill_task(bool restart) {
+    vector<int>pids;
+#ifdef _WIN32
+    // On Win, in protected mode we won't be able to get
+    // handles for the descendant processes;
+    // all we can do is terminate the main process,
+    // using the handle we got when we created it.
+    //
+    TerminateProcess(process_handle, 1);
+#endif
+    get_descendants(pid, pids);
+    pids.push_back(pid);
+    for (unsigned int i=0; i<pids.size(); i++) {
+        kill_app_process(pids[i]);
+    }
     cleanup_task();
 	if (restart) {
 		set_task_state(PROCESS_UNINITIALIZED, "kill_task");
         char buf[256];
         sprintf(buf, "restarting %s", result->name);
-		gstate.request_enforce_schedule(result->project, buf);
+		gstate.request_schedule_cpus(buf);
 	} else {
 		set_task_state(PROCESS_ABORTED, "kill_task");
 	}
@@ -251,7 +317,7 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
 
     if (log_flags.task_debug) {
         msg_printf(result->project, MSG_INFO,
-            "[task_debug] Process for %s exited",
+            "[task] Process for %s exited",
             result->name
         );
     }
@@ -276,6 +342,11 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
             }
             double x;
             if (temporary_exit_file_present(x)) {
+                if (log_flags.task_debug) {
+                    msg_printf(result->project, MSG_INFO,
+                        "[task] task called temporary_exit(%f)", x
+                    );
+                }
                 set_task_state(PROCESS_UNINITIALIZED, "temporary exit");
                 will_restart = true;
                 result->schedule_backoff = gstate.now + x;
@@ -305,11 +376,11 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
             );
             if (log_flags.task_debug) {
                 msg_printf(result->project, MSG_INFO,
-                    "[task_debug] Process for %s exited",
+                    "[task] Process for %s exited",
                     result->name
                 );
                 msg_printf(result->project, MSG_INFO,
-                    "[task_debug] exit code %d (0x%x): %s",
+                    "[task] exit code %d (0x%x): %s",
                     exit_code, exit_code,
                     windows_format_error_string(exit_code, szError, sizeof(szError))
                 );
@@ -335,18 +406,30 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
                     handle_premature_exit(will_restart);
                 }
             }
-            if (log_flags.task_debug) {
-                msg_printf(result->project, MSG_INFO,
-                    "[task_debug] exit status %d\n",
-                    result->exit_status
-                );
+            double x;
+            if (temporary_exit_file_present(x)) {
+                if (log_flags.task_debug) {
+                    msg_printf(result->project, MSG_INFO,
+                        "[task] task called temporary_exit(%f)", x
+                    );
+                }
+                set_task_state(PROCESS_UNINITIALIZED, "temporary exit");
+                will_restart = true;
+                result->schedule_backoff = gstate.now + x;
+            } else {
+                if (log_flags.task_debug) {
+                    msg_printf(result->project, MSG_INFO,
+                        "[task] process exited with status %d\n",
+                        result->exit_status
+                    );
+                }
             }
         } else if (WIFSIGNALED(stat)) {
             int got_signal = WTERMSIG(stat);
 
             if (log_flags.task_debug) {
                 msg_printf(result->project, MSG_INFO,
-                    "[task_debug] process got signal %d", signal
+                    "[task] process got signal %d", signal
                 );
             }
 
@@ -384,7 +467,7 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
 
     cleanup_task();
 
-    if (gstate.exit_after_finish) {
+    if (config.exit_after_finish) {
         exit(0);
     }
 
@@ -455,16 +538,19 @@ void ACTIVE_TASK_SET::send_heartbeats() {
 			"<max_wss>%e</max_wss>",
 			atp->procinfo.working_set_size, ar
 		);
+        if (gstate.network_suspended) {
+            strcat(buf, "<network_suspended/>");
+        }
         bool sent = atp->app_client_shm.shm->heartbeat.send_msg(buf);
-        if (log_flags.app_msg_send) {
+        if (log_flags.heartbeat_debug) {
             if (sent) {
                 msg_printf(atp->result->project, MSG_INFO,
-                    "[app_msg_send] sent heartbeat to %s",
+                    "[heartbeat] Heartbeat sent to task %s",
                     atp->result->name
                 );
             } else {
                 msg_printf(atp->result->project, MSG_INFO,
-                    "[app_msg_send] failed to send heartbeat to %s",
+                    "[heartbeat] Heartbeat to task %s failed, previous message unread",
                     atp->result->name
                 );
             }
@@ -521,7 +607,7 @@ bool ACTIVE_TASK_SET::check_app_exited() {
             if (log_flags.task_debug) {
                 char errmsg[1024];
                 msg_printf(atp->result->project, MSG_INFO,
-                    "[task_debug] task %s GetExitCodeProcess() failed - %s GLE %d (0x%x)",
+                    "[task] task %s GetExitCodeProcess() failed - %s GLE %d (0x%x)",
                     atp->result->name,
                     windows_format_error_string(
                         GetLastError(), errmsg, sizeof(errmsg)
@@ -609,10 +695,12 @@ bool ACTIVE_TASK_SET::check_rsc_limits_exceeded() {
     for (i=0; i<active_tasks.size(); i++) {
         atp = active_tasks[i];
         if (atp->task_state() != PROCESS_EXECUTING) continue;
-		if (!atp->result->project->non_cpu_intensive && (atp->elapsed_time > atp->max_elapsed_time)) {
-			msg_printf(atp->result->project, MSG_INFO,
-				"Aborting task %s: exceeded elapsed time limit %f\n",
-				atp->result->name, atp->max_elapsed_time
+        if (!atp->result->non_cpu_intensive() && (atp->elapsed_time > atp->max_elapsed_time)) {
+            msg_printf(atp->result->project, MSG_INFO,
+                "Aborting task %s: exceeded elapsed time limit %.2f (%.2fG/%.2fG)",
+                atp->result->name, atp->max_elapsed_time,
+                atp->result->wup->rsc_fpops_bound/1e9,
+                atp->result->avp->flops/1e9
 			);
 			atp->abort_task(ERR_RSC_LIMIT_EXCEEDED, "Maximum elapsed time exceeded");
 			did_anything = true;
@@ -666,7 +754,7 @@ int ACTIVE_TASK::abort_task(int exit_status, const char* msg) {
 // check for the stderr file, copy to result record
 //
 bool ACTIVE_TASK::read_stderr_file() {
-    std::string stderr_file;
+    char* buf1, *buf2;
     char path[256];
 
     // truncate stderr output to the last 63KB;
@@ -675,12 +763,13 @@ bool ACTIVE_TASK::read_stderr_file() {
     int max_len = 63*1024;
     sprintf(path, "%s/%s", slot_dir, STDERR_FILE);
     if (!boinc_file_exists(path)) return false;
-    if (read_file_string(path, stderr_file, max_len, !config.stderr_head)) {
+    if (read_file_malloc(path, buf1, max_len, !config.stderr_head)) {
         return false;
     }
-
+    buf2 = (char*)malloc(2*max_len);
+    non_ascii_escape(buf1, buf2, 2*max_len);
     result->stderr_out += "<stderr_txt>\n";
-    result->stderr_out += stderr_file;
+    result->stderr_out += buf2;
     result->stderr_out += "\n</stderr_txt>\n";
     return true;
 }
@@ -747,14 +836,37 @@ void ACTIVE_TASK_SET::request_reread_app_info() {
 // or when a project is detached or reset
 //
 int ACTIVE_TASK_SET::exit_tasks(PROJECT* proj) {
+    if (log_flags.task_debug) {
+        msg_printf(NULL, MSG_INFO, "[task_debug] requesting tasks to exit");
+    }
     request_tasks_exit(proj);
 
-    // Wait 5 seconds for them to exit normally; if they don't then kill them
+    // Wait 15 seconds for them to exit normally; if they don't then kill them
     //
-    if (wait_for_exit(5, proj)) {
+    if (wait_for_exit(MAX_EXIT_TIME, proj)) {
+        if (log_flags.task_debug) {
+            msg_printf(NULL, MSG_INFO,
+                "[task_debug] all tasks haven't exited after %d sec; killing them",
+                MAX_EXIT_TIME
+            );
+        }
         kill_tasks(proj);
+        if (wait_for_exit(5, proj)) {
+            if (log_flags.task_debug) {
+                msg_printf(NULL, MSG_INFO,
+                    "[task_debug] tasks still not exited after 5 secs; giving up"
+                );
+            }
+        } else {
+            if (log_flags.task_debug) {
+                msg_printf(NULL, MSG_INFO, "[task_debug] all tasks exited");
+            }
+        }
+    } else {
+        if (log_flags.task_debug) {
+            msg_printf(NULL, MSG_INFO, "[task_debug] all tasks exited");
+        }
     }
-    wait_for_exit(5, proj);
 
     // get final checkpoint_cpu_times
     //
@@ -820,19 +932,26 @@ void ACTIVE_TASK_SET::suspend_all(int reason) {
         if (atp->task_state() != PROCESS_EXECUTING) continue;
         switch (reason) {
         case SUSPEND_REASON_CPU_THROTTLE:
-			// if we're doing CPU throttling, don't bother suspending apps
-			// that don't use a full CPU
-			//
-			if (atp->result->project->non_cpu_intensive) continue;
-			if (atp->app_version->avg_ncpus < 1) continue;
+            // if we're doing CPU throttling, don't bother suspending apps
+            // that don't use a full CPU
+            //
+            if (atp->result->non_cpu_intensive()) continue;
+            if (atp->app_version->avg_ncpus < 1) continue;
             atp->preempt(REMOVE_NEVER);
             break;
         case SUSPEND_REASON_BENCHMARKS:
             atp->preempt(REMOVE_NEVER);
             break;
         case SUSPEND_REASON_CPU_USAGE:
-            if (atp->result->project->non_cpu_intensive) break;
-            // fall through
+            // If we're suspending because of non-BOINC CPU load,
+            // don't remove from memory.
+            // Some systems do a security check when apps are launched,
+            // which uses a lot of CPU.
+            // Avoid going into a preemption loop.
+            //
+            if (atp->result->non_cpu_intensive()) break;
+            atp->preempt(REMOVE_NEVER);
+            break;
         default:
             atp->preempt(REMOVE_MAYBE_USER);
         }
@@ -988,7 +1107,10 @@ bool ACTIVE_TASK::get_app_status_msg() {
         // until the app's first call to boinc_fraction_done().
         // So ignore zeros.
         //
-        if (fd) fraction_done = fd;
+        if (fd) {
+            fraction_done = fd;
+            fraction_done_elapsed_time = elapsed_time;
+        }
     }
     parse_double(msg_buf, "<current_cpu_time>", current_cpu_time);
     parse_double(msg_buf, "<checkpoint_cpu_time>", checkpoint_cpu_time);
@@ -1020,12 +1142,22 @@ bool ACTIVE_TASK::get_trickle_up_msg() {
     if (!app_client_shm.shm) return false;
     if (app_client_shm.shm->trickle_up.get_msg(msg_buf)) {
         if (match_tag(msg_buf, "<have_new_trickle_up/>")) {
+            if (log_flags.app_msg_receive) {
+                msg_printf(NULL, MSG_INFO,
+                    "[app_msg_receive] got msg from slot %d: %s", slot, msg_buf
+                );
+            }
             retval = move_trickle_file();
             if (!retval) {
                 wup->project->trickle_up_pending = true;
             }
         }
         if (match_tag(msg_buf, "<have_new_upload_file/>")) {
+            if (log_flags.app_msg_receive) {
+                msg_printf(NULL, MSG_INFO,
+                    "[app_msg_receive] got msg from slot %d: %s", slot, msg_buf
+                );
+            }
             handle_upload_files();
         }
         found = true;
@@ -1068,18 +1200,22 @@ void ACTIVE_TASK_SET::get_msgs() {
             if (old_time != atp->checkpoint_cpu_time) {
                 char buf[256];
                 sprintf(buf, "%s checkpointed", atp->result->name);
-                gstate.request_enforce_schedule(atp->result->project, buf);
+                if (atp->overdue_checkpoint) {
+                    gstate.request_schedule_cpus(buf);
+                }
                 atp->checkpoint_wall_time = gstate.now;
                 atp->premature_exit_count = 0;
                 atp->checkpoint_elapsed_time = atp->elapsed_time;
-                if (log_flags.task_debug) {
+                atp->checkpoint_fraction_done = atp->fraction_done;
+                atp->checkpoint_fraction_done_elapsed_time = atp->fraction_done_elapsed_time;
+                if (log_flags.checkpoint_debug) {
                     msg_printf(atp->wup->project, MSG_INFO,
-                        "[task_debug] result %s checkpointed",
+                        "[checkpoint] result %s checkpointed",
                         atp->result->name
                     );
-                } else if (log_flags.checkpoint_debug) {
+                } else if (log_flags.task_debug) {
                     msg_printf(atp->wup->project, MSG_INFO,
-                        "[checkpoint_debug] result %s checkpointed",
+                        "[task] result %s checkpointed",
                         atp->result->name
                     );
                 }
@@ -1131,15 +1267,27 @@ void ACTIVE_TASK::read_task_state_file() {
     // sanity checks - project and result name must match
     //
     if (!parse_str(buf, "<project_master_url>", s, sizeof(s))) {
+        msg_printf(wup->project, MSG_INTERNAL_ERROR,
+            "no project URL in task state file"
+        );
         return;
     }
     if (strcmp(s, result->project->master_url)) {
+        msg_printf(wup->project, MSG_INTERNAL_ERROR,
+            "wrong project URL in task state file"
+        );
         return;
     }
     if (!parse_str(buf, "<result_name>", s, sizeof(s))) {
+        msg_printf(wup->project, MSG_INTERNAL_ERROR,
+            "no task name in task state file"
+        );
         return;
     }
     if (strcmp(s, result->name)) {
+        msg_printf(wup->project, MSG_INTERNAL_ERROR,
+            "wrong task name in task state file"
+        );
         return;
     }
     if (parse_double(buf, "<checkpoint_cpu_time>", x)) {
