@@ -22,9 +22,13 @@
 #ifdef _WIN32
 #include "boinc_win.h"
 #include "win_util.h"
+#ifdef _MSC_VER
+#define snprintf _snprintf
+#define strdup   _strdup
+#endif
 #else
 #include "config.h"
-#ifdef HAVE_SCHED_SETSCHEDULER
+#if defined (HAVE_SCHED_SETSCHEDULER) && defined (__linux__)
 #include <sched.h>
 #endif
 #if HAVE_SYS_TIME_H
@@ -103,11 +107,9 @@ typedef BOOL (WINAPI *tDEB)(LPVOID lpEnvironment);
 //
 #ifndef _WIN32
 static void debug_print_argv(char** argv) {
-    msg_printf(0, MSG_INFO, "[task_debug] Arguments:");
+    msg_printf(0, MSG_INFO, "[task] Arguments:");
     for (int i=0; argv[i]; i++) {
-        msg_printf(0, MSG_INFO,
-            "[task_debug]    argv[%d]: %s\n", i, argv[i]
-        );
+        msg_printf(0, MSG_INFO, "[task]    argv[%d]: %s\n", i, argv[i]);
     }
 }
 #endif
@@ -117,7 +119,10 @@ static void debug_print_argv(char** argv) {
 static void coproc_cmdline(
     int rsc_type, RESULT* rp, double ninstances, char* cmdline
 ) {
-    COPROC* coproc = (rsc_type==RSC_TYPE_CUDA)?(COPROC*)coproc_cuda:(COPROC*)coproc_ati;
+    COPROC* coproc = (rsc_type==RSC_TYPE_CUDA)
+        ?(COPROC*)&gstate.host_info.coprocs.cuda
+        :(COPROC*)&gstate.host_info.coprocs.ati
+    ;
     for (int j=0; j<ninstances; j++) {
         int k = rp->coproc_indices[j];
         // sanity check
@@ -201,10 +206,12 @@ int ACTIVE_TASK::write_app_init_file() {
     aid.app_version = app_version->version_num;
     safe_strcpy(aid.app_name, wup->app->name);
     safe_strcpy(aid.symstore, wup->project->symstore);
-    safe_strcpy(aid.acct_mgr_url, gstate.acct_mgr_info.acct_mgr_url);
+    safe_strcpy(aid.acct_mgr_url, gstate.acct_mgr_info.master_url);
     if (wup->project->project_specific_prefs.length()) {
         aid.project_preferences = strdup(wup->project->project_specific_prefs.c_str());
     }
+    aid.userid = wup->project->userid;
+    aid.teamid = wup->project->teamid;
     aid.hostid = wup->project->hostid;
     safe_strcpy(aid.user_name, wup->project->user_name);
     safe_strcpy(aid.team_name, wup->project->team_name);
@@ -215,6 +222,7 @@ int ACTIVE_TASK::write_app_init_file() {
     strcpy(aid.authenticator, wup->project->authenticator);
     aid.slot = slot;
     strcpy(aid.wu_name, wup->name);
+    strcpy(aid.result_name, result->name);
     aid.user_total_credit = wup->project->user_total_credit;
     aid.user_expavg_credit = wup->project->user_expavg_credit;
     aid.host_total_credit = wup->project->host_total_credit;
@@ -383,7 +391,7 @@ int ACTIVE_TASK::copy_output_files() {
 //
 int ACTIVE_TASK::start(bool first_time) {
     char exec_name[256], file_path[256], buf[256], exec_path[256];
-    char cmdline[8192];
+    char cmdline[80000];    // 64KB plus some extra
     unsigned int i;
     FILE_REF fref;
     FILE_INFO* fip;
@@ -393,17 +401,6 @@ int ACTIVE_TASK::start(bool first_time) {
     //
     bool high_priority = (app_version->avg_ncpus < 1);
 
-    if (first_time && log_flags.task) {
-        msg_printf(wup->project, MSG_INFO,
-            "Starting %s", result->name
-        );
-    }
-    if (log_flags.cpu_sched) {
-        msg_printf(wup->project, MSG_INFO,
-            "[cpu_sched] Starting %s%s", result->name, first_time?" (initial)":"(resume)"
-        );
-    }
-
     if (wup->project->verify_files_on_app_start) {
         fip=0;
         retval = gstate.input_files_available(result, true, &fip);
@@ -718,7 +715,7 @@ int ACTIVE_TASK::start(bool first_time) {
 
     if (log_flags.task_debug) {
         msg_printf(wup->project, MSG_INFO,
-            "[task_debug] ACTIVE_TASK::start(): forked process: pid %d\n", pid
+            "[task] ACTIVE_TASK::start(): forked process: pid %d\n", pid
         );
     }
 
@@ -884,7 +881,7 @@ int ACTIVE_TASK::start(bool first_time) {
                 perror("setpriority");
             }
 #endif
-#ifdef HAVE_SCHED_SETSCHEDULER
+#if defined (HAVE_SCHED_SETSCHEDULER) && defined(SCHED_BATCH) && defined (__linux__)
             if (!high_priority) {
                 struct sched_param p;
                 p.sched_priority = 0;
@@ -929,7 +926,7 @@ int ACTIVE_TASK::start(bool first_time) {
 
     if (log_flags.task_debug) {
         msg_printf(wup->project, MSG_INFO,
-            "[task_debug] ACTIVE_TASK::start(): forked process: pid %d\n", pid
+            "[task] ACTIVE_TASK::start(): forked process: pid %d\n", pid
         );
     }
 
@@ -945,10 +942,10 @@ error:
     // Verify it to trigger another download.
     //
     gstate.input_files_available(result, true);
-    gstate.report_result_error(*result, buf);
+    gstate.report_result_error(*result, "couldn't start %s: %d", buf, retval);
     if (log_flags.task_debug) {
         msg_printf(wup->project, MSG_INFO,
-            "[task_debug] couldn't start app: %s", buf
+            "[task] couldn't start app: %s", buf
         );
     }
     set_task_state(PROCESS_COULDNT_START, "start");
