@@ -19,11 +19,8 @@
 
 #ifdef _WIN32
 #include "boinc_win.h"
-#else 
+#else
 #include "config.h"
-#endif
-
-#ifndef _WIN32
 #include <cstring>
 #endif
 
@@ -235,6 +232,7 @@ void AUTO_UPDATE::install() {
 // Wait an additional 10 seconds in any case
 
 void AUTO_UPDATE::poll() {
+#ifndef SIM
     if (!present) return;
     static double last_time = 0;
     static bool ready_to_install = false;
@@ -263,6 +261,7 @@ void AUTO_UPDATE::poll() {
         ready_to_install = true;
         gstate.gui_rpcs.send_quits();
     }
+#endif
 }
 
 int VERSION_INFO::parse(MIOFILE& in) {
