@@ -56,43 +56,43 @@ void usage() {
     fprintf(stderr, "\n\
 usage: boinccmd [--host hostname] [--passwd passwd] command\n\n\
 Commands:\n\
- --lookup_account URL email passwd\n\
  --create_account URL email passwd name\n\
- --project_attach URL auth          attach to project\n\
- --join_acct_mgr URL name passwd    attach account manager\n\
- --quit_acct_mgr                    quit current account manager\n\
- --get_state                        show entire state\n\
- --get_results                      show results\n\
- --get_simple_gui_info              show status of projects and active results\n\
- --get_file_transfers               show file transfers\n\
- --get_project_status               show status of all attached projects\n\
- --get_disk_usage                   show disk usage\n\
- --get_proxy_settings\n\
- --get_messages [ seqno ]           show messages > seqno\n\
- --get_message_count                show largest message seqno\n\
- --get_host_info\n\
- --version, -V                      show core client version\n\
- --result url result_name op        job operation\n\
-   op = suspend | resume | abort | graphics_window | graphics_fullscreen\n\
- --project URL op                   project operation\n\
-   op = reset | detach | update | suspend | resume | nomorework | allowmorework\n\
  --file_transfer URL filename op    file transfer operation\n\
    op = retry | abort\n\
- --set_run_mode mode duration       set run mode for given duration\n\
-   mode = always | auto | never\n\
+ --get_cc_status\n\
+ --get_disk_usage                   show disk usage\n\
+ --get_file_transfers               show file transfers\n\
+ --get_host_info\n\
+ --get_message_count                show largest message seqno\n\
+ --get_messages [ seqno ]           show messages > seqno\n\
+ --get_notices [ seqno ]            show notices > seqno\n\
+ --get_project_config URL\n\
+ --get_project_status               show status of all attached projects\n\
+ --get_proxy_settings\n\
+ --get_simple_gui_info              show status of projects and active tasks\n\
+ --get_state                        show entire state\n\
+ --get_tasks                        show tasks\n\
+ --join_acct_mgr URL name passwd    attach account manager\n\
+ --lookup_account URL email passwd\n\
+ --network_available\n\
+ --project URL op                   project operation\n\
+   op = reset | detach | update | suspend | resume | nomorework | allowmorework\n\
+ --project_attach URL auth          attach to project\n\
+ --quit                             tell client to exit\n\
+ --quit_acct_mgr                    quit current account manager\n\
+ --read_cc_config\n\
+ --read_global_prefs_override\n\
+ --run_benchmarks\n\
+ --set_debts URL1 std1 ltd1 [URL2 std2 ltd2 ...]\n\
  --set_gpu_mode mode duration       set GPU run mode for given duration\n\
    mode = always | auto | never\n\
  --set_network_mode mode duration\n\
  --set_proxy_settings\n\
- --run_benchmarks\n\
- --read_global_prefs_override\n\
- --quit\n\
- --read_cc_config\n\
- --set_debts URL1 std1 ltd1 [URL2 std2 ltd2 ...]\n\
- --get_project_config URL\n\
- --get_project_config_poll\n\
- --network_available\n\
- --get_cc_status\n\
+ --set_run_mode mode duration       set run mode for given duration\n\
+   mode = always | auto | never\n\
+ --task url task_name op            task operation\n\
+   op = suspend | resume | abort | graphics_window | graphics_fullscreen\n\
+ --version, -V                      show core client version\n\
 "
 );
     exit(1);
@@ -129,9 +129,9 @@ char* next_arg(int argc, char** argv, int& i) {
 
 const char* prio_name(int prio) {
     switch (prio) {
-    case 1: return "low";
-    case 2: return "medium";
-    case 3: return "high";
+    case MSG_INFO: return "low";
+    case MSG_USER_ALERT: return "user notification";
+    case MSG_INTERNAL_ERROR: return "internal error";
     }
     return "unknown";
 }
@@ -140,6 +140,7 @@ int main(int argc, char** argv) {
     RPC_CLIENT rpc;
     int i, retval, port=0;
     MESSAGES messages;
+    NOTICES notices;
 	char passwd_buf[256], hostname_buf[256], *hostname=0;
     char* passwd = passwd_buf, *p;
 
@@ -218,7 +219,7 @@ int main(int argc, char** argv) {
         CC_STATE state;
         retval = rpc.get_state(state);
         if (!retval) state.print();
-    } else if (!strcmp(cmd, "--get_results")) {
+    } else if (!strcmp(cmd, "--get_tasks")) {
         RESULTS results;
         retval = rpc.get_results(results);
         if (!retval) results.print();
@@ -238,12 +239,12 @@ int main(int argc, char** argv) {
         DISK_USAGE du;
         retval = rpc.get_disk_usage(du);
         if (!retval) du.print();
-    } else if (!strcmp(cmd, "--result")) {
+    } else if (!strcmp(cmd, "--task")) {
         RESULT result;
         char* project_url = next_arg(argc, argv, i);
-        result.project_url = project_url;
+        strcpy(result.project_url, project_url);
         char* name = next_arg(argc, argv, i);
-        result.name = name;
+        strcpy(result.name, name);
         char* op = next_arg(argc, argv, i);
         if (!strcmp(op, "suspend")) {
             retval = rpc.result_op(result, "suspend");
@@ -264,7 +265,7 @@ int main(int argc, char** argv) {
         }
     } else if (!strcmp(cmd, "--project")) {
         PROJECT project;
-        project.master_url =  next_arg(argc, argv, i);
+        strcpy(project.master_url, next_arg(argc, argv, i));
         canonicalize_master_url(project.master_url);
         char* op = next_arg(argc, argv, i);
         if (!strcmp(op, "reset")) {
@@ -410,6 +411,26 @@ int main(int argc, char** argv) {
                 );
             }
         }
+    } else if (!strcmp(cmd, "--get_notices")) {
+        int seqno;
+        if (i == argc) {
+            seqno = 0;
+        } else {
+            seqno = atoi(next_arg(argc, argv, i));
+        }
+        retval = rpc.get_notices(seqno, notices);
+        if (!retval) {
+            unsigned int j;
+            for (j=0; j<notices.notices.size(); j++) {
+                NOTICE& n = *notices.notices[j];
+                strip_whitespace(n.description);
+                printf("%d: (%s) %s\n",
+                    n.seqno,
+                    time_to_string(n.create_time),
+                    n.description.c_str()
+                );
+            }
+        }
     } else if (!strcmp(cmd, "--get_host_info")) {
         HOST_INFO hi;
         retval = rpc.get_host_info(hi);
@@ -450,13 +471,23 @@ int main(int argc, char** argv) {
     } else if (!strcmp(cmd, "--get_project_config")) {
         char* gpc_url = next_arg(argc, argv,i);
         retval = rpc.get_project_config(string(gpc_url));
-    } else if (!strcmp(cmd, "--get_project_config_poll")) {
-        PROJECT_CONFIG pc;
-        retval = rpc.get_project_config_poll(pc);
-        if (retval) {
-            printf("retval: %d\n", retval);
-        } else {
-            pc.print();
+        if (!retval) {
+            while (1) {
+                PROJECT_CONFIG pc;
+                retval = rpc.get_project_config_poll(pc);
+                if (retval) {
+                    printf("poll status: %s\n", boincerror(retval));
+                } else {
+                    if (pc.error_num) {
+                        printf("poll status: %s\n", boincerror(pc.error_num));
+                        if (pc.error_num != ERR_IN_PROGRESS) break;
+                        boinc_sleep(1);
+                    } else {
+                        pc.print();
+                        break;
+                    }
+                }
+            }
         }
     } else if (!strcmp(cmd, "--lookup_account")) {
         ACCOUNT_IN lai;
@@ -522,11 +553,12 @@ int main(int argc, char** argv) {
         if (!retval) {
             retval = cs.network_status;
         }
+        cs.print();
     } else if (!strcmp(cmd, "--set_debts")) {
         vector<PROJECT>projects;
         while (i < argc) {
             PROJECT proj;
-            proj.master_url = string(next_arg(argc, argv, i));
+            strcpy(proj.master_url, next_arg(argc, argv, i));
             int std = atoi(next_arg(argc, argv, i));
             proj.cpu_short_term_debt = std;
             proj.cuda_short_term_debt = std;
@@ -541,7 +573,7 @@ int main(int argc, char** argv) {
     } else if (!strcmp(cmd, "--quit")) {
         retval = rpc.quit();
     } else {
-        fprintf(stderr, "unrecognized command %s\n", cmd);
+        usage();
     }
     if (retval < 0) {
         show_error(retval);
