@@ -28,11 +28,13 @@ using std::string;
 using std::vector;
 
 #include "coproc.h"
+#include "util.h"
 
 #include "acct_mgr.h"
 #include "acct_setup.h"
 #include "app.h"
 #include "client_types.h"
+#include "current_version.h"
 #include "file_xfer.h"
 #include "gui_rpc_server.h"
 #include "gui_http.h"
@@ -44,6 +46,10 @@ using std::vector;
 #include "scheduler_op.h"
 #include "time_stats.h"
 
+#ifdef SIM
+#include "../sched/edf_sim.h"
+#endif
+
 #define WORK_FETCH_DONT_NEED 0
     // project: suspended, deferred, or no new work (can't ask for more work)
     // overall: not work_fetch_ok (from CPU policy)
@@ -60,8 +66,7 @@ using std::vector;
 // encapsulates the global variables of the core client.
 // If you add anything here, initialize it in the constructor
 //
-class CLIENT_STATE {
-public:
+struct CLIENT_STATE {
     vector<PLATFORM> platforms;
     vector<PROJECT*> projects;
     vector<APP*> apps;
@@ -74,13 +79,21 @@ public:
     PERS_FILE_XFER_SET* pers_file_xfers;
     HTTP_OP_SET* http_ops;
     FILE_XFER_SET* file_xfers;
-    ACTIVE_TASK_SET active_tasks;
-    HOST_INFO host_info;
+#ifndef SIM
+    GUI_RPC_CONN_SET gui_rpcs;
+#endif
+    GUI_HTTP gui_http;
+    AUTO_UPDATE auto_update;
+    LOOKUP_WEBSITE_OP lookup_website_op;
+    GET_CURRENT_VERSION_OP get_current_version_op;
+    GET_PROJECT_LIST_OP get_project_list_op;
+    ACCT_MGR_OP acct_mgr_op;
+
+    TIME_STATS time_stats;
     GLOBAL_PREFS global_prefs;
     NET_STATS net_stats;
-    GUI_RPC_CONN_SET gui_rpcs;
-    TIME_STATS time_stats;
-    GUI_HTTP gui_http;
+    ACTIVE_TASK_SET active_tasks;
+    HOST_INFO host_info;
 
     VERSION_INFO core_client_version;
     string statefile_platform_name;
@@ -89,49 +102,55 @@ public:
     MODE gpu_mode;
     MODE network_mode;
     bool started_by_screensaver;
-    bool exit_when_idle;
     bool exit_before_start;
-    bool exit_after_finish;
     bool check_all_logins;
     bool user_active;       // there has been recent mouse/kbd input
     int cmdline_gui_rpc_port;
     bool show_projects;
     bool requested_exit;
-    bool requested_suspend;
-    bool requested_resume;
+        // we should exit now.  Set when
+        // - got a "quit" GUI RPC
+        // - (Unix) got a HUP, INT, QUIT, TERM, or PWR signal
+        // - (Win) got CTRL_LOGOFF, CTRL_C, CTRL_BREAK, etc. event
+        // - (Mac) client was started from screensaver,
+        //   which has since exited
+    bool os_requested_suspend;
+        // we should suspend for OS reasonts (used on Win only).
+        // Set when
+        // - got BATTERY_LOW, SUSPEND, SERVICE_CONTROL_PAUSE
     bool cleanup_completed;
-        /// Determine when it is safe to leave the quit_client() handler
-        /// and to finish cleaning up.
     bool in_abort_sequence;
-        /// stores URL for -detach_project option
+        // Determine when it is safe to leave the quit_client() handler
+        // and to finish cleaning up.
     char detach_project_url[256];
-        /// stores URL for -reset_project option
+        // stores URL for -detach_project option
     char reset_project_url[256];
-        /// stores URL for -update_prefs option
+        // stores URL for -reset_project option
     char update_prefs_url[256];
-        /// venue from project or AMS that gave us general prefs
+        // stores URL for -update_prefs option
     char main_host_venue[256];
+        // venue from project or AMS that gave us general prefs
     char attach_project_url[256];
     char attach_project_auth[256];
-        /// exit when about to upload a file
     bool exit_before_upload;
+        // exit when about to upload a file
 #ifndef _WIN32
     gid_t boinc_project_gid;
 #endif
 
     // backoff-related variables
     //
-        /// fetch project's master URL (and stop doing scheduler RPCs)
-        /// if get this many successive RPC failures (default 10)
     int master_fetch_period;
-        /// cap project->nrpc_failures at this number
+        // fetch project's master URL (and stop doing scheduler RPCs)
+        // if get this many successive RPC failures (default 10)
     int retry_cap;
-        /// after this many master-fetch failures,
-        /// move into a state in which we retry master fetch
-        /// at the frequency below
+        // cap project->nrpc_failures at this number
     int master_fetch_retry_cap;
-        /// see above
+        // after this many master-fetch failures,
+        // move into a state in which we retry master fetch
+        // at the frequency below
     int master_fetch_interval;
+        // see above
 
     int sched_retry_delay_min;
     int sched_retry_delay_max;
@@ -139,89 +158,75 @@ public:
     int pers_retry_delay_max;
     int pers_giveup;
 
-        /// Don't use CPU.  See check_suspend_activities for logic
     bool tasks_suspended;
         // Don't run apps.
     int suspend_reason;
+
     bool network_suspended;
         // Don't use network.
     bool file_xfers_suspended;
         // Don't do file xfers (but allow other network activity).
     int network_suspend_reason;
-        /// true if --daemon is on the commandline
 
-        /// this means we are running as a daemon on unix,
-        /// or as a service on Windows
     bool executing_as_daemon;
-        /// redirect stdout, stderr to log files
+        // true if --daemon is on the commandline
+        // this means we are running as a daemon on unix,
+        // or as a service on Windows
     bool redirect_io;
-        /// a condition has occurred in which we know graphics will
-        /// not be displayable, so GUIs shouldn't offer graphics.
+        // redirect stdout, stderr to log files
     bool disable_graphics;
+        // a condition has occurred in which we know graphics will
+        // not be displayable, so GUIs shouldn't offer graphics.
     bool detach_console;
-        /// this affects auto-update
     bool launched_by_manager;
+        // this affects auto-update
     bool run_by_updater;
     double now;
     double client_start_time;
     double last_wakeup_time;
     bool initialized;
-        /// failed to write state file.
-        /// In this case we continue to run for 1 minute,
-        /// handling GUI RPCs but doing nothing else,
-        /// so that the Manager can tell the user what the problem is
     bool cant_write_state_file;
-        /// accept unsigned app files (use for testing only!!)
-    bool unsigned_apps_ok;
-        /// use hardwired numbers rather than running benchmarks
-    bool skip_cpu_benchmarks;
-private:
+        // failed to write state file.
+        // In this case we continue to run for 1 minute,
+        // handling GUI RPCs but doing nothing else,
+        // so that the Manager can tell the user what the problem is
+
     bool client_state_dirty;
     int old_major_version;
     int old_minor_version;
     int old_release;
-        /// if set, run benchmarks on client startup
     bool run_cpu_benchmarks;
-        /// set if a benchmark fails to start because of a job that doesn't exit
-        /// Persists so that the next start of BOINC runs the benchmarks.
+        // if set, run benchmarks on client startup
     bool cpu_benchmarks_pending;
+        // set if a benchmark fails to start because of a job that doesn't exit
+        // Persists so that the next start of BOINC runs the benchmarks.
 
-        /// if nonzero, exit this many seconds after starting an app
     int exit_after_app_start_secs;
-        /// when the most recent app was started
+        // if nonzero, exit this many seconds after starting an app
     double app_started;
+        // when the most recent app was started
 
 // --------------- acct_mgr.cpp:
-public:
-    ACCT_MGR_OP acct_mgr_op;
     ACCT_MGR_INFO acct_mgr_info;
 
 // --------------- acct_setup.cpp:
-public:
     PROJECT_INIT project_init;
     PROJECT_ATTACH project_attach;
-    LOOKUP_WEBSITE_OP lookup_website_op;
-    GET_CURRENT_VERSION_OP get_current_version_op;
-    GET_PROJECT_LIST_OP get_project_list_op;
     void new_version_check();
     void all_projects_list_check();
     double new_version_check_time;
     double all_projects_list_check_time;
+        // the time we last successfully fetched the project list
     string newer_version;
 
-// --------------- auto_update.cpp:
-public:
-    AUTO_UPDATE auto_update;
-
 // --------------- client_state.cpp:
-public:
     CLIENT_STATE();
     void show_host_info();
     int init();
-        /// Never blocks.
-        /// Returns true if it actually did something,
-        /// in which case it should be called again immediately.
     bool poll_slow_events();
+        // Never blocks.
+        // Returns true if it actually did something,
+        // in which case it should be called again immediately.
     void do_io_or_sleep(double dt);
     bool time_to_exit();
     PROJECT* lookup_project(const char*);
@@ -239,7 +244,7 @@ public:
     void start_abort_sequence();
     bool abort_sequence_done();
     int quit_activities();
-private:
+
     int link_app(PROJECT*, APP*);
     int link_file_info(PROJECT*, FILE_INFO*);
     int link_file_ref(PROJECT*, FILE_REF*);
@@ -253,36 +258,39 @@ private:
     bool update_results();
     int nresults_for_project(PROJECT*);
     void check_clock_reset();
+    void clear_absolute_times();
+    inline void set_now() {
+        double x = dtime();
+        if (x < (now-60)) {
+            clear_absolute_times();
+        }
+        now = x;
+    }
 
 // --------------- cpu_sched.cpp:
-private:
     double total_resource_share();
     double potentially_runnable_resource_share();
     double nearly_runnable_resource_share();
     double fetchable_resource_share();
     double debt_interval_start;
     double total_cpu_time_this_debt_interval;
-    bool work_fetch_no_new_work;
     bool must_enforce_cpu_schedule;
     bool must_schedule_cpus;
     bool must_check_work_fetch;
     void assign_results_to_projects();
     RESULT* largest_debt_project_best_result();
-    RESULT* earliest_deadline_result(bool coproc_only);
     void reset_debt_accounting();
-    bool possibly_schedule_cpus();
-    void schedule_cpus();
-    bool enforce_schedule();
+    bool schedule_cpus();
+    void make_run_list(vector<RESULT*>&);
+    bool enforce_run_list(vector<RESULT*>&);
     void append_unfinished_time_slice(vector<RESULT*>&);
-public:
+
     double runnable_resource_share(int);
-        /// Check if work fetch needed.
     void adjust_debts();
-    std::vector <RESULT*> ordered_scheduled_results;
-        /// if we fail to start a task due to no shared-mem segments,
-        /// wait until at least this time to try running
-        /// another task that needs a shared-mem seg
     double retry_shmem_time;
+        // if we fail to start a task due to no shared-mem segments,
+        // wait until at least this time to try running
+        // another task that needs a shared-mem seg
     inline double work_buf_min() {
         return global_prefs.work_buf_min_days * 86400;
     }
@@ -294,27 +302,25 @@ public:
         if (x < 1) x = 1;
         return x;
     }
-    void request_enforce_schedule(PROJECT*, const char*);
-        /// Check for reschedule CPUs ASAP.
 
-        /// Called when:
-        /// - core client starts (CS::init())
-        /// - an app exits (ATS::check_app_exited())
-        /// - Tasks are killed (ATS::exit_tasks())
-        /// - a result's input files finish downloading (CS::update_results())
-        /// - an app fails to start (CS::schedule_cpus())
-        /// - any project op is done via RPC (suspend/resume)
-        /// - any result op is done via RPC (suspend/resume)
     void request_schedule_cpus(const char*);
+        // Reschedule CPUs ASAP.
+        // Called when:
+        // - core client starts (CS::init())
+        // - an app exits (ATS::check_app_exited())
+        // - Tasks are killed (ATS::exit_tasks())
+        // - a result's input files finish downloading (CS::update_results())
+        // - an app fails to start (CS::schedule_cpus())
+        // - any project op is done via RPC (suspend/resume)
+        // - any result op is done via RPC (suspend/resume)
     void set_ncpus();
 
 // --------------- cs_account.cpp:
-public:
     int add_project(
         const char* master_url, const char* authenticator,
         const char* project_name, bool attached_via_acct_mgr
     );
-private:
+
     int parse_account_files();
     int parse_account_files_venue();
     int parse_preferences_for_user_files();
@@ -322,27 +328,24 @@ private:
         // should be move to a new file, but this will do it for testing
 
 // --------------- cs_apps.cpp:
-public:
     double get_fraction_done(RESULT* result);
     int input_files_available(RESULT*, bool, FILE_INFO** f=0);
     ACTIVE_TASK* lookup_active_task_by_result(RESULT*);
-        /// number of usable cpus
     int ncpus;
         // Act like there are this many CPUs.
         // By default this is the # of physical CPUs,
         // but it can be changed in two ways:
         // - type <ncpus>N</ncpus> in the config file
         // - type the max_ncpus_pct pref
-private:
+
     int latest_version(APP*, char*);
     int app_finished(ACTIVE_TASK&);
     bool start_apps();
     bool handle_finished_apps();
-public:
+
     ACTIVE_TASK* get_task(RESULT*);
 
 // --------------- cs_benchmark.cpp:
-public:
     bool should_run_cpu_benchmarks();
     void start_cpu_benchmarks();
     bool cpu_benchmarks_poll();
@@ -353,33 +356,28 @@ public:
     void print_benchmark_results();
 
 // --------------- cs_cmdline.cpp:
-public:
     void parse_cmdline(int argc, char** argv);
     void parse_env_vars();
     void do_cmdline_actions();
 
 // --------------- cs_files.cpp:
-public:
     void check_file_existence();
     bool start_new_file_xfer(PERS_FILE_XFER&);
-private:
+
     int make_project_dirs();
     bool handle_pers_file_xfers();
 
 // --------------- cs_platforms.cpp:
-public:
     const char* get_primary_platform();
-private:
     void add_platform(const char*);
     void detect_platforms();
     void write_platforms(PROJECT*, MIOFILE&);
     bool is_supported_platform(const char*);
 
 // --------------- cs_prefs.cpp:
-public:
     int project_disk_usage(PROJECT*, double&);
-        /// returns the total disk usage of BOINC on this host
     int total_disk_usage(double&);
+        // returns the total disk usage of BOINC on this host
     double allowed_disk_usage(double boinc_total);
     int allowed_project_disk_usage(double&);
     int suspend_tasks(int reason);
@@ -388,8 +386,6 @@ public:
     int save_global_prefs(char* prefs, char* url, char* sched);
     double available_ram();
     double max_available_ram();
-    const char* suspend_reason_string(int reason);
-private:
     int check_suspend_processing();
     void check_suspend_network();
     void install_global_prefs();
@@ -397,15 +393,14 @@ private:
     void show_global_prefs_source(bool);
 
 // --------------- cs_scheduler.cpp:
-public:
-        /// Called when:
-        /// - core client starts (CS::init())
-        /// - task is completed or fails
-        /// - tasks are killed
-        /// - an RPC completes
-        /// - project suspend/detch/attach/reset GUI RPC
-        /// - result suspend/abort GUI RPC
     void request_work_fetch(const char*);
+        // Called when:
+        // - core client starts (CS::init())
+        // - task is completed or fails
+        // - tasks are killed
+        // - an RPC completes
+        // - project suspend/detch/attach/reset GUI RPC
+        // - result suspend/abort GUI RPC
     int make_scheduler_request(PROJECT*);
     int handle_scheduler_reply(PROJECT*, char* scheduler_url);
     SCHEDULER_OP* scheduler_op;
@@ -413,16 +408,10 @@ public:
     PROJECT* next_project_sched_rpc_pending();
     PROJECT* next_project_trickle_up_pending();
     PROJECT* find_project_with_overdue_results();
-private:
-    bool contacted_sched_server;
-    int overall_work_fetch_urgency;
-
+    bool had_or_requested_work;
     bool scheduler_rpc_poll();
-    double avg_proc_rate();
-    bool should_get_work();
 
 // --------------- cs_statefile.cpp:
-public:
     void set_client_state_dirty(const char*);
     int parse_state_file();
     int write_state(MIOFILE&);
@@ -436,16 +425,13 @@ public:
     void sort_results();
 
 // --------------- cs_trickle.cpp:
-private:
     int read_trickle_files(PROJECT*, FILE*);
     int remove_trickle_files(PROJECT*);
-public:
     int handle_trickle_down(PROJECT*, FILE*);
 
 // --------------- check_state.cpp:
 // stuff related to data-structure integrity checking
 //
-public:
     void check_project_pointer(PROJECT*);
     void check_app_pointer(APP*);
     void check_file_info_pointer(FILE_INFO*);
@@ -473,25 +459,38 @@ public:
     void print_deadline_misses();
 
 // --------------- work_fetch.cpp:
-public:
     int proj_min_results(PROJECT*, double);
     void check_project_timeout();
     double overall_cpu_frac();
+    double overall_gpu_frac();
     double time_until_work_done(PROJECT*, int, double);
     bool compute_work_requests();
     void scale_duration_correction_factors(double);
     void generate_new_host_cpid();
     void compute_nuploading_results();
 
+#ifdef SIM
+    void html_start(bool);
+    void html_rec();
+    void html_end(bool);
+    std::string html_msg;
+    double share_violation();
+    double monotony();
+
+    void do_client_simulation();
+    void handle_completed_results(PROJECT*);
+    void get_workload(vector<IP_RESULT>&);
+    void simulate();
+    bool simulate_rpc(PROJECT*);
+    void print_project_results(FILE*);
+#endif
 };
 
 extern CLIENT_STATE gstate;
 
-extern COPROC_CUDA* coproc_cuda;
-extern COPROC_ATI* coproc_ati;
 extern bool gpus_usable;
 
-/// return a random double in the range [MIN,min(e^n,MAX))
+// return a random double in the range [MIN,min(e^n,MAX))
 
 extern double calculate_exponential_backoff(
     int n, double MIN, double MAX
@@ -499,25 +498,15 @@ extern double calculate_exponential_backoff(
 
 extern void print_suspend_tasks_message(int);
 
+//////// TIME-RELATED CONSTANTS ////////////
+
+//////// CLIENT INTERNAL
+
 #define POLL_INTERVAL   1.0
     // the client will handle I/O (including GUI RPCs)
     // for up to POLL_INTERVAL seconds before calling poll_slow_events()
     // to call the polling functions
 
-#define CPU_PESSIMISM_FACTOR 0.9
-    // assume actual CPU utilization will be this multiple
-    // of what we've actually measured recently
-
-#define WORK_FETCH_PERIOD   60
-    // see if we need to fetch work at least this often
-
-#define CPU_SCHED_ENFORCE_PERIOD    60
-    // enforce CPU schedule at least this often
-
-#define DEBT_ADJUST_PERIOD CPU_SCHED_ENFORCE_PERIOD
-    // debt is adjusted at least this often,
-    // since adjust_debts() is called from enforce_schedule()
-
 #define GARBAGE_COLLECT_PERIOD  10
     // how often to garbage collect
 
@@ -538,6 +527,44 @@ extern void print_suspend_tasks_message(int);
 
 #define GUI_HTTP_POLL_PERIOD    1.0
 
+#define MEMORY_USAGE_PERIOD     10
+
+//////// WORK FETCH
+
+#define WORK_FETCH_PERIOD   60
+    // see if we need to fetch work at least this often
+#define WF_MIN_BACKOFF_INTERVAL    600
+#define WF_MAX_BACKOFF_INTERVAL    86400
+    // if we ask a project for work for a resource and don't get it,
+    // we do exponential backoff.
+    // This constant is an upper bound for this.
+    // E.g., if we need GPU work, we'll end up asking once a day,
+    // so if the project develops a GPU app,
+    // we'll find out about it within a day.
+
+#define WF_DEFER_INTERVAL   300
+    // if a project is uploading, and the last upload started within this interval,
+    // don't fetch work from it.
+    // This allows the work fetch to be merged with the reporting of the
+    // jobs that are currently uploading.
+
+//////// CPU SCHEDULING
+
+#define CPU_SCHED_PERIOD    60
+    // do CPU schedule at least this often
+
+#define DEBT_ADJUST_PERIOD CPU_SCHED_PERIOD
+    // debt is adjusted at least this often,
+    // since adjust_debts() is called from enforce_schedule()
+
+#define MAX_EXIT_TIME   15
+    // if an app takes this long to exit, kill it
+
+#define MAX_STARTUP_TIME    10
+    // if app startup takes longer than this, quit loop
+
+//////// NETWORK
+
 #define CONNECT_ERROR_PERIOD    600.0
 
 #define ALLOW_NETWORK_IF_RECENT_RPC_PERIOD  300
@@ -545,9 +572,19 @@ extern void print_suspend_tasks_message(int);
     // that requires network access (e.g. attach to project)
     // allow it even if setting is "no access"
 
+//////// MISC
+
+#define EXCLUSIVE_APP_WAIT   30
+    // if "exclusive app" feature used,
+    // wait this long after app exits before restarting jobs
+
 #define DAILY_XFER_HISTORY_PERIOD   60
 
 #define MAX_STD   (86400)
     // maximum short-term debt
 
+#define ACCT_MGR_MIN_BACKOFF    600
+#define ACCT_MGR_MAX_BACKOFF    86400
+    // min/max account manager RPC backoff
+
 #endif
