@@ -39,6 +39,11 @@
 
 #include "rr_sim.h"
 #include "work_fetch.h"
+#include "cs_notice.h"
+
+#ifdef SIM
+#include "sim.h"
+#endif
 
 #define MAX_FILE_INFO_LEN   4096
 #define MAX_SIGNATURE_LEN   4096
@@ -54,8 +59,7 @@
 #define FILE_NOT_PRESENT    0
 #define FILE_PRESENT        1
 
-class FILE_INFO {
-public:
+struct FILE_INFO {
     char name[256];
     char md5_cksum[33];
     double max_nbytes;
@@ -68,47 +72,47 @@ public:
     bool upload_when_present;
     bool sticky;            // don't delete unless instructed to do so
     bool report_on_rpc;     // include this in each scheduler request
-        /// if not in use, delete even if sticky is true
-        /// don't report to server even if report_on_rpc is true
     bool marked_for_delete;     // server requested delete;
+        // if not in use, delete even if sticky is true
+        // don't report to server even if report_on_rpc is true
     bool signature_required;    // true iff associated with app version
     bool is_user_file;
     bool is_project_file;
 	bool is_auto_update_file;
-        /// for output files: gzip file when done, and append .gz to its name
     bool gzip_when_done;
-        /// nonzero if in the process of being up/downloaded
+        // for output files: gzip file when done, and append .gz to its name
     class PERS_FILE_XFER* pers_file_xfer;
-        /// for upload files (to authenticate)
-    struct RESULT* result;
-    class PROJECT* project;
+        // nonzero if in the process of being up/downloaded
+    RESULT* result;
+        // for upload files (to authenticate)
+    PROJECT* project;
     int ref_cnt;
     std::vector<std::string> urls;
     int start_url;
     int current_url;
-        /// if the file_info is signed (for uploadable files)
-        /// this is the text that is signed
-        /// Otherwise it is the FILE_INFO's XML descriptor
-        /// (without enclosing <file_info> tags)
     char signed_xml[MAX_FILE_INFO_LEN];
-        /// ... and this is the signature
+        // if the file_info is signed (for uploadable files)
+        // this is the text that is signed
+        // Otherwise it is the FILE_INFO's XML descriptor
+        // (without enclosing <file_info> tags)
     char xml_signature[MAX_SIGNATURE_LEN];
-        /// if the file itself is signed (for executable files)
-        /// this is the signature
+        // ... and this is the signature
     char file_signature[MAX_SIGNATURE_LEN];
-        /// if permanent error occurs during file xfer, it's recorded here
+        // if the file itself is signed (for executable files)
+        // this is the signature
     std::string error_msg;
+        // if permanent error occurs during file xfer, it's recorded here
     CERT_SIGS* cert_sigs;
 
     FILE_INFO();
-    ~FILE_INFO();
+    ~FILE_INFO(){}
     void reset();
     int set_permissions();
     int parse(MIOFILE&, bool from_server);
     int write(MIOFILE&, bool to_server);
     int write_gui(MIOFILE&);
-        /// attempt to delete the underlying file
     int delete_file();
+        // attempt to delete the underlying file
     const char* get_init_url();
     const char* get_next_url();
     const char* get_current_url();
@@ -117,29 +121,29 @@ public:
     int merge_info(FILE_INFO&);
     int verify_file(bool, bool);
     bool verify_file_certs();
-        /// gzip file and add .gz to name
     int gzip();
+        // gzip file and add .gz to name
 };
 
-/// Describes a connection between a file and a workunit, result, or application
+// Describes a connection between a file and a workunit, result, or application
 
-/// In the first two cases,
-/// the app will either use open() or fopen() to access the file
-/// (in which case "open_name" is the name it will use)
-/// or the app will be connected by the given fd (in which case fd is nonzero)
+// In the first two cases,
+// the app will either use open() or fopen() to access the file
+// (in which case "open_name" is the name it will use)
+// or the app will be connected by the given fd (in which case fd is nonzero)
 
 struct FILE_REF {
-        /// physical name
     char file_name[256];
-        /// logical name
+        // physical name
     char open_name[256];
+        // logical name
     bool main_program;
     FILE_INFO* file_info;
-        /// if true, core client will copy the file instead of linking
     bool copy_file;
-		/// for output files: app may not generate file;
-		/// don't treat as error if file is missing.
+        // if true, core client will copy the file instead of linking
 	bool optional;
+		// for output files: app may not generate file;
+		// don't treat as error if file is missing.
     int parse(MIOFILE&);
     int write(MIOFILE&);
 };
@@ -166,7 +170,7 @@ struct FILE_XFER_BACKOFF {
     }
 };
 
-/// statistics at a specific day
+// statistics at a specific day
 
 struct DAILY_STATS {
     double user_total_credit;
@@ -180,58 +184,85 @@ struct DAILY_STATS {
     int parse(FILE*);
 };
 bool operator < (const DAILY_STATS&, const DAILY_STATS&);
-class ACTIVE_TASK;
 
-class PROJECT {
-public:
+// base class for PROJECT and ACCT_MGR_INFO
+//
+struct PROJ_AM {
+    char master_url[256];
+    char project_name[256];
+        // descriptive.  not unique
+    std::vector<RSS_FEED> proj_feeds;
+    inline char *get_project_name() {
+        if (strlen(project_name)) {
+            return project_name;
+        } else {
+            return master_url;
+        }
+    }
+};
+
+struct PROJECT : PROJ_AM {
     // the following items come from the account file
     // They are a function only of the user and the project
     //
-        /// url of site that contains scheduler tags for this project
-    char master_url[256];
-        /// user's authenticator on this project
     char authenticator[256];
-        /// without the enclosing <project_preferences> tags.
-        /// May include <venue> elements
-        /// This field is used only briefly: between handling a
-        /// scheduler RPC reply and writing the account file
+        // user's authenticator on this project
     std::string project_prefs;
-        /// without enclosing <project_specific> tags
-        /// Does not include <venue> elements
+        // without the enclosing <project_preferences> tags.
+        // May include <venue> elements
+        // This field is used only briefly: between handling a
+        // scheduler RPC reply and writing the account file
     std::string project_specific_prefs;
-        /// GUI URLs, with enclosing <gui_urls> tags
+        // without enclosing <project_specific> tags
+        // Does not include <venue> elements
     std::string gui_urls;
-        /// project's resource share relative to other projects.
+        // GUI URLs, with enclosing <gui_urls> tags
     double resource_share;
+        // project's resource share relative to other projects.
+    double resource_share_frac;
+        // fraction of RS of non-suspended, compute-intensive projects
+
+    // the following are from the user's project prefs
+    //
     bool no_cpu_pref;
     bool no_cuda_pref;
     bool no_ati_pref;
-        // the following are from the project itself
+
+    // the following are from the project itself
+    // (or derived from app version list if anonymous platform)
+    //
     bool no_cpu_apps;
     bool no_cuda_apps;
     bool no_ati_apps;
-        // the following set dynamically
+
+    // the following are from the account manager, if any
+    //
+    bool no_cpu_ams;
+    bool no_cuda_ams;
+    bool no_ati_ams;
+
+    // the following set dynamically
+    //
     bool cuda_defer_sched;
         // This project has a CUDA job for which there's insuff. video RAM.
         // Don't fetch more CUDA jobs; they might have same problem
     bool ati_defer_sched;
+        // same, ATI
 
-        /// logically, this belongs in the client state file
-        /// rather than the account file.
-        /// But we need it in the latter in order to parse prefs.
     char host_venue[256];
+        // logically, this belongs in the client state file
+        // rather than the account file.
+        // But we need it in the latter in order to parse prefs.
     bool using_venue_specific_prefs;
 
     // the following items come from client_state.xml
     // They may depend on the host as well as user and project
     // NOTE: if you add anything, add it to copy_state_fields() also!!!
     //
-        /// where to find scheduling servers
     std::vector<std::string> scheduler_urls;
-        /// descriptive.  not unique
-    char project_name[256];
-        /// URL of symbol server (Windows)
+        // where to find scheduling servers
     char symstore[256];
+        // URL of symbol server (Windows)
     char user_name[256];
     char team_name[256];
     char email_hash[MD5_LEN];
@@ -240,6 +271,8 @@ public:
     double user_total_credit;
     double user_expavg_credit;
     double user_create_time;
+    int userid;
+    int teamid;
     int hostid;
     double host_total_credit;
     double host_expavg_credit;
@@ -251,114 +284,130 @@ public:
     // stuff related to scheduler RPCs and master fetch
     //
     int rpc_seqno;
-        /// # of consecutive times we've failed to contact all scheduling servers
     int nrpc_failures;
+        // # of consecutive times we've failed to contact all scheduling servers
     int master_fetch_failures;
-        /// earliest time to contact any server of this project (or zero)
     double min_rpc_time;
+        // earliest time to contact any server of this project (or zero)
     void set_min_rpc_time(double future_time, const char* reason);
-        /// if nonzero, specifies a time when another scheduler RPC
-        /// should be done (as requested by server).
-        /// An RPC could be done sooner than this.
     double next_rpc_time;
-        /// returns true if min_rpc_time > now
+        // if nonzero, specifies a time when another scheduler RPC
+        // should be done (as requested by server).
+        // An RPC could be done sooner than this.
     bool waiting_until_min_rpc_time();
-        /// need to fetch and parse the master URL
+        // returns true if min_rpc_time > now
     bool master_url_fetch_pending;
-        /// we need to do a scheduler RPC, for various possible reasons:
-        /// user request, propagate host CPID, time-based, etc.
-		/// Reasons are enumerated in scheduler_op.h
+        // need to fetch and parse the master URL
     int sched_rpc_pending;
-        /// we need to call request_work_fetch() when a project
-        /// transitions from being backed off to not.
-        /// This (slightly misnamed) keeps track of whether this
-        /// may still need to be done for given project
+        // we need to do a scheduler RPC, for various possible reasons:
+        // user request, propagate host CPID, time-based, etc.
+		// Reasons are enumerated in scheduler_op.h
 	bool possibly_backed_off;
-        /// have trickle up to send
+        // we need to call request_work_fetch() when a project
+        // transitions from being backed off to not.
+        // This (slightly misnamed) keeps track of whether this
+        // may still need to be done for given project
     bool trickle_up_pending;
-        /// when last RPC finished
+        // have trickle up to send
     double last_rpc_time;
+        // when last RPC finished
 
     // Other stuff
 
-        /// app_versions.xml file found in project dir;
-        /// use those apps rather then getting from server
     bool anonymous_platform;
+        // app_versions.xml file found in project dir;
+        // use those apps rather then getting from server
     bool non_cpu_intensive;
+        // All this project's apps are non-CPU-intensive.
+        // Apps can also be individually marked as NCI
     bool verify_files_on_app_start;
+        // Check app version and input files on app startup,
+        // to make sure they haven't been tampered with.
+        // This provides only the illusion of security.
     bool use_symlinks;
 
     // items send in scheduler replies, requesting that
     // various things be sent in the next request
     //
-        /// send the list of permanent files associated with the project
-        /// in the next scheduler reply
     bool send_file_list;
-        /// if nonzero, send time stats log from that point on
+        // send the list of permanent files associated with the project
+        // in the next scheduler reply
     int send_time_stats_log;
-        /// if nonzero, send this project's job log from that point on
+        // if nonzero, send time stats log from that point on
     int send_job_log;
+        // if nonzero, send this project's job log from that point on
+    bool send_full_workload;
 
     bool suspended_via_gui;
-        /// Return work, but don't request more
-        /// Used for a clean exit to a project,
-        /// or if a user wants to pause doing work for the project
     bool dont_request_more_work; 
+        // Return work, but don't request more
+        // Used for a clean exit to a project,
+        // or if a user wants to pause doing work for the project
     bool attached_via_acct_mgr;
-        /// when no results for this project, detach it.
     bool detach_when_done;
-        /// project has ended; advise user to detach
+        // when no results for this project, detach it.
     bool ended;
+        // project has ended; advise user to detach
     char code_sign_key[MAX_KEY_LEN];
     std::vector<FILE_REF> user_files;
-        /// files not specific to apps or work - e.g. icons
     std::vector<FILE_REF> project_files;
+        // files not specific to apps or work - e.g. icons
     int parse_preferences_for_user_files();
     int parse_project_files(MIOFILE&, bool delete_existing_symlinks);
     void write_project_files(MIOFILE&);
     void link_project_files(bool recreate_symlink_files);
     int write_symlink_for_project_file(FILE_INFO*);
-        /// when last project file download finished
     double project_files_downloaded_time;
-        /// called when a project file download finishes.
-        /// If it's the last one, set project_files_downloaded_time to now
+        // when last project file download finished
     void update_project_files_downloaded_time();
+        // called when a project file download finishes.
+        // If it's the last one, set project_files_downloaded_time to now
 
-        /// Multiply by this when estimating the CPU time of a result
-        /// (based on FLOPs estimated and benchmarks).
-        /// This is dynamically updated in a way that maintains an upper bound.
-        /// it goes down slowly but if a new estimate X is larger,
-        /// the factor is set to X.
     double duration_correction_factor;
+        // Multiply by this when estimating the CPU time of a result
+        // (based on FLOPs estimated and benchmarks).
+        // This is dynamically updated in a way that maintains an upper bound.
+        // it goes down slowly but if a new estimate X is larger,
+        // the factor is set to X.
+        //
+        // Deprecated - current server logic handles this,
+        // and this should go to 1.
+        // But we need to keep it around for older projects
     void update_duration_correction_factor(ACTIVE_TASK*);
     
     // fields used by CPU scheduler and work fetch
     // everything from here on applies only to CPU intensive projects
 
-        /// not suspended and not deferred and not no more work
     bool can_request_work();
-        /// has a runnable result using the given resource type
+        // not suspended and not deferred and not no more work
     bool runnable(int rsc_type);
-        /// has a result in downloading state
+        // has a runnable result using the given resource type
     bool downloading();
-        /// runnable or contactable or downloading
+        // has a result in downloading state
     bool potentially_runnable();
-        /// runnable or downloading
+        // runnable or contactable or downloading
     bool nearly_runnable();
-        /// the project has used too much CPU time recently
+        // runnable or downloading
     bool overworked();
-        /// a download is backed off
+        // the project has used too much CPU time recently
     bool some_download_stalled();
+        // a download is backed off
     bool some_result_suspended();
+    double last_upload_start;
+        // the last time an upload was started.
+        // Used for "work fetch deferral" mechanism:
+        // don't request work from a project if an upload started
+        // in last X minutes and is still active
+    bool uploading();
 
     RR_SIM_PROJECT_STATUS rr_sim_status;
         // temps used in CLIENT_STATE::rr_simulation();
 
-        /// the next result to run for this project
     struct RESULT *next_runnable_result;
-        /// number of results in UPLOADING state
-        /// Don't start new results if these exceeds 2*ncpus.
+        // the next result to run for this project
     int nuploading_results;
+        // number of results in UPLOADING state
+        // Don't start new results if these exceeds 2*ncpus.
     bool too_many_uploading_results;
 
     // stuff related to work fetch
@@ -379,13 +428,23 @@ public:
         }
         return cpu_pwf.deadlines_missed;
     }
+#ifndef USE_REC
+    inline double anticipated_debt(int rsc_type) {
+        switch(rsc_type) {
+        case RSC_TYPE_CUDA: return cuda_pwf.anticipated_debt;
+        case RSC_TYPE_ATI: return ati_pwf.anticipated_debt;
+        }
+        return cpu_pwf.anticipated_debt;
+    }
+#endif
+    void get_task_durs(double& not_started_dur, double& in_progress_dur);
 
-        /// # of results being returned in current scheduler op
     int nresults_returned;
-        /// get scheduler URL with random offset r
+        // # of results being returned in current scheduler op
     const char* get_scheduler_url(int index, double r);
-        /// temporary used when scanning projects
+        // get scheduler URL with random offset r
     bool checked;
+        // temporary used when scanning projects
 
     FILE_XFER_BACKOFF download_backoff;
     FILE_XFER_BACKOFF upload_backoff;
@@ -397,7 +456,6 @@ public:
     ~PROJECT(){}
     void init();
     void copy_state_fields(PROJECT&);
-    char *get_project_name();
     int write_account_file();
     int parse_account(FILE*);
     int parse_account_file_venue();
@@ -411,12 +469,57 @@ public:
     int parse_statistics(FILE*);
     int write_statistics(MIOFILE&, bool gui_rpc=false);
     int write_statistics_file();
+
+    void suspend();
+    void resume();
+    void abort_not_started();
+        // abort unstarted jobs
+
+    // clear AMS-related fields
+    inline void detach_ams() {
+        attached_via_acct_mgr = false;
+        ams_resource_share = -1;
+        no_cpu_ams = false;
+        no_cuda_ams = false;
+        no_ati_ams = false;
+    }
+
+#ifdef SIM
+    RANDOM_PROCESS available;
+    int index;
+    int result_index;
+    double idle_time;
+    double idle_time_sumsq;
+    bool idle;
+    int max_infeasible_count;
+    // for DCF variants:
+    int completed_task_count;
+    double completions_ratio_mean;
+    double completions_ratio_s;
+    double completions_ratio_stdev;
+    double completions_required_stdevs;
+    PROJECT_RESULTS project_results;
+    void print_results(FILE*, SIM_RESULTS&);
+    void backoff();
+    void update_dcf_stats(RESULT*);
+#endif
 };
 
 struct APP {
     char name[256];
     char user_friendly_name[256];
+    bool non_cpu_intensive;
     PROJECT* project;
+#ifdef SIM
+    double latency_bound;
+    double fpops_est;
+    NORMAL_DIST fpops;
+    NORMAL_DIST checkpoint_period;
+    double working_set;
+    double weight;
+    bool has_version;
+    APP() {memset(this, 0, sizeof(APP));}
+#endif
 
     int parse(MIOFILE&);
     int write(MIOFILE&);
@@ -434,8 +537,8 @@ struct APP_VERSION {
     double natis;
     double gpu_ram;
     double flops;
-        /// additional cmdline args
     char cmdline[256];
+        // additional cmdline args
 
     APP* app;
     PROJECT* project;
@@ -445,8 +548,12 @@ struct APP_VERSION {
     char graphics_exec_file[256];
     double max_working_set_size;
         // max working set of tasks using this app version.
-        // temp var used in schedule_cpus()
-    //double temp_dcf;
+        // unstarted jobs using this app version are assumed
+        // to use this much RAM,
+        // so that we don't run a long sequence of jobs,
+        // each of which turns out not to fit in available RAM
+
+    int index;  // temp var for make_scheduler_request()
 
     APP_VERSION(){}
     ~APP_VERSION(){}
@@ -474,9 +581,9 @@ struct APP_VERSION {
 struct WORKUNIT {
     char name[256];
     char app_name[256];
-        /// Deprecated, but need to keep around to let people revert
-        /// to versions before multi-platform support
     int version_num;
+        // Deprecated, but need to keep around to let people revert
+        // to versions before multi-platform support
     std::string command_line;
     std::vector<FILE_REF> input_files;
     PROJECT* project;
@@ -506,16 +613,19 @@ struct RESULT {
     char platform[256];
     APP_VERSION* avp;
     std::vector<FILE_REF> output_files;
-        /// we're ready to report this result to the server;
-        /// either computation is done and all the files have been uploaded
-        /// or there was an error
     bool ready_to_report;
-        /// time when ready_to_report was set
+        // we're ready to report this result to the server;
+        // either computation is done and all the files have been uploaded
+        // or there was an error
     double completed_time;
-        /// we've received the ack for this result from the server
+        // time when ready_to_report was set
     bool got_server_ack;
+        // we've received the ack for this result from the server
     double final_cpu_time;
     double final_elapsed_time;
+#ifdef SIM
+    double peak_flop_count;
+#endif
 
     // the following are nonzero if reported by app
     double fpops_per_cpu_sec;
@@ -523,34 +633,34 @@ struct RESULT {
     double intops_per_cpu_sec;
     double intops_cumulative;
 
-        /// state of this result: see lib/result_state.h
     int _state;
+        // state of this result: see lib/result_state.h
     inline int state() { return _state; }
     void set_state(int, const char*);
-        /// return value from the application
     int exit_status;
-        /// the concatenation of:
-        ///
-        /// - if report_result_error() is called for this result:
-        ///   <message>x</message>
-        ///   <exit_status>x</exit_status>
-        ///   <signal>x</signal>
-        ///   - if called in FILES_DOWNLOADED state:
-        ///     <couldnt_start>x</couldnt_start>
-        ///   - if called in NEW state:
-        ///     <download_error>x</download_error> for each failed download
-        ///   - if called in COMPUTE_DONE state:
-        ///     <upload_error>x</upload_error> for each failed upload
-        ///
-        /// - <stderr_txt>X</stderr_txt>, where X is the app's stderr output
+        // return value from the application
     std::string stderr_out;
+        // the concatenation of:
+        //
+        // - if report_result_error() is called for this result:
+        //   <message>x</message>
+        //   <exit_status>x</exit_status>
+        //   <signal>x</signal>
+        //   - if called in FILES_DOWNLOADED state:
+        //     <couldnt_start>x</couldnt_start>
+        //   - if called in NEW state:
+        //     <download_error>x</download_error> for each failed download
+        //   - if called in COMPUTE_DONE state:
+        //     <upload_error>x</upload_error> for each failed upload
+        //
+        // - <stderr_txt>X</stderr_txt>, where X is the app's stderr output
     bool suspended_via_gui;
     bool coproc_missing;
         // a coproc needed by this job is missing
         // (e.g. because user removed their GPU board).
+    bool report_immediately;
 
     APP* app;
-        /// this may be NULL after result is finished
     WORKUNIT* wup;
     PROJECT* project;
 
@@ -566,20 +676,19 @@ struct RESULT {
     void clear_uploaded_flags();
     FILE_REF* lookup_file(FILE_INFO*);
     FILE_INFO* lookup_file_logical(const char*);
-        /// abort the result if it hasn't started computing yet
-
-        /// Called only for results with no active task
-        /// (otherwise you need to abort the active task)
     void abort_inactive(int);
+        // abort the result if it hasn't started computing yet
+        // Called only for results with no active task
+        // (otherwise you need to abort the active task)
     void append_log_record();
 
     // stuff related to CPU scheduling
 
-    double estimated_duration(bool for_work_fetch);
+    double estimated_duration();
     double estimated_duration_uncorrected();
-    double estimated_time_remaining(bool for_work_fetch);
+    double estimated_time_remaining();
     inline double estimated_flops_remaining() {
-        return estimated_time_remaining(false)*avp->flops;
+        return estimated_time_remaining()*avp->flops;
     }
 
     inline bool computing_done() {
@@ -588,16 +697,16 @@ struct RESULT {
         return false;
     }
     bool not_started();
-        /// downloaded, not finished, not suspended, project not suspended
     bool runnable();
-        /// downloading or downloaded,
-        /// not finished, suspended, project not suspended
+        // downloaded, not finished, not suspended, project not suspended
     bool nearly_runnable();
-        /// downloading, not downloaded, not suspended, project not suspended
+        // downloading or downloaded,
+        // not finished, suspended, project not suspended
     bool downloading();
-        /// some input or app file is downloading, and backed off
-        /// i.e. it may be a long time before we can run this result
+        // downloading, not downloaded, not suspended, project not suspended
     bool some_download_stalled();
+        // some input or app file is downloading, and backed off
+        // i.e. it may be a long time before we can run this result
     inline bool uses_cuda() {
         return (avp->ncudas > 0);
     }
@@ -609,25 +718,33 @@ struct RESULT {
         if (avp->natis > 0) return true;
         return false;
     }
+    inline int resource_type() {
+        if (uses_cuda()) return RSC_TYPE_CUDA;
+        if (uses_ati()) return RSC_TYPE_ATI;
+        return RSC_TYPE_CPU;
+    }
+    inline bool non_cpu_intensive() {
+        return project->non_cpu_intensive || app->non_cpu_intensive;
+    }
 
     // temporaries used in CLIENT_STATE::rr_simulation():
     double rrsim_flops_left;
     double rrsim_finish_delay;
     double rrsim_flops;
 
-        /// used to keep cpu scheduler from scheduling a result twice
-        /// transient; used only within schedule_cpus()
     bool already_selected;
-        /// report deadline - prefs.work_buf_min - time slice
+        // used to keep cpu scheduler from scheduling a result twice
+        // transient; used only within schedule_cpus()
     double computation_deadline();
+        // report deadline - prefs.work_buf_min - time slice
     bool rr_sim_misses_deadline;
 
     // temporaries used in enforce_schedule():
     bool unfinished_time_slice;
     int seqno;
 
-        /// temporary used to tell GUI that this result is deadline-scheduled
     bool edf_scheduled;
+        // temporary used to tell GUI that this result is deadline-scheduled
 
     int coproc_indices[MAX_COPROCS_PER_JOB];
         // keep track of coprocessor reservations
@@ -638,14 +755,12 @@ struct RESULT {
         // (wait for free GPU RAM)
 };
 
-/// represents an always/auto/never value, possibly temporarily overridden
+// represents an always/auto/never value, possibly temporarily overridden
 
-class MODE {
-private:
+struct MODE {
     int perm_mode;
     int temp_mode;
     double temp_timeout;
-public:
     MODE();
     void set(int mode, double duration);
     int get_perm();
@@ -653,7 +768,7 @@ public:
 	double delay();
 };
 
-/// a platform supported by the client.
+// a platform supported by the client.
 
 class PLATFORM {
 public:
