@@ -26,15 +26,3 @@
 
 #include "version.h"         // version numbers from autoconf
 #endif
-
-#if !defined(_WIN32) || defined(__CYGWIN32__)
-// Please don't create a header of headers for UNIX systems.
-// This is partially causes the mess with looking for symbols named "*open64"
-// Not to mention the assumption that every source file wants these 
-// headers is wrong.
-// #include "config.h"
-// #include <iostream>
-// #include <vector>
-// #include <string>
-// #include <cstring>
-#endif
