@@ -26,9 +26,6 @@ typedef BOOL (WINAPI *LPFN_ISWOW64PROCESS) (HANDLE, PBOOL);
 LPFN_ISWOW64PROCESS fnIsWow64Process;
 #else
 #include "config.h"
-#endif
-
-#ifndef _WIN32
 #include <cstdio>
 #include <cstdlib>
 #include <signal.h>
@@ -118,11 +115,15 @@ void CLIENT_STATE::detect_platforms() {
         HOSTTYPE = "i686-apple-darwin";
     }
 
-    // Supported on both Mac Intel architectures
+    // Supported on both Mac Intel architectures    
     add_platform("i686-apple-darwin");
-#endif
-    // Supported on all 3 Mac architectures
+#else
+    // We no longer request PowerPC applications on Intel Macs 
+    // because all projects supporting Macs should have Intel 
+    // applications by now, and PowerPC emulation ("Rosetta") 
+    // is not always supported in newer versions of OS X.
     add_platform("powerpc-apple-darwin");
+#endif
 
 #elif defined(__linux__) && ( defined(__i386__) || defined(__x86_64__) )
     // Let's try to support both 32 and 64 bit applications in one client
@@ -137,86 +138,88 @@ void CLIENT_STATE::detect_platforms() {
     do {
         if (boinc_file_exists(uname[eno])) break; 
     } while (uname[++eno] != 0);
+
     // run it and check the kernel machine architecture.
     if ( uname[eno] != 0 ) {
         strlcpy(cmdline,uname[eno],256);
         strlcat(cmdline," -m",256);
         if ((f=popen(cmdline,"r"))) {
             while (!std::feof(f)) {
-	        fgets(cmdline,256,f);
-		if (strstr(cmdline,"x86_64")) support64=1;
-	    }
-	    pclose(f);
-	}
+                fgets(cmdline,256,f);
+                if (strstr(cmdline,"x86_64")) support64=1;
+            }
+            pclose(f);
+        }
+
         if (!support64) {
-	    // we're running on a 32 bit kernel, so we will assume
-	    // we are i686-pc-linux-gnu only.
-	    support32=1;
-	} else {
-	    // we're running on a 64 bit kernel.
-	    // Now comes the hard part.  How to tell whether we can run
-	    // 32-bit binaries.
+	        // we're running on a 32 bit kernel, so we will assume
+	        // we are i686-pc-linux-gnu only.
+	        support32=1;
+	    } else {
+	        // we're running on a 64 bit kernel.
+	        // Now comes the hard part.  How to tell whether we can run
+	        // 32-bit binaries.
 #if defined(__i386__) && !defined(__x86_64__)
             // If we're a 32 bit binary, then we obviously can.
-	    support32=1;
+	        support32=1;
 #else
             // If we're a 64 bit binary, the check is a bit harder.
-	    // We'll use the file command to check installation of
-	    // 32 bit libraries or executables.
-	    const char *file[]={"/usr/bin/file","/bin/file",0};
-	    const char *libdir[]={"/lib","/lib32","/lib/32","/usr/lib","/usr/lib32","/usr/lib/32"};
-	    const int nlibdirs=sizeof(libdir)/sizeof(char *);
-	    
-
+	        // We'll use the file command to check installation of
+	        // 32 bit libraries or executables.
+	        const char *file[]={"/usr/bin/file","/bin/file",0};
+	        const char *libdir[]={"/lib","/lib32","/lib/32","/usr/lib","/usr/lib32","/usr/lib/32"};
+	        const int nlibdirs=sizeof(libdir)/sizeof(char *);
+    	    
             // find 'file'
             eno=0;
             do {
                 if (boinc_file_exists(file[eno])) break; 
             } while (file[++eno] != 0);
-	    // now try to find a 32-bit C library.
-	    if (file[eno] != 0) {
-	        int i;
-	        for (i=0;i<nlibdirs;i++) {
-	            struct dirent *entry;
-	            DIR *a=opendir(libdir[i]);
-		    // if dir doesn't exist, do to the next one
-		    if ( a == 0 ) continue;
-		    // dir exists. read each entry until you find a 32bit lib
-		    while ((support32 == 0) && ((entry=readdir(a)) != 0)) {
-		        strlcpy(cmdline,file[eno],256);
-		        strlcat(cmdline," -L ",256);
-		        strlcat(cmdline,libdir[i],256);
-		        strlcat(cmdline,"/",256);
-		        strlcat(cmdline,entry->d_name,256);
-		        if ((f=popen(cmdline,"r"))) {
+    	    
+            // now try to find a 32-bit C library.
+	        if (file[eno] != 0) {
+	            int i;
+	            for (i=0; i < nlibdirs; i++) {
+	                struct dirent *entry;
+	                DIR *a = opendir(libdir[i]);
+                    // if dir doesn't exist, do to the next one
+                    if (a == 0) continue;
+                    // dir exists. read each entry until you find a 32bit lib
+                    while ((support32 == 0) && ((entry=readdir(a)) != 0)) {
+                        strlcpy(cmdline, file[eno], 256);
+                        strlcat(cmdline, " -L ", 256);
+                        strlcat(cmdline, libdir[i], 256);
+                        strlcat(cmdline, "/", 256);
+                        strlcat(cmdline, entry->d_name, 256);
+                        f = popen(cmdline, "r");
+                        if (f) {
                             while (!std::feof(f)) {
-	                        fgets(cmdline,256,f);
-			        // If the library is 32-bit ELF, then we're
-			        // golden.
-		                if (strstr(cmdline,"ELF") && 
-			            strstr(cmdline,"32-bit")) support32=1;
-	                    }
-			    pclose(f);
-		        }  
-		    }
-		    closedir(a);
-		    if (support32) break;
-                 }
-	     }
+	                            fgets(cmdline,256,f);
+                                // If the library is 32-bit ELF, then we're
+                                // golden.
+		                        if (strstr(cmdline, "ELF") && strstr(cmdline, "32-bit")) support32=1;
+	                        }
+			                pclose(f);
+                        }  
+                    }
+                    closedir(a);
+                    if (support32) break;
+                }
+	        }
 #endif
-	 }
+        }
     }
 
     if (support64) {
-       add_platform("x86_64-pc-linux-gnu");
+        add_platform("x86_64-pc-linux-gnu");
     }
     if (support32) {
-       add_platform("i686-pc-linux-gnu");
+        add_platform("i686-pc-linux-gnu");
     }
 
     if (!(support64 || support32)) {
         // Something went wrong. Assume HOSTTYPE and HOSTTYPEALT
-	// are correct
+        // are correct
         add_platform(HOSTTYPE);
 #ifdef HOSTTYPEALT
         add_platform(HOSTTYPEALT);
@@ -290,6 +293,7 @@ void CLIENT_STATE::detect_platforms() {
 #endif  // !defined(UNKNOWN_SOLARIS_PROCESSOR)
 
 #else                   
+
     // Any other platform, fall back to the previous method
     add_platform(HOSTTYPE);
 #ifdef HOSTTYPEALT
