@@ -15,25 +15,42 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
-#include "cpp.h"
+#if defined(_WIN32) && !defined(__STDWX_H__) && !defined(_BOINC_WIN_) && !defined(_AFX_STDAFX_H_)
 #include "boinc_win.h"
+#endif
+
+#ifdef _MSC_VER
+#define snprintf _snprintf
+#endif
 
 #ifndef __CYGWIN__
 #include <intrin.h>
 #endif
 
-#include "client_types.h"
+#include "error_numbers.h"
 #include "filesys.h"
 #include "str_util.h"
 #include "str_replace.h"
+#include "util.h"
+
 #include "client_msgs.h"
+#include "client_types.h"
 #include "hostinfo_network.h"
 #include "hostinfo.h"
-
-HINSTANCE g_hClientLibraryDll;
+#include "idlemon.h"
 
 
-// Newer system metrics values
+// Newer system metrics values than what is currently defined in
+//   Visual Studio 2005
+#ifndef SM_TABLETPC
+#define SM_TABLETPC 86
+#endif
+#ifndef SM_MEDIACENTER
+#define SM_MEDIACENTER 87
+#endif
+#ifndef SM_STARTER
+#define SM_STARTER 88
+#endif
 #ifndef SM_SERVERR2
 #define SM_SERVERR2 89   
 #endif
@@ -179,6 +196,84 @@ HINSTANCE g_hClientLibraryDll;
 #ifndef PRODUCT_STORAGE_ENTERPRISE_SERVER_CORE
 #define PRODUCT_STORAGE_ENTERPRISE_SERVER_CORE      0x0000002E
 #endif
+#ifndef PRODUCT_PROFESSIONAL
+#define PRODUCT_PROFESSIONAL                        0x00000030
+#endif
+#ifndef PRODUCT_PROFESSIONAL_N
+#define PRODUCT_PROFESSIONAL_N                      0x00000031
+#endif
+#ifndef PRODUCT_SB_SOLUTION_SERVER
+#define PRODUCT_SB_SOLUTION_SERVER                  0x00000032
+#endif
+#ifndef PRODUCT_SERVER_FOR_SB_SOLUTIONS
+#define PRODUCT_SERVER_FOR_SB_SOLUTIONS             0x00000033
+#endif
+#ifndef PRODUCT_STANDARD_SERVER_SOLUTIONS
+#define PRODUCT_STANDARD_SERVER_SOLUTIONS           0x00000034
+#endif
+#ifndef PRODUCT_STANDARD_SERVER_SOLUTIONS_CORE
+#define PRODUCT_STANDARD_SERVER_SOLUTIONS_CORE      0x00000035
+#endif
+#ifndef PRODUCT_SB_SOLUTION_SERVER_EM
+#define PRODUCT_SB_SOLUTION_SERVER_EM               0x00000036
+#endif
+#ifndef PRODUCT_SERVER_FOR_SB_SOLUTIONS_EM
+#define PRODUCT_SERVER_FOR_SB_SOLUTIONS_EM          0x00000037
+#endif
+#ifndef PRODUCT_SOLUTION_EMBEDDEDSERVER
+#define PRODUCT_SOLUTION_EMBEDDEDSERVER             0x00000038
+#endif
+#ifndef PRODUCT_SOLUTION_EMBEDDEDSERVER_CORE
+#define PRODUCT_SOLUTION_EMBEDDEDSERVER_CORE        0x00000039
+#endif
+#ifndef PRODUCT_SMALLBUSINESS_SERVER_PREMIUM_CORE
+#define PRODUCT_SMALLBUSINESS_SERVER_PREMIUM_CORE   0x0000003F
+#endif
+#ifndef PRODUCT_ESSENTIALBUSINESS_SERVER_MGMT
+#define PRODUCT_ESSENTIALBUSINESS_SERVER_MGMT       0x0000003B
+#endif
+#ifndef PRODUCT_ESSENTIALBUSINESS_SERVER_ADDL
+#define PRODUCT_ESSENTIALBUSINESS_SERVER_ADDL       0x0000003C
+#endif
+#ifndef PRODUCT_ESSENTIALBUSINESS_SERVER_MGMTSVC
+#define PRODUCT_ESSENTIALBUSINESS_SERVER_MGMTSVC    0x0000003D
+#endif
+#ifndef PRODUCT_ESSENTIALBUSINESS_SERVER_ADDLSVC
+#define PRODUCT_ESSENTIALBUSINESS_SERVER_ADDLSVC    0x0000003E
+#endif
+#ifndef PRODUCT_CLUSTER_SERVER_V
+#define PRODUCT_CLUSTER_SERVER_V                    0x00000040
+#endif
+#ifndef PRODUCT_EMBEDDED
+#define PRODUCT_EMBEDDED                            0x00000041
+#endif
+#ifndef PRODUCT_STARTER_E
+#define PRODUCT_STARTER_E                           0x00000042
+#endif
+#ifndef PRODUCT_HOME_BASIC_E
+#define PRODUCT_HOME_BASIC_E                        0x00000043
+#endif
+#ifndef PRODUCT_HOME_PREMIUM_E
+#define PRODUCT_HOME_PREMIUM_E                      0x00000044
+#endif
+#ifndef PRODUCT_PROFESSIONAL_E
+#define PRODUCT_PROFESSIONAL_E                      0x00000045
+#endif
+#ifndef PRODUCT_ENTERPRISE_E
+#define PRODUCT_ENTERPRISE_E                        0x00000046
+#endif
+#ifndef PRODUCT_ULTIMATE_E
+#define PRODUCT_ULTIMATE_E                          0x00000047
+#endif
+
+
+// Newer suite types than what is currently defined in
+//   Visual Studio 2005
+#ifndef VER_SUITE_WH_SERVER
+#define VER_SUITE_WH_SERVER                         0x00008000
+#endif
+
+
 
 
 // Returns the number of seconds difference from UTC
@@ -260,7 +355,7 @@ int get_os_information(
     pGPI = (PGPI) GetProcAddress(GetModuleHandle(_T("kernel32.dll")), "GetProductInfo");
 
 
-    // Windows s a Microsoft OS
+    // Windows is a Microsoft OS
     strcpy(os_name, "Microsoft ");
 
     switch (osvi.dwPlatformId)
@@ -301,26 +396,38 @@ int get_os_information(
             }
 
             if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1 )
+            {
                 strcat(os_name, "Windows XP");
+            }
 
             if ( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 0 )
+            {
                 strcat(os_name, "Windows 2000");
+            }
 
             if ( osvi.dwMajorVersion <= 4 )
+            {
                 strcat(os_name, "Windows NT");
+            }
 
             break;
 
         case VER_PLATFORM_WIN32_WINDOWS:
 
             if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 0)
+            {
                 strcat(os_name, "Windows 95");
+            }
 
             if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 10)
+            {
                 strcat( os_name, "Windows 98");
+            }
 
             if (osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 90)
+            {
                 strcat( os_name, "Windows Millennium");
+            }
 
             break;
 
@@ -351,6 +458,9 @@ int get_os_information(
                             case PRODUCT_ULTIMATE:
                                strcat(szSKU, "Ultimate ");
                                break;
+                            case PRODUCT_PROFESSIONAL:
+                               strcat(szSKU, "Professional ");
+                               break;
                             case PRODUCT_HOME_PREMIUM:
                                strcat(szSKU, "Home Premium ");
                                break;
@@ -365,6 +475,9 @@ int get_os_information(
                                break;
                             case PRODUCT_STARTER:
                                strcat(szSKU, "Starter ");
+                               break;
+							case PRODUCT_PROFESSIONAL_N:
+                               strcat(szSKU, "Professional N ");
                                break;
 							case PRODUCT_HOME_PREMIUM_N:
                                strcat(szSKU, "Home Premium N ");
@@ -380,6 +493,21 @@ int get_os_information(
                                break;
                             case PRODUCT_BUSINESS_N:
                                strcat(szSKU, "Business N ");
+                               break;
+							case PRODUCT_PROFESSIONAL_E:
+                               strcat(szSKU, "Professional E ");
+                               break;
+							case PRODUCT_HOME_PREMIUM_E:
+                               strcat(szSKU, "Home Premium E ");
+                               break;
+							case PRODUCT_HOME_BASIC_E:
+                               strcat(szSKU, "Home Basic E ");
+                               break;
+                            case PRODUCT_ULTIMATE_E:
+                               strcat(szSKU, "Ultimate E ");
+                               break;
+                            case PRODUCT_ENTERPRISE_E:
+                               strcat(szSKU, "Enterprise E ");
                                break;
                         }
                     } else if( (osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2) ) {
@@ -391,6 +519,12 @@ int get_os_information(
                     } else if( (osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 1) ) {
                         if( osvi.wSuiteMask & VER_SUITE_PERSONAL ) {
                             strcat(szSKU, "Home ");
+                        } else if( GetSystemMetrics(SM_TABLETPC) ) {
+                            strcat(szSKU, "Tablet PC ");
+                        } else if( GetSystemMetrics(SM_MEDIACENTER) ) {
+                            strcat(szSKU, "Media Center ");
+                        } else if( GetSystemMetrics(SM_STARTER) ) {
+                            strcat(szSKU, "Starter ");
                         } else {
                             strcat(szSKU, "Professional ");
                         }
@@ -448,6 +582,21 @@ int get_os_information(
                             case PRODUCT_HOME_PREMIUM_SERVER:
                                strcat( szSKU, "Home Premium Server ");
                                break;
+                            case PRODUCT_STORAGE_EXPRESS_SERVER:
+                               strcat( szSKU, "Storage Server Express ");
+                               break;
+                            case PRODUCT_STORAGE_STANDARD_SERVER:
+                               strcat( szSKU, "Storage Server Standard ");
+                               break;
+                            case PRODUCT_STORAGE_WORKGROUP_SERVER:
+                               strcat( szSKU, "Storage Server Workgroup ");
+                               break;
+                            case PRODUCT_STORAGE_ENTERPRISE_SERVER:
+                               strcat( szSKU, "Storage Server Enterprise ");
+                               break;
+                            case PRODUCT_SERVER_FOR_SMALLBUSINESS:
+                               strcat( szSKU, "Server For Small Business ");
+                               break;
                         }
 
                     } else if( (osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2) ) {
@@ -456,8 +605,10 @@ int get_os_information(
                             strcat( szSKU, "Datacenter Server " );
                         } else if( osvi.wSuiteMask & VER_SUITE_ENTERPRISE ) {
                             strcat( szSKU, "Enterprise Server " );
-                        } else if ( osvi.wSuiteMask == VER_SUITE_BLADE ) {
+                        } else if ( osvi.wSuiteMask & VER_SUITE_BLADE ) {
                             strcat( szSKU, "Web Server " );
+                        } else if ( osvi.wSuiteMask & VER_SUITE_WH_SERVER ) {
+                            strcat( szSKU, "Home Server " );
                         } else {
                             strcat( szSKU, "Standard Server " );
                         }
@@ -598,7 +749,7 @@ int get_os_information(
 // Handle the cpuid instruction on supported compilers
 // NOTE: This only handles structured exceptions with Microsoft compilers.
 // 
-int get_cpuid(int info_type, int& a, int& b, int& c, int& d) {
+int get_cpuid(unsigned int info_type, unsigned int& a, unsigned int& b, unsigned int& c, unsigned int& d) {
 
 #ifdef _MSC_VER
 
@@ -636,9 +787,10 @@ int get_cpuid(int info_type, int& a, int& b, int& c, int& d) {
 // see: http://www.intel.com/Assets/PDF/appnote/241618.pdf
 // see: http://www.amd.com/us-en/assets/content_type/white_papers_and_tech_docs/25481.pdf
 int get_processor_vendor(char* name, int name_size) {
-    int eax, ebx, ecx, edx;
+    unsigned int eax = 0, ebx = 0, ecx = 0, edx = 0;
 
-    if (name_size < 13) return 1;
+    if (!name) return ERR_INVALID_PARAM;
+    if (name_size < 13) return ERR_WRONG_SIZE;
 
     memset(name, 0, sizeof(name_size));
 
@@ -657,7 +809,10 @@ int get_processor_vendor(char* name, int name_size) {
 // see: http://www.intel.com/Assets/PDF/appnote/241618.pdf
 // see: http://www.amd.com/us-en/assets/content_type/white_papers_and_tech_docs/25481.pdf
 int get_processor_version(int& family, int& model, int& stepping) {
-    int eax, ebx, ecx, edx;
+    unsigned int eax = 0, ebx = 0, ecx = 0, edx = 0;
+
+    get_cpuid(0x00000000, eax, ebx, ecx, edx);
+    if (eax < 0x00000001) return ERR_NOT_IMPLEMENTED;
 
     if (!get_cpuid(0x00000001, eax, ebx, ecx, edx)) {
         family = (((eax >> 8) + (eax >> 20)) & 0xff);
@@ -673,12 +828,17 @@ int get_processor_version(int& family, int& model, int& stepping) {
 // see: http://www.intel.com/Assets/PDF/appnote/241618.pdf
 // see: http://www.amd.com/us-en/assets/content_type/white_papers_and_tech_docs/25481.pdf
 int get_processor_name(char* name, int name_size) {
-    int eax, ebx, ecx, edx, i;
+    unsigned int eax = 0, ebx = 0, ecx = 0, edx = 0;
+    size_t i = 0;
 
-    if (name_size < 48) return 1;
+    if (!name) return ERR_INVALID_PARAM;
+    if (name_size < 48) return ERR_WRONG_SIZE;
 
     memset(name, 0, sizeof(name_size));
 
+    get_cpuid(0x80000000, eax, ebx, ecx, edx);
+    if (!(eax >= 0x80000004)) return ERR_NOT_IMPLEMENTED;
+
     if (!get_cpuid(0x80000002, eax, ebx, ecx, edx)) {
         *((int*)(name + 0))  = eax;
         *((int*)(name + 4))  = ebx;
@@ -716,7 +876,10 @@ int get_processor_name(char* name, int name_size) {
 // see: http://www.intel.com/Assets/PDF/appnote/241618.pdf
 // see: http://www.amd.com/us-en/assets/content_type/white_papers_and_tech_docs/25481.pdf
 int get_processor_cache(int& cache) {
-    int eax, ebx, ecx, edx;
+    unsigned int eax = 0, ebx = 0, ecx = 0, edx = 0;
+
+    get_cpuid(0x80000000, eax, ebx, ecx, edx);
+    if (!(eax >= 0x80000006)) return ERR_NOT_IMPLEMENTED;
 
     if (!get_cpuid(0x80000006, eax, ebx, ecx, edx)) {
         cache = ((ecx >> 16) & 0xffff) * 1024;
@@ -730,22 +893,20 @@ int get_processor_cache(int& cache) {
 // see: http://msdn.microsoft.com/en-us/library/hskdteyh.aspx
 // see: http://www.intel.com/Assets/PDF/appnote/241618.pdf
 // see: http://www.amd.com/us-en/assets/content_type/white_papers_and_tech_docs/25481.pdf
-#define FEATURE_TEST(test, feature_name) \
-    if (test) strncat(features, feature_name, features_size - strlen(features))
+#define FEATURE_TEST(feature_set_supported, test, feature_name) \
+    if (feature_set_supported && test) strncat(features, feature_name, features_size - strlen(features))
 
 int get_processor_features(char* vendor, char* features, int features_size) {
-    int std_eax, std_ebx, std_ecx, std_edx;
-    int ext_eax, ext_ebx, ext_ecx, ext_edx;
-    int std_supported = 0, ext_supported = 0, intel_supported = 0, amd_supported = 0;
+    unsigned int std_eax = 0, std_ebx = 0, std_ecx = 0, std_edx = 0;
+    unsigned int ext_eax = 0, ext_ebx = 0, ext_ecx = 0, ext_edx = 0;
+    unsigned int std_supported = 0, ext_supported = 0, intel_supported = 0, amd_supported = 0;
+
+    if (!vendor) return ERR_INVALID_PARAM;
+    if (!features) return ERR_INVALID_PARAM;
+    if (features_size < 250) return ERR_WRONG_SIZE;
 
     memset(features, 0, sizeof(features_size));
 
-    if (!get_cpuid(0x00000001, std_eax, std_ebx, std_ecx, std_edx)) {
-        std_supported = 1;
-    }
-    if (!get_cpuid(0x80000001, ext_eax, ext_ebx, ext_ecx, ext_edx)) {
-        ext_supported = 1;
-    }
     if (strcmp(vendor, "GenuineIntel") == 0) {
         intel_supported = 1;
     }
@@ -753,76 +914,88 @@ int get_processor_features(char* vendor, char* features, int features_size) {
         amd_supported = 1;
     }
 
-    FEATURE_TEST((std_edx & (1 << 0)), "fpu ");
-    FEATURE_TEST((std_edx & (1 << 1)), "vme ");
-    FEATURE_TEST((std_edx & (1 << 2)), "de ");
-    FEATURE_TEST((std_edx & (1 << 3)), "pse ");
-    FEATURE_TEST((std_edx & (1 << 4)), "tsc ");
-    FEATURE_TEST((std_edx & (1 << 5)), "msr ");
-    FEATURE_TEST((std_edx & (1 << 6)), "pae ");
-    FEATURE_TEST((std_edx & (1 << 7)), "mce ");
-    FEATURE_TEST((std_edx & (1 << 8)), "cx8 ");
-    FEATURE_TEST((std_edx & (1 << 9)), "apic ");
-    FEATURE_TEST((std_edx & (1 << 11)), "sep ");
-    FEATURE_TEST((std_edx & (1 << 12)), "mtrr ");
-    FEATURE_TEST((std_edx & (1 << 13)), "pge ");
-    FEATURE_TEST((std_edx & (1 << 14)), "mca ");
-    FEATURE_TEST((std_edx & (1 << 15)), "cmov ");
-    FEATURE_TEST((std_edx & (1 << 16)), "pat ");
-    FEATURE_TEST((std_edx & (1 << 17)), "pse36 ");
-    FEATURE_TEST((std_edx & (1 << 18)), "psn ");
-    FEATURE_TEST((std_edx & (1 << 19)), "clflush ");
-    FEATURE_TEST((std_edx & (1 << 21)), "dts ");
-    FEATURE_TEST((std_edx & (1 << 22)), "acpi ");
-    FEATURE_TEST((std_edx & (1 << 23)), "mmx ");
-    FEATURE_TEST((std_edx & (1 << 24)), "fxsr ");
-    FEATURE_TEST((std_edx & (1 << 25)), "sse ");
-    FEATURE_TEST((std_edx & (1 << 26)), "sse2 ");
-    FEATURE_TEST((std_edx & (1 << 27)), "ss ");
-    FEATURE_TEST((std_edx & (1 << 28)), "htt ");
-    FEATURE_TEST((std_edx & (1 << 29)), "tm ");
+    get_cpuid(0x00000000, std_eax, std_ebx, std_ecx, std_edx);
+    if (std_eax >= 0x00000001) {
+        std_supported = 1;
+        get_cpuid(0x00000001, std_eax, std_ebx, std_ecx, std_edx);
+    }
 
-    FEATURE_TEST((std_ecx & (1 << 0)), "pni ");
-    FEATURE_TEST((std_ecx & (1 << 9)), "ssse3 ");
-    FEATURE_TEST((std_ecx & (1 << 13)), "cx16 ");
-    FEATURE_TEST((std_ecx & (1 << 19)), "sse4_1 ");
-    FEATURE_TEST((std_ecx & (1 << 20)), "sse4_2 ");
+    get_cpuid(0x80000000, ext_eax, ext_ebx, ext_ecx, ext_edx);
+    if (ext_eax >= 0x80000001) {
+        ext_supported = 1;
+        get_cpuid(0x80000001, ext_eax, ext_ebx, ext_ecx, ext_edx);
+    }
 
-    FEATURE_TEST((ext_edx & (1 << 11)), "syscall ");
-    FEATURE_TEST((ext_edx & (1 << 20)), "nx ");
-    FEATURE_TEST((ext_edx & (1 << 29)), "lm ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 0)), "fpu ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 1)), "vme ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 2)), "de ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 3)), "pse ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 4)), "tsc ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 5)), "msr ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 6)), "pae ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 7)), "mce ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 8)), "cx8 ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 9)), "apic ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 11)), "sep ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 12)), "mtrr ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 13)), "pge ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 14)), "mca ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 15)), "cmov ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 16)), "pat ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 17)), "pse36 ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 18)), "psn ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 19)), "clflush ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 21)), "dts ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 22)), "acpi ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 23)), "mmx ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 24)), "fxsr ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 25)), "sse ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 26)), "sse2 ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 27)), "ss ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 28)), "htt ");
+    FEATURE_TEST(std_supported, (std_edx & (1 << 29)), "tm ");
+
+    FEATURE_TEST(std_supported, (std_ecx & (1 << 0)), "pni ");
+    FEATURE_TEST(std_supported, (std_ecx & (1 << 9)), "ssse3 ");
+    FEATURE_TEST(std_supported, (std_ecx & (1 << 13)), "cx16 ");
+    FEATURE_TEST(std_supported, (std_ecx & (1 << 19)), "sse4_1 ");
+    FEATURE_TEST(std_supported, (std_ecx & (1 << 20)), "sse4_2 ");
+
+    FEATURE_TEST(ext_supported, (ext_edx & (1 << 11)), "syscall ");
+    FEATURE_TEST(ext_supported, (ext_edx & (1 << 20)), "nx ");
+    FEATURE_TEST(ext_supported, (ext_edx & (1 << 29)), "lm ");
 
     if (intel_supported) {
         // Intel only features
-        FEATURE_TEST((std_ecx & (1 << 5)), "vmx ");
-        FEATURE_TEST((std_ecx & (1 << 6)), "smx ");
-        FEATURE_TEST((std_ecx & (1 << 8)), "tm2 ");
-		FEATURE_TEST((std_ecx & (1 << 12)), "fma ");
-        FEATURE_TEST((std_ecx & (1 << 18)), "dca ");
-		FEATURE_TEST((std_ecx & (1 << 22)), "movebe ");
-        FEATURE_TEST((std_ecx & (1 << 23)), "popcnt ");
-		FEATURE_TEST((std_ecx & (1 << 25)), "aes ");
+        FEATURE_TEST(std_supported, (std_ecx & (1 << 5)), "vmx ");
+        FEATURE_TEST(std_supported, (std_ecx & (1 << 6)), "smx ");
+        FEATURE_TEST(std_supported, (std_ecx & (1 << 8)), "tm2 ");
+		FEATURE_TEST(std_supported, (std_ecx & (1 << 12)), "fma ");
+        FEATURE_TEST(std_supported, (std_ecx & (1 << 18)), "dca ");
+		FEATURE_TEST(std_supported, (std_ecx & (1 << 22)), "movebe ");
+        FEATURE_TEST(std_supported, (std_ecx & (1 << 23)), "popcnt ");
+		FEATURE_TEST(std_supported, (std_ecx & (1 << 25)), "aes ");
 
-        FEATURE_TEST((std_edx & (1 << 31)), "pbe ");
+        FEATURE_TEST(std_supported, (std_edx & (1 << 31)), "pbe ");
     }
 
     if (amd_supported) {
         // AMD only features
-		FEATURE_TEST((ext_ecx & (1 << 2)), "svm ");
-        FEATURE_TEST((ext_ecx & (1 << 6)), "sse4a ");
-        FEATURE_TEST((ext_ecx & (1 << 9)), "osvw "); 
- 		FEATURE_TEST((ext_ecx & (1 << 10)), "ibs ");
-		FEATURE_TEST((ext_ecx & (1 << 11)), "xop ");
-		FEATURE_TEST((ext_ecx & (1 << 12)), "skinit ");
-		FEATURE_TEST((ext_ecx & (1 << 13)), "wdt ");
-        FEATURE_TEST((ext_ecx & (1 << 15)), "lwp ");
-		FEATURE_TEST((ext_ecx & (1 << 16)), "fma4 ");
-        FEATURE_TEST((ext_ecx & (1 << 18)), "cvt16 ");
+		FEATURE_TEST(ext_supported, (ext_ecx & (1 << 2)), "svm ");
+        FEATURE_TEST(ext_supported, (ext_ecx & (1 << 6)), "sse4a ");
+        FEATURE_TEST(ext_supported, (ext_ecx & (1 << 9)), "osvw "); 
+ 		FEATURE_TEST(ext_supported, (ext_ecx & (1 << 10)), "ibs ");
+		FEATURE_TEST(ext_supported, (ext_ecx & (1 << 11)), "xop ");
+		FEATURE_TEST(ext_supported, (ext_ecx & (1 << 12)), "skinit ");
+		FEATURE_TEST(ext_supported, (ext_ecx & (1 << 13)), "wdt ");
+        FEATURE_TEST(ext_supported, (ext_ecx & (1 << 15)), "lwp ");
+		FEATURE_TEST(ext_supported, (ext_ecx & (1 << 16)), "fma4 ");
+        FEATURE_TEST(ext_supported, (ext_ecx & (1 << 18)), "cvt16 ");
 
-		FEATURE_TEST((ext_edx & (1 << 26)), "page1gb ");
-		FEATURE_TEST((ext_edx & (1 << 27)), "rdtscp ");
-        FEATURE_TEST((ext_edx & (1 << 30)), "3dnowext ");
-        FEATURE_TEST((ext_edx & (1 << 31)), "3dnow ");
+		FEATURE_TEST(ext_supported, (ext_edx & (1 << 26)), "page1gb ");
+		FEATURE_TEST(ext_supported, (ext_edx & (1 << 27)), "rdtscp ");
+        FEATURE_TEST(ext_supported, (ext_edx & (1 << 30)), "3dnowext ");
+        FEATURE_TEST(ext_supported, (ext_edx & (1 << 31)), "3dnow ");
     }
 
     strip_whitespace(features);
@@ -833,12 +1006,25 @@ int get_processor_features(char* vendor, char* features, int features_size) {
 
 // Returns the CPU count
 //
+typedef DWORD (WINAPI *GAPC)(WORD);
+#ifndef ALL_PROCESSOR_GROUPS
+#define ALL_PROCESSOR_GROUPS 0xffff
+#endif
 int get_processor_count(int& processor_count) {
-    SYSTEM_INFO SystemInfo;
-    memset( &SystemInfo, NULL, sizeof( SystemInfo ) );
-    ::GetSystemInfo( &SystemInfo );
+    GAPC gapc = (GAPC) GetProcAddress(
+        GetModuleHandle(_T("kernel32.dll")),
+        "GetActiveProcessorCount"
+    );
 
-    processor_count = SystemInfo.dwNumberOfProcessors;
+    if (gapc) {
+        processor_count = gapc(ALL_PROCESSOR_GROUPS);
+    } else {
+        SYSTEM_INFO SystemInfo;
+        memset( &SystemInfo, NULL, sizeof( SystemInfo ) );
+        ::GetSystemInfo( &SystemInfo );
+
+        processor_count = SystemInfo.dwNumberOfProcessors;
+    }
     return 0;
 }
 
@@ -882,6 +1068,102 @@ int get_processor_info(
 }
 
 
+// detect the network usage totals for the host.
+//
+int get_network_usage_totals(unsigned int& total_received, unsigned int& total_sent) {
+    int i;
+    int iRetVal = 0;
+    DWORD dwSize = 0;
+    MIB_IFTABLE* pIfTable;
+    MIB_IFROW* pIfRow;
+
+    // Allocate memory for our pointers.
+    pIfTable = (MIB_IFTABLE*)malloc(sizeof(MIB_IFTABLE));
+    if (pIfTable == NULL) {
+        return ERR_MALLOC;
+    }
+
+    // Make an initial call to GetIfTable to get the
+    // necessary size into dwSize
+    dwSize = sizeof(MIB_IFTABLE);
+    if (GetIfTable(pIfTable, &dwSize, FALSE) == ERROR_INSUFFICIENT_BUFFER) {
+        free(pIfTable);
+        pIfTable = (MIB_IFTABLE*)malloc(dwSize);
+        if (pIfTable == NULL) {
+            return ERR_MALLOC;
+        }
+    }
+
+    // Make a second call to GetIfTable to get the actual
+    // data we want.
+    iRetVal = (int)GetIfTable(pIfTable, &dwSize, FALSE);
+    if (iRetVal == NO_ERROR) {
+        for (i = 0; i < (int)pIfTable->dwNumEntries; i++) {
+            pIfRow = (MIB_IFROW *) & pIfTable->table[i];
+            if (IF_TYPE_SOFTWARE_LOOPBACK != pIfRow->dwType) {
+                total_received += pIfRow->dwInOctets;
+                total_sent += pIfRow->dwOutOctets;
+            }
+        }
+    }
+
+    if (pIfTable != NULL) {
+        free(pIfTable);
+        pIfTable = NULL;
+    }
+
+    return iRetVal;
+}
+
+
+// see if Virtualbox is installed
+//
+int HOST_INFO::get_virtualbox_version() {
+    HKEY hKey;
+    char szInstallDir[256];
+    char szVersion[256];
+    DWORD dwInstallDir = sizeof(szInstallDir);
+    DWORD dwVersion = sizeof(szVersion);
+    LONG lRet;
+
+    strcpy(virtualbox_version, "");
+
+    lRet = RegOpenKeyEx( HKEY_LOCAL_MACHINE,
+        "SOFTWARE\\Oracle\\VirtualBox",
+        0, KEY_QUERY_VALUE, &hKey
+    );
+    if (lRet == ERROR_SUCCESS) {
+        lRet = RegQueryValueEx(hKey, "InstallDir", NULL, NULL,
+            (LPBYTE) szInstallDir, &dwInstallDir
+        );
+        if((lRet != ERROR_SUCCESS) || (dwInstallDir > sizeof(szInstallDir))) {
+            return 1;
+        }
+
+        lRet = RegQueryValueEx(
+            hKey, "VersionExt", NULL, NULL, (LPBYTE) szVersion, &dwVersion
+        );
+        if((lRet != ERROR_SUCCESS) || (dwVersion > sizeof(szVersion))) {
+            lRet = RegQueryValueEx(
+                hKey, "Version", NULL, NULL, (LPBYTE) szVersion, &dwVersion
+            );
+            if((lRet != ERROR_SUCCESS) || (dwVersion > sizeof(szVersion))) {
+                return 1;
+            }
+        }
+
+        strncat(szInstallDir, "\\virtualbox.exe", sizeof(szInstallDir) - strlen(szInstallDir));
+
+        if (boinc_file_exists(szInstallDir)) {
+            safe_strcpy(virtualbox_version, szVersion);
+        }
+    }
+
+    RegCloseKey( hKey );
+    return 0;
+}
+
+
 // Gets host information; called on startup and before each sched RPC
 //
 int HOST_INFO::get_host_info() {
@@ -891,6 +1173,7 @@ int HOST_INFO::get_host_info() {
     get_os_information(
         os_name, sizeof(os_name), os_version, sizeof(os_version)
     );
+    get_virtualbox_version();
     get_processor_info(
         p_vendor, sizeof(p_vendor),
         p_model, sizeof(p_model),
@@ -924,14 +1207,8 @@ bool HOST_INFO::host_is_running_on_batteries() {
 }
 
 bool HOST_INFO::users_idle(bool /*check_all_logins*/, double idle_time_to_run) {
-    typedef DWORD (CALLBACK* GetFn)();
-    static GetFn fn = (GetFn)GetProcAddress(g_hClientLibraryDll, "BOINCGetIdleTickCount");
-
-    if (g_hClientLibraryDll && fn) {
-        double seconds_idle = fn() / 1000;
-        double seconds_time_to_run = 60 * idle_time_to_run;
-        return seconds_idle > seconds_time_to_run;
-    }
-
+    double seconds_idle = get_idle_tick_count() / 1000;
+    double seconds_time_to_run = 60 * idle_time_to_run;
+    return seconds_idle > seconds_time_to_run;
 }
 
