@@ -19,21 +19,22 @@
 
 #ifdef _WIN32
 #include "boinc_win.h"
+#ifdef _MSC_VER
+#define chdir _chdir
+#endif
 #else
 #include "config.h"
-#endif
-
-#ifndef _WIN32
 #include <cstdio>
 #include <cstring>
 #include <unistd.h>
 #endif
 
-#include "error_numbers.h"
 #include "common_defs.h"
+#include "diagnostics.h"
+#include "error_numbers.h"
+#include "filesys.h"
 #include "parse.h"
 #include "str_util.h"
-#include "filesys.h"
 
 #include "file_names.h"
 #include "client_state.h"
@@ -46,6 +47,10 @@ LOG_FLAGS log_flags;
 CONFIG config;
 
 LOG_FLAGS::LOG_FLAGS() {
+    init();
+}
+
+void LOG_FLAGS::init() {
     memset(this, 0, sizeof(LOG_FLAGS));
     // on by default (others are off by default)
     //
@@ -62,8 +67,11 @@ int LOG_FLAGS::parse(XML_PARSER& xp) {
 
     while (!xp.get(tag, sizeof(tag), is_tag)) {
         if (!is_tag) {
-            msg_printf(NULL, MSG_USER_ERROR,
-               "Unexpected text %s in %s", tag, CONFIG_FILE
+            msg_printf_notice(NULL, false,
+                "http://boinc.berkeley.edu/manager_links.php?target=notice&controlid=log_flags",
+                "%s: %s",
+                _("Unexpected text in cc_config.xml"),
+                tag
             );
             continue;
         }
@@ -84,7 +92,8 @@ int LOG_FLAGS::parse(XML_PARSER& xp) {
         if (xp.parse_bool(tag, "debt_debug", debt_debug)) continue;
         if (xp.parse_bool(tag, "std_debug", std_debug)) continue;
         if (xp.parse_bool(tag, "file_xfer_debug", file_xfer_debug)) continue;
-        if (xp.parse_bool(tag, "guirpc_debug", guirpc_debug)) continue;
+        if (xp.parse_bool(tag, "gui_rpc_debug", gui_rpc_debug)) continue;
+        if (xp.parse_bool(tag, "heartbeat_debug", heartbeat_debug)) continue;
         if (xp.parse_bool(tag, "http_debug", http_debug)) continue;
         if (xp.parse_bool(tag, "http_xfer_debug", http_xfer_debug)) continue;
         if (xp.parse_bool(tag, "mem_usage_debug", mem_usage_debug)) continue;
@@ -101,9 +110,13 @@ int LOG_FLAGS::parse(XML_PARSER& xp) {
         if (xp.parse_bool(tag, "time_debug", time_debug)) continue;
         if (xp.parse_bool(tag, "unparsed_xml", unparsed_xml)) continue;
         if (xp.parse_bool(tag, "work_fetch_debug", work_fetch_debug)) continue;
+        if (xp.parse_bool(tag, "notice_debug", notice_debug)) continue;
 
-        msg_printf(NULL, MSG_USER_ERROR, "Unrecognized tag in %s: <%s>\n",
-            CONFIG_FILE, tag
+        msg_printf_notice(NULL, false,
+            "http://boinc.berkeley.edu/manager_links.php?target=notice&controlid=log_flags",
+            "%s: <%s>",
+            _("Unrecognized tag in cc_config.xml"),
+            tag
         );
         xp.skip_unexpected(tag, true, "LOG_FLAGS::parse");
     }
@@ -143,7 +156,8 @@ void LOG_FLAGS::show() {
     show_flag(buf, dcf_debug, "dcf_debug");
     show_flag(buf, debt_debug, "debt_debug");
     show_flag(buf, file_xfer_debug, "file_xfer_debug");
-    show_flag(buf, guirpc_debug, "guirpc_debug");
+    show_flag(buf, gui_rpc_debug, "gui_rpc_debug");
+    show_flag(buf, heartbeat_debug, "heartbeat_debug");
     show_flag(buf, http_debug, "http_debug");
     show_flag(buf, http_xfer_debug, "http_xfer_debug");
     show_flag(buf, mem_usage_debug, "mem_usage_debug");
@@ -161,6 +175,7 @@ void LOG_FLAGS::show() {
     show_flag(buf, time_debug, "time_debug");
     show_flag(buf, unparsed_xml, "unparsed_xml");
     show_flag(buf, work_fetch_debug, "work_fetch_debug");
+    show_flag(buf, notice_debug, "notice_debug");
 
     if (strlen(buf)) {
         msg_printf(NULL, MSG_INFO, "log flags: %s", buf);
@@ -178,11 +193,14 @@ static void show_gpu_ignore(vector<int>& devs, const char* name) {
 void CONFIG::show() {
     unsigned int i;
     if (ncpus>0) {
-        msg_printf(NULL, MSG_INFO, "Config: use at most %d CPUs", config.ncpus);
+        msg_printf(NULL, MSG_INFO, "Config: simulate %d CPUs", config.ncpus);
     }
     if (no_gpus) {
         msg_printf(NULL, MSG_INFO, "Config: don't use coprocessors");
     }
+    if (no_info_fetch) {
+        msg_printf(NULL, MSG_INFO, "Config: don't fetch project list or client version info");
+    }
     if (no_priority_change) {
         msg_printf(NULL, MSG_INFO, "Config: run apps at regular priority");
     }
@@ -195,6 +213,9 @@ void CONFIG::show() {
     if (zero_debts) {
         msg_printf(NULL, MSG_INFO, "Config: zero long-term debts on startup");
     }
+    if (fetch_minimal_work) {
+        msg_printf(NULL, MSG_INFO, "Config: fetch minimal work");
+    }
     show_gpu_ignore(ignore_cuda_dev, "NVIDIA");
     show_gpu_ignore(ignore_ati_dev, "ATI");
     for (i=0; i<exclusive_apps.size(); i++) {
@@ -250,27 +271,36 @@ void CONFIG::clear() {
     dont_contact_ref_site = false;
     exclusive_apps.clear();
     exclusive_gpu_apps.clear();
+    exit_after_finish = false;
+    exit_when_idle = false;
+    fetch_minimal_work = false;
     force_auth = "default";
     http_1_0 = false;
+    http_transfer_timeout = 300;
+    http_transfer_timeout_bps = 10;
     ignore_cuda_dev.clear();
     ignore_ati_dev.clear();
     max_file_xfers = MAX_FILE_XFERS;
     max_file_xfers_per_project = MAX_FILE_XFERS_PER_PROJECT;
     max_stderr_file_size = 0;
     max_stdout_file_size = 0;
+    max_tasks_reported = 0;
     ncpus = -1;
     network_test_url = "http://www.google.com/";
     no_alt_platform = false;
     no_gpus = false;
+    no_info_fetch = false;
     no_priority_change = false;
     os_random_only = false;
     report_results_immediately = false;
     run_apps_manually = false;
     save_stats_days = 30;
     simple_gui_only = false;
+    skip_cpu_benchmarks = false;
     start_delay = 0;
     stderr_head = false;
     suppress_net_info = false;
+    unsigned_apps_ok = false;
     use_all_gpus = false;
     use_certs = false;
     use_certs_only = false;
@@ -279,7 +309,7 @@ void CONFIG::clear() {
 
 int CONFIG::parse_options(XML_PARSER& xp) {
     char tag[1024], path[256];
-    bool is_tag, btemp;
+    bool is_tag;
     string s;
     int n;
 
@@ -296,8 +326,11 @@ int CONFIG::parse_options(XML_PARSER& xp) {
 
     while (!xp.get(tag, sizeof(tag), is_tag)) {
         if (!is_tag) {
-            msg_printf(NULL, MSG_USER_ERROR,
-               "Unexpected text %s in %s", tag, CONFIG_FILE
+            msg_printf_notice(NULL, false,
+                "http://boinc.berkeley.edu/manager_links.php?target=notice&controlid=config",
+                "%s: %s",
+                _("Unexpected text in cc_config.xml"),
+                tag
             );
             continue;
         }
@@ -330,18 +363,32 @@ int CONFIG::parse_options(XML_PARSER& xp) {
         if (xp.parse_bool(tag, "dont_check_file_sizes", dont_check_file_sizes)) continue;
         if (xp.parse_bool(tag, "dont_contact_ref_site", dont_contact_ref_site)) continue;
         if (xp.parse_string(tag, "exclusive_app", s)) {
-            exclusive_apps.push_back(s);
+            if (!strstr(s.c_str(), "boinc")) {
+                exclusive_apps.push_back(s);
+            }
             continue;
         }
         if (xp.parse_string(tag, "exclusive_gpu_app", s)) {
-            exclusive_gpu_apps.push_back(s);
+            if (!strstr(s.c_str(), "boinc")) {
+                exclusive_gpu_apps.push_back(s);
+            }
             continue;
         }
+        if (xp.parse_bool(tag, "exit_after_finish", exit_after_finish)) continue;
+        if (xp.parse_bool(tag, "exit_when_idle", exit_when_idle)) {
+            if (exit_when_idle) {
+                report_results_immediately = true;
+            }
+            continue;
+        }
+        if (xp.parse_bool(tag, "fetch_minimal_work", fetch_minimal_work)) continue;
         if (xp.parse_string(tag, "force_auth", force_auth)) {
             downcase_string(force_auth);
             continue;
         }
         if (xp.parse_bool(tag, "http_1_0", http_1_0)) continue;
+        if (xp.parse_int(tag, "http_transfer_timeout", http_transfer_timeout)) continue;
+        if (xp.parse_int(tag, "http_transfer_timeout_bps", http_transfer_timeout_bps)) continue;
         if (xp.parse_int(tag, "ignore_cuda_dev", n)) {
             ignore_cuda_dev.push_back(n);
             continue;
@@ -354,6 +401,7 @@ int CONFIG::parse_options(XML_PARSER& xp) {
         if (xp.parse_int(tag, "max_file_xfers_per_project", max_file_xfers_per_project)) continue;
         if (xp.parse_int(tag, "max_stderr_file_size", max_stderr_file_size)) continue;
         if (xp.parse_int(tag, "max_stdout_file_size", max_stdout_file_size)) continue;
+        if (xp.parse_int(tag, "max_tasks_reported", max_tasks_reported)) continue;
         if (xp.parse_int(tag, "ncpus", ncpus)) continue;
         if (xp.parse_string(tag, "network_test_url", network_test_url)) {
             downcase_string(network_test_url);
@@ -361,6 +409,7 @@ int CONFIG::parse_options(XML_PARSER& xp) {
         }
         if (xp.parse_bool(tag, "no_alt_platform", no_alt_platform)) continue;
         if (xp.parse_bool(tag, "no_gpus", no_gpus)) continue;
+        if (xp.parse_bool(tag, "no_info_fetch", no_info_fetch)) continue;
         if (xp.parse_bool(tag, "no_priority_change", no_priority_change)) continue;
         if (xp.parse_bool(tag, "os_random_only", os_random_only)) continue;
 #ifndef SIM
@@ -374,28 +423,21 @@ int CONFIG::parse_options(XML_PARSER& xp) {
         if (xp.parse_bool(tag, "run_apps_manually", run_apps_manually)) continue;
         if (xp.parse_int(tag, "save_stats_days", save_stats_days)) continue;
         if (xp.parse_bool(tag, "simple_gui_only", simple_gui_only)) continue;
+        if (xp.parse_bool(tag, "skip_cpu_benchmarks", skip_cpu_benchmarks)) continue;
         if (xp.parse_double(tag, "start_delay", start_delay)) continue;
         if (xp.parse_bool(tag, "stderr_head", stderr_head)) continue;
         if (xp.parse_bool(tag, "suppress_net_info", suppress_net_info)) continue;
+        if (xp.parse_bool(tag, "unsigned_apps_ok", unsigned_apps_ok)) continue;
         if (xp.parse_bool(tag, "use_all_gpus", use_all_gpus)) continue;
         if (xp.parse_bool(tag, "use_certs", use_certs)) continue;
         if (xp.parse_bool(tag, "use_certs_only", use_certs_only)) continue;
         if (xp.parse_bool(tag, "zero_debts", zero_debts)) continue;
-        if (xp.parse_bool(tag, "skip_cpu_benchmarks", btemp)) {
-            gstate.skip_cpu_benchmarks = btemp;
-            continue;
-        }
-        if (xp.parse_bool(tag, "unsigned_apps_ok", btemp)) {
-            gstate.unsigned_apps_ok = btemp;
-            continue;
-        }
-        if (xp.parse_bool(tag, "exit_after_finish", btemp)) {
-            gstate.exit_after_finish = btemp;
-            continue;
-        }
 
-        msg_printf(NULL, MSG_USER_ERROR, "Unrecognized tag in %s: <%s>\n",
-            CONFIG_FILE, tag
+        msg_printf_notice(NULL, false,
+            "http://boinc.berkeley.edu/manager_links.php?target=notice&controlid=config",
+            "%s: <%s>",
+            _("Unrecognized tag in cc_config.xml"),
+            tag
         );
         xp.skip_unexpected(tag, true, "CONFIG::parse_options");
     }
@@ -410,13 +452,20 @@ int CONFIG::parse(FILE* f) {
 
     mf.init_file(f);
     if (!xp.parse_start("cc_config")) {
-        msg_printf(NULL, MSG_USER_ERROR, "Missing start tag in %s", CONFIG_FILE);
+        msg_printf_notice(NULL, false,
+            "http://boinc.berkeley.edu/manager_links.php?target=notice&controlid=config",
+            "%s",
+            _("Missing start tag in cc_config.xml")
+        );
         return ERR_XML_PARSE;
     }
     while (!xp.get(tag, sizeof(tag), is_tag)) {
         if (!is_tag) {
-            msg_printf(NULL, MSG_USER_ERROR,
-               "Unexpected text %s in %s", tag, CONFIG_FILE
+            msg_printf_notice(NULL, false,
+                "http://boinc.berkeley.edu/manager_links.php?target=notice&controlid=config",
+                "%s: %s",
+                _("Unexpected text in cc_config.xml"),
+                tag
             );
             continue;
         }
@@ -429,12 +478,21 @@ int CONFIG::parse(FILE* f) {
             parse_options(xp);
             continue;
         }
-        msg_printf(NULL, MSG_USER_ERROR, "Unparsed tag in %s: <%s>\n",
-            CONFIG_FILE, tag
+        if (!strcmp(tag, "options/")) continue;
+        if (!strcmp(tag, "log_flags/")) continue;
+        msg_printf_notice(NULL, false,
+            "http://boinc.berkeley.edu/manager_links.php?target=notice&controlid=config",
+            "%s: <%s>",
+            _("Unrecognized tag in cc_config.xml"),
+            tag
         );
         xp.skip_unexpected(tag, true, "CONFIG.parse");
     }
-    msg_printf(NULL, MSG_USER_ERROR, "Missing end tag in %s", CONFIG_FILE);
+    msg_printf_notice(NULL, false,
+        "http://boinc.berkeley.edu/manager_links.php?target=notice&controlid=config",
+        "%s",
+        _("Missing end tag in cc_config.xml")
+    );
     return ERR_XML_PARSE;
 }
 
@@ -444,11 +502,18 @@ int read_config_file(bool init) {
     if (!init) {
         msg_printf(NULL, MSG_INFO, "Re-reading cc_config.xml");
         config.clear();
+        log_flags.init();
     }
     f = boinc_fopen(CONFIG_FILE, "r");
     if (!f) return ERR_FOPEN;
-    config.parse(f);
+    int retval = config.parse(f);
     fclose(f);
+    if (retval) return retval;
+#ifndef SIM
+    diagnostics_set_max_file_sizes(
+        config.max_stdout_file_size, config.max_stderr_file_size
+    );
+#endif
     return 0;
 }
 
