@@ -31,22 +31,24 @@ using std::deque;
 
 #include "miofile.h"
 
+#define NEED_NETWORK_MSG _("BOINC can't access Internet - check network connection or proxy configuration.")
+
 class FILE_XFER_SET;
 class HTTP_OP_SET;
 
-/// there's one of these each for upload and download
+// there's one of these each for upload and download
 
 struct NET_INFO {
-        /// estimate of max transfer rate; computed as an average of
-        /// the rates of recent file transfers, weighted by file size.
-        /// This ignores concurrency of transfers.
     double max_rate;
-        /// recent average transfer rate
+        // estimate of max transfer rate; computed as an average of
+        // the rates of recent file transfers, weighted by file size.
+        // This ignores concurrency of transfers.
     double avg_rate;
-        /// when avg_rate was last updated
+        // recent average transfer rate
     double avg_time;
-        /// updates the above vars
+        // when avg_rate was last updated
     void update(double nbytes, double dt);
+        // updates the above vars
 
 };
 
@@ -63,29 +65,29 @@ public:
 
 class NET_STATUS {
 public:
-		/// contact the reference site as soon as GUI_HTTP is idle
-		/// polled from NET_STATS::poll(), for want of a better place
 	bool need_to_contact_reference_site;
+		// contact the reference site as soon as GUI_HTTP is idle
+		// polled from NET_STATS::poll(), for want of a better place
 	void contact_reference_site();
     bool show_ref_message;
-        /// client wants to do network comm and no physical connection exists.
-
-        /// Initially false; set whenever a Curl operation
-        /// returns CURLE_COULDNT_RESOLVE_HOST,
-        /// and a subsequent request to a highly-available site
-        /// also returns CURLE_COULDNT_RESOLVE_HOST.
-        /// cleared whenever we transfer data,
-        /// or an operation returns some other value
     bool need_physical_connection;
-        /// we have a network connection, but it's likely to go away soon,
-        /// so do as much network comm as possible
-        /// (e.g. report completed results)
+        // client wants to do network comm and no physical connection exists.
+        // Initially false; set whenever a Curl operation
+        // returns CURLE_COULDNT_RESOLVE_HOST,
+        // and a subsequent request to a highly-available site
+        // also returns CURLE_COULDNT_RESOLVE_HOST.
+        // cleared whenever we transfer data,
+        // or an operation returns some other value
     bool have_sporadic_connection;
+        // we have a network connection, but it's likely to go away soon,
+        // so do as much network comm as possible
+        // (e.g. report completed results)
 	double last_comm_time;
 
     int network_status();
     void network_available();
     void got_http_error();
+    void http_op_succeeded();
     NET_STATUS() {
         need_physical_connection = false;
         have_sporadic_connection = false;
@@ -96,9 +98,9 @@ public:
     void poll();
 };
 
-/// This is used to access a reference website (like yahoo or google)
-/// that is assumed to be 100% available.
-/// It is used ONLY from the HTTP code, when a transaction fails
+// This is used to access a reference website (like yahoo or google)
+// that is assumed to be 100% available.
+// It is used ONLY from the HTTP code, when a transaction fails
 
 struct LOOKUP_WEBSITE_OP: public GUI_HTTP_OP {
     int error_num;
