@@ -21,9 +21,6 @@
 #include "boinc_win.h"
 #else
 #include "config.h"
-#endif
-
-#ifndef _WIN32
 #include <cmath>
 #include <cstdlib>
 #endif
@@ -130,11 +127,11 @@ int PERS_FILE_XFER::create_xfer() {
     if (retval) {
         if (log_flags.http_debug) {
             msg_printf(
-                fip->project, MSG_INFO, "[file_xfer_debug] Couldn't start %s of %s",
+                fip->project, MSG_INFO, "[file_xfer] Couldn't start %s of %s",
                 (is_upload ? "upload" : "download"), fip->name
             );
             msg_printf(
-                fip->project, MSG_INFO, "[file_xfer_debug] URL %s: %s",
+                fip->project, MSG_INFO, "[file_xfer] URL %s: %s",
                 fip->get_current_url(), boincerror(retval)
             );
         }
@@ -153,7 +150,7 @@ int PERS_FILE_XFER::create_xfer() {
     }
     if (log_flags.file_xfer_debug) {
         msg_printf(fip->project, MSG_INFO,
-            "[file_xfer_debug] URL: %s\n",
+            "[file_xfer] URL: %s\n",
             fip->get_current_url()
         );
     }
@@ -183,7 +180,7 @@ bool PERS_FILE_XFER::poll() {
 #if 0
             if (log_flags.file_xfer_debug) {
                 msg_printf(fip->project, MSG_INFO,
-                    "[file_xfer_debug] delaying %s of %s: project-wide backoff %f sec",
+                    "[file_xfer] delaying %s of %s: project-wide backoff %f sec",
                     is_upload?"upload":"download", fip->name,
                     fxb.next_xfer_time - gstate.now
                 );
@@ -199,7 +196,7 @@ bool PERS_FILE_XFER::poll() {
     // copy bytes_xferred for use in GUI
     //
     last_bytes_xferred = fxp->bytes_xferred;
-    if (fxp->is_upload) {
+    if (is_upload) {
         last_bytes_xferred += fxp->file_offset;
     }
 
@@ -214,8 +211,8 @@ bool PERS_FILE_XFER::poll() {
     if (fxp->file_xfer_done) {
         if (log_flags.file_xfer_debug) {
             msg_printf(fip->project, MSG_INFO,
-                "[file_xfer_debug] file transfer status %d",
-                fxp->file_xfer_retval
+                "[file_xfer] file transfer status %d (%s)",
+                fxp->file_xfer_retval, boincerror(fxp->file_xfer_retval)
             );
         }
         switch (fxp->file_xfer_retval) {
@@ -229,10 +226,10 @@ bool PERS_FILE_XFER::poll() {
             }
             if (log_flags.file_xfer_debug) {
                 if (fxp->xfer_speed < 0) {
-                    msg_printf(fip->project, MSG_INFO, "[file_xfer_debug] No data transferred");
+                    msg_printf(fip->project, MSG_INFO, "[file_xfer] No data transferred");
                 } else {
                     msg_printf(
-                        fip->project, MSG_INFO, "[file_xfer_debug] Throughput %d bytes/sec",
+                        fip->project, MSG_INFO, "[file_xfer] Throughput %d bytes/sec",
                         (int)fxp->xfer_speed
                     );
                 }
@@ -286,6 +283,11 @@ bool PERS_FILE_XFER::poll() {
             delete fxp;
             fxp = NULL;
         }
+
+        if (is_upload && !fip->project->uploading()) {
+            gstate.request_work_fetch("project finished uploading");
+        }
+
         return true;
     }
     return false;
@@ -466,7 +468,14 @@ bool PERS_FILE_XFER_SET::poll() {
     if (gstate.now - last_time < PERS_FILE_XFER_POLL_PERIOD) return false;
     last_time = gstate.now;
 
+    // try to finish ones we've already started
+    //
     for (i=0; i<pers_file_xfers.size(); i++) {
+        if (!pers_file_xfers[i]->last_bytes_xferred) continue;
+        action |= pers_file_xfers[i]->poll();
+    }
+    for (i=0; i<pers_file_xfers.size(); i++) {
+        if (pers_file_xfers[i]->last_bytes_xferred) continue;
         action |= pers_file_xfers[i]->poll();
     }
 
