@@ -49,10 +49,10 @@
 #define SCHED_RETRY_DELAY_MAX    (60*60*4)         // 4 hours
 
 
-/// SCHEDULER_OP encapsulates the mechanism for
-/// 1) fetching master files
-/// 2) communicating with scheduling servers
-/// Only one such operation can be in progress at once.
+// SCHEDULER_OP encapsulates the mechanism for
+// 1) fetching master files
+// 2) communicating with scheduling servers
+// Only one such operation can be in progress at once.
 
 class SCHEDULER_OP {
 private:
@@ -60,15 +60,15 @@ private:
     HTTP_OP http_op;
     HTTP_OP_SET* http_ops;
     char scheduler_url[256];
-        /// index within project's URL list
     int url_index;
+        // index within project's URL list
 public:
-        /// project we're currently contacting
     PROJECT* cur_proj;
+        // project we're currently contacting
     int state;
     int reason;
-        /// used to randomize order
     double url_random;
+        // used to randomize order
 
 public:
     SCHEDULER_OP(HTTP_OP_SET*);
@@ -77,8 +77,8 @@ public:
     int init_master_fetch(PROJECT*);
     bool check_master_fetch_start();
     void backoff(PROJECT* p, const char *error_msg);
-        /// if we're doing an op to this project, abort it
     void abort(PROJECT*);
+        // if we're doing an op to this project, abort it
 private:
     bool update_urls(PROJECT*, std::vector<std::string> &urls);
     int start_op(PROJECT*);
@@ -98,12 +98,12 @@ struct SCHEDULER_REPLY {
     double request_delay;
     double next_rpc_delay;
     std::vector<USER_MESSAGE> messages;
-        /// not including <global_preferences> tags;
-        /// may include <venue> elements
     char* global_prefs_xml;
-        /// not including <project_preferences> tags
-        /// may include <venue> elements
+        // not including <global_preferences> tags;
+        // may include <venue> elements
     char* project_prefs_xml;
+        // not including <project_preferences> tags
+        // may include <venue> elements
     char master_url[256];
     char host_venue[256];
     unsigned int user_create_time;
@@ -121,6 +121,7 @@ struct SCHEDULER_REPLY {
     bool message_ack;
     bool project_is_down;
     bool send_file_list;      
+    bool send_full_workload;      
     int send_time_stats_log;
     int send_job_log;
     int scheduler_version;
@@ -128,7 +129,11 @@ struct SCHEDULER_REPLY {
     double cpu_backoff;
     double cuda_backoff;
     double ati_backoff;
+    std::vector<RSS_FEED> sr_feeds;
+    bool got_rss_feeds;
+        // whether scheduler reply included <rss_feeds>
 
+    void clear();
     SCHEDULER_REPLY();
     ~SCHEDULER_REPLY();
     int parse(FILE*, PROJECT*);
