@@ -32,13 +32,14 @@
 #include "sandbox.h"
 #include "main.h"
 #include "cs_proxy.h"
+#include "net_stats.h"
 
 #include "sysmon_win.h"
 
 
-static HANDLE g_hWindowsMonitorSystemThread = NULL;
-static DWORD g_WindowsMonitorSystemThreadID = NULL;
-static HWND g_hWndWindowsMonitorSystem = NULL;
+static HANDLE g_hWindowsMonitorSystemPowerThread = NULL;
+static HWND g_hWndWindowsMonitorSystemPower = NULL;
+static HANDLE g_hWindowsMonitorSystemProxyThread = NULL;
 
 
 // The following 3 functions are called in a separate thread,
@@ -56,19 +57,13 @@ static void quit_client() {
 }
 
 // Suspend client operations
-static void suspend_client(bool wait) {
-    gstate.requested_suspend = true;
-    if (wait) {
-        while (1) {
-            boinc_sleep(1.0);
-            if (!gstate.active_tasks.is_task_executing()) break;
-        }
-    }
+static void suspend_client() {
+    gstate.os_requested_suspend = true;
 }
 
 // Resume client operations
 static void resume_client() {
-    gstate.requested_resume = true;
+    gstate.os_requested_suspend = false;
 }
 
 // Process console messages sent by the system
@@ -98,10 +93,117 @@ static BOOL WINAPI console_control_handler( DWORD dwCtrlType ){
     return bReturnStatus;
 }
 
+// Trap events on Windows so we can clean ourselves up.
+static LRESULT CALLBACK WindowsMonitorSystemPowerWndProc(
+    HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam
+) {
+    switch(uMsg) {
+        // On Windows power events are broadcast via the WM_POWERBROADCAST
+        //   window message.  It has the following parameters:
+        //     PBT_APMQUERYSUSPEND
+        //     PBT_APMQUERYSUSPENDFAILED
+        //     PBT_APMSUSPEND
+        //     PBT_APMRESUMECRITICAL
+        //     PBT_APMRESUMESUSPEND
+        //     PBT_APMBATTERYLOW
+        //     PBT_APMPOWERSTATUSCHANGE
+        //     PBT_APMOEMEVENT
+        //     PBT_APMRESUMEAUTOMATIC 
+        case WM_POWERBROADCAST:
+            switch(wParam) {
+                // System is preparing to suspend.  This is valid on
+                //   Windows versions older than Vista
+                case PBT_APMQUERYSUSPEND:
+                    return TRUE;
+                    break;
+
+                // System is resuming from a failed request to suspend
+                //   activity.  This is only valid on Windows versions
+                //   older than Vista
+                case PBT_APMQUERYSUSPENDFAILED:
+                    resume_client();
+                    break;
+
+                // System is critically low on battery power.  This is
+                //   only valid on Windows versions older than Vista
+                case PBT_APMBATTERYLOW:
+                    msg_printf(NULL, MSG_INFO, "Critical battery alarm, Windows is suspending operations");
+                    suspend_client();
+                    break;
+
+                // System is suspending
+                case PBT_APMSUSPEND:
+                    msg_printf(NULL, MSG_INFO, "Windows is suspending operations");
+                    suspend_client();
+                    break;
+
+                // System is resuming from a normal power event
+                case PBT_APMRESUMESUSPEND:
+                    msg_printf(NULL, MSG_INFO, "Windows is resuming operations");
+
+                    // Check for a proxy
+                    working_proxy_info.need_autodetect_proxy_settings = true;
+
+                    resume_client();
+                    break;
+            }
+            break;
+        default:
+            break;
+    }
+    return (DefWindowProc(hWnd, uMsg, wParam, lParam));
+}
+
+// Create a thread to monitor system events
+static DWORD WINAPI WindowsMonitorSystemPowerThread( LPVOID  ) {
+    WNDCLASS wc;
+    MSG msg;
+
+    wc.style         = CS_GLOBALCLASS;
+    wc.lpfnWndProc   = (WNDPROC)WindowsMonitorSystemPowerWndProc;
+    wc.cbClsExtra    = 0;
+    wc.cbWndExtra    = 0;
+    wc.hInstance     = NULL;
+    wc.hIcon         = NULL;
+    wc.hCursor       = NULL;
+    wc.hbrBackground = NULL;
+    wc.lpszMenuName  = NULL;
+	wc.lpszClassName = "BOINCWindowsMonitorSystemPower";
+
+    if (!RegisterClass(&wc)) {
+        log_message_error("Failed to register the WindowsMonitorSystem window class.");
+        return 1;
+    }
+
+    g_hWndWindowsMonitorSystemPower = CreateWindow(
+        wc.lpszClassName,
+		"BOINC Monitor System (Power)",
+        WS_OVERLAPPEDWINDOW & ~WS_VISIBLE,
+        CW_USEDEFAULT,
+        CW_USEDEFAULT,
+        CW_USEDEFAULT,
+        CW_USEDEFAULT,
+        NULL,
+        NULL,
+        NULL,
+        NULL);
+
+    if (!g_hWndWindowsMonitorSystemPower) {
+        log_message_error("Failed to create the WindowsMonitorSystem window.");
+        return 0;
+    }
+
+    while (GetMessage(&msg, NULL, 0, 0)) {
+        TranslateMessage(&msg);
+        DispatchMessage(&msg);
+    }
+    return 0;
+}
+
 // Detect any proxy configuration settings automatically.
 static void windows_detect_autoproxy_settings() {
     if (log_flags.proxy_debug) {
-        msg_printf(NULL, MSG_INFO, "[proxy_debug] automatic proxy check in progress");
+        msg_printf(NULL, MSG_INFO, "[proxy] automatic proxy check in progress");
     }
 
     HMODULE hModWinHttp = LoadLibrary("winhttp.dll");
@@ -154,7 +256,7 @@ static void windows_detect_autoproxy_settings() {
     if (pWinHttpGetProxyForUrl(hWinHttp, network_test_url.c_str(), &autoproxy_options, &proxy_info)) {
 
         if (log_flags.proxy_debug) {
-            msg_printf(NULL, MSG_INFO, "[proxy_debug] successfully executed proxy check", hWinHttp);
+            msg_printf(NULL, MSG_INFO, "[proxy] successfully executed proxy check", hWinHttp);
         }
 
         // Apparently there are some conditions where WinHttpGetProxyForUrl can return
@@ -167,7 +269,7 @@ static void windows_detect_autoproxy_settings() {
             std::string new_proxy;
 
             if (log_flags.proxy_debug) {
-                msg_printf(NULL, MSG_INFO, "[proxy_debug] proxy list '%s'", proxy.c_str());
+                msg_printf(NULL, MSG_INFO, "[proxy] proxy list '%s'", proxy.c_str());
             }
 
             if (!proxy.empty()) {
@@ -193,13 +295,18 @@ static void windows_detect_autoproxy_settings() {
                 parse_url(proxy.c_str(), purl);
 
                 // Store the results for future use.
-                working_proxy_info.autodetect_protocol = purl.protocol;
-                strcpy(working_proxy_info.autodetect_server_name, purl.host);
-                working_proxy_info.autodetect_port = purl.port;
+                if (0 != strcmp(working_proxy_info.autodetect_server_name, purl.host)) {
+                    // Reset clients connection error detection path
+                    net_status.need_physical_connection = false;
+
+                    working_proxy_info.autodetect_protocol = purl.protocol;
+                    strcpy(working_proxy_info.autodetect_server_name, purl.host);
+                    working_proxy_info.autodetect_port = purl.port;
+                }
 
                 if (log_flags.proxy_debug) {
                     msg_printf(NULL, MSG_INFO,
-                        "[proxy_debug] automatic proxy detected %s:%d",
+                        "[proxy] automatic proxy detected %s:%d",
                         purl.host, purl.port
                     );
                 }
@@ -216,142 +323,34 @@ static void windows_detect_autoproxy_settings() {
         strcpy(working_proxy_info.autodetect_server_name, "");
         working_proxy_info.autodetect_port = 0;
         if (log_flags.proxy_debug) {
-            msg_printf(NULL, MSG_INFO, "[proxy_debug] no automatic proxy detected");
+            msg_printf(NULL, MSG_INFO, "[proxy] no automatic proxy detected");
         }
     }
     if (hWinHttp) pWinHttpCloseHandle(hWinHttp);
     FreeLibrary(hModWinHttp);
     if (log_flags.proxy_debug) {
-        msg_printf(NULL, MSG_INFO, "[proxy_debug] automatic proxy check finished");
+        msg_printf(NULL, MSG_INFO, "[proxy] automatic proxy check finished");
     }
 }
 
-// Trap events on Windows so we can clean ourselves up.
-static LRESULT CALLBACK WindowsMonitorSystemWndProc(
-    HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam
-) {
-    switch(uMsg) {
-        // Process Timer Events
-        case WM_TIMER:
-            switch(wParam) {
+static DWORD WINAPI WindowsMonitorSystemProxyThread( LPVOID  ) {
+    
+    // notify the main client thread that detecting proxies is
+    // supported.
+    working_proxy_info.autodetect_proxy_supported = true;
 
-                // System Monitor 1 second timer
-                case 1:
-                    if (working_proxy_info.need_autodetect_proxy_settings) {
-                        working_proxy_info.have_autodetect_proxy_settings = false;
-                        windows_detect_autoproxy_settings();
-                        working_proxy_info.need_autodetect_proxy_settings = false;
-                        working_proxy_info.have_autodetect_proxy_settings = true;
-                    }
-                default:
-                    break;
-            }
-            break;
+    while (1) {
 
-        // On Windows power events are broadcast via the WM_POWERBROADCAST
-        //   window message.  It has the following parameters:
-        //     PBT_APMQUERYSUSPEND
-        //     PBT_APMQUERYSUSPENDFAILED
-        //     PBT_APMSUSPEND
-        //     PBT_APMRESUMECRITICAL
-        //     PBT_APMRESUMESUSPEND
-        //     PBT_APMBATTERYLOW
-        //     PBT_APMPOWERSTATUSCHANGE
-        //     PBT_APMOEMEVENT
-        //     PBT_APMRESUMEAUTOMATIC 
-        case WM_POWERBROADCAST:
-            switch(wParam) {
-                // System is preparing to suspend.  This is valid on
-                //   Windows versions older than Vista
-                case PBT_APMQUERYSUSPEND:
-                    return TRUE;
-                    break;
+        if (working_proxy_info.need_autodetect_proxy_settings) {
+            working_proxy_info.have_autodetect_proxy_settings = false;
+            windows_detect_autoproxy_settings();
+            working_proxy_info.need_autodetect_proxy_settings = false;
+            working_proxy_info.have_autodetect_proxy_settings = true;
+        }
 
-                // System is resuming from a failed request to suspend
-                //   activity.  This is only valid on Windows versions
-                //   older than Vista
-                case PBT_APMQUERYSUSPENDFAILED:
-                    resume_client();
-                    break;
-
-                // System is critically low on battery power.  This is
-                //   only valid on Windows versions older than Vista
-                case PBT_APMBATTERYLOW:
-                    msg_printf(NULL, MSG_INFO, "Critical battery alarm, Windows is suspending operations");
-                    suspend_client(true);
-                    break;
-
-                // System is suspending
-                case PBT_APMSUSPEND:
-                    msg_printf(NULL, MSG_INFO, "Windows is suspending operations");
-                    suspend_client(true);
-                    break;
-
-                // System is resuming from a normal power event
-                case PBT_APMRESUMESUSPEND:
-                    msg_printf(NULL, MSG_INFO, "Windows is resuming operations");
-
-                    // Check for a proxy
-                    working_proxy_info.need_autodetect_proxy_settings = true;
-
-                    resume_client();
-                    break;
-            }
-            break;
-        default:
-            break;
-    }
-    return (DefWindowProc(hWnd, uMsg, wParam, lParam));
-}
-
-// Create a thread to monitor system events
-static DWORD WINAPI WindowsMonitorSystemThread( LPVOID  ) {
-    WNDCLASS wc;
-    MSG msg;
-
-    wc.style         = CS_GLOBALCLASS;
-    wc.lpfnWndProc   = (WNDPROC)WindowsMonitorSystemWndProc;
-    wc.cbClsExtra    = 0;
-    wc.cbWndExtra    = 0;
-    wc.hInstance     = NULL;
-    wc.hIcon         = NULL;
-    wc.hCursor       = NULL;
-    wc.hbrBackground = NULL;
-    wc.lpszMenuName  = NULL;
-	wc.lpszClassName = "BOINCWindowsMonitorSystem";
-
-    if (!RegisterClass(&wc)) {
-        log_message_error("Failed to register the WindowsMonitorSystem window class.");
-        return 1;
+        Sleep(1000);
     }
 
-    /* Create an invisible window */
-    g_hWndWindowsMonitorSystem = CreateWindow(
-        wc.lpszClassName,
-		"BOINC Monitor System",
-        WS_OVERLAPPEDWINDOW & ~WS_VISIBLE,
-        CW_USEDEFAULT,
-        CW_USEDEFAULT,
-        CW_USEDEFAULT,
-        CW_USEDEFAULT,
-        NULL,
-        NULL,
-        NULL,
-        NULL);
-
-    if (!g_hWndWindowsMonitorSystem) {
-        log_message_error("Failed to create the WindowsMonitorSystem window.");
-        return 0;
-    }
-
-    if (!SetTimer(g_hWndWindowsMonitorSystem, 1, 1000, NULL)) {
-        log_message_error("Failed to create the WindowsMonitorSystem timer.");
-    }
-
-    while (GetMessage(&msg, NULL, 0, 0)) {
-        TranslateMessage(&msg);
-        DispatchMessage(&msg);
-    }
     return 0;
 }
 
@@ -364,21 +363,31 @@ int initialize_system_monitor(int /*argc*/, char** /*argv*/) {
         return ERR_IO;
     }
 
-    // Create a window to receive system events that are
-    //   not taken care of by the console APIs.  The console
-    //   APIs haven't been updated to handle various power states.
-    g_hWindowsMonitorSystemThread = CreateThread(
+    // Create a window to receive system power events.
+    g_hWindowsMonitorSystemPowerThread = CreateThread(
         NULL,
         0,
-        WindowsMonitorSystemThread,
+        WindowsMonitorSystemPowerThread,
         NULL,
         0,
-        &g_WindowsMonitorSystemThreadID);
+        NULL);
 
-    if (!g_hWindowsMonitorSystemThread) {
-        g_hWindowsMonitorSystemThread = NULL;
-        g_WindowsMonitorSystemThreadID = NULL;
-        g_hWndWindowsMonitorSystem = NULL;
+    if (!g_hWindowsMonitorSystemPowerThread) {
+        g_hWindowsMonitorSystemPowerThread = NULL;
+        g_hWndWindowsMonitorSystemPower = NULL;
+    }
+
+    // Create a thread to handle proxy auto-detection.
+    g_hWindowsMonitorSystemProxyThread = CreateThread(
+        NULL,
+        0,
+        WindowsMonitorSystemProxyThread,
+        NULL,
+        0,
+        NULL);
+
+    if (!g_hWindowsMonitorSystemProxyThread) {
+        g_hWindowsMonitorSystemProxyThread = NULL;
     }
 
     return 0;
@@ -387,18 +396,17 @@ int initialize_system_monitor(int /*argc*/, char** /*argv*/) {
 // Cleanup the system event monitor
 int cleanup_system_monitor() {
 
-    KillTimer(g_hWndWindowsMonitorSystem, 1);
-
-    if (g_WindowsMonitorSystemThreadID) {
-	    PostThreadMessage(g_WindowsMonitorSystemThreadID, WM_QUIT, 0, 0);
-        WaitForSingleObject(g_hWindowsMonitorSystemThread, 10000);
+    if (g_hWindowsMonitorSystemPowerThread) {
+        TerminateThread(g_hWindowsMonitorSystemPowerThread, 0);
+        CloseHandle(g_hWindowsMonitorSystemPowerThread);
+        g_hWindowsMonitorSystemPowerThread = NULL;
+        g_hWndWindowsMonitorSystemPower = NULL;
     }
 
-    if (g_hWindowsMonitorSystemThread) {
-        CloseHandle(g_hWindowsMonitorSystemThread);
-        g_hWindowsMonitorSystemThread = NULL;
-        g_WindowsMonitorSystemThreadID = NULL;
-        g_hWndWindowsMonitorSystem = NULL;
+    if (g_hWindowsMonitorSystemProxyThread) {
+        TerminateThread(g_hWindowsMonitorSystemProxyThread, 0);
+        CloseHandle(g_hWindowsMonitorSystemProxyThread);
+        g_hWindowsMonitorSystemProxyThread = NULL;
     }
 
     return 0;
@@ -517,7 +525,7 @@ VOID WINAPI BOINCServiceCtrl(DWORD dwCtrlCode)
         //
 		case SERVICE_CONTROL_PAUSE:
             ReportStatus(SERVICE_PAUSE_PENDING, ERROR_SUCCESS, 10000);
-			suspend_client(true);
+            suspend_client();
             ReportStatus(SERVICE_PAUSED, ERROR_SUCCESS, 10000);
             return;
 
