@@ -23,16 +23,22 @@
 
 #include <vector>
 
+//#define USE_REC
+#define REC_HALF_LIFE (10*86400)
+//#define REC_HALF_LIFE (1*86400)
+
+
 #define RSC_TYPE_ANY    0
 #define RSC_TYPE_CPU    1
 #define RSC_TYPE_CUDA   2
 #define RSC_TYPE_ATI    3
 
-class PROJECT;
+struct PROJECT;
 struct RESULT;
-class ACTIVE_TASK;
+struct ACTIVE_TASK;
 struct RSC_WORK_FETCH;
 struct SCHEDULER_REPLY;
+struct APP_VERSION;
 
 // state per (resource, project) pair
 //
@@ -40,12 +46,14 @@ struct RSC_PROJECT_WORK_FETCH {
     // the following are persistent (saved in state file)
     double backoff_time;
     double backoff_interval;
+#ifndef USE_REC
     double long_term_debt;
     double short_term_debt;
-
-    // the following used by debt accounting
     double anticipated_debt;
         // short-term debt, adjusted by scheduled jobs
+#endif
+
+    // the following used by debt accounting
     double secs_this_debt_interval;
     inline void reset_debt_accounting() {
         secs_this_debt_interval = 0;
@@ -78,6 +86,7 @@ struct RSC_PROJECT_WORK_FETCH {
         memset(this, 0, sizeof(*this));
     }
 
+#ifndef USE_REC
     // whether this project should accumulate debt for this resource
     //
     bool debt_eligible(PROJECT*, RSC_WORK_FETCH&);
@@ -86,13 +95,16 @@ struct RSC_PROJECT_WORK_FETCH {
         long_term_debt = 0;
         short_term_debt = 0;
     }
+#endif
 
     inline void reset() {
         backoff_time = 0;
         backoff_interval = 0;
+#ifndef USE_REC
         long_term_debt = 0;
         short_term_debt = 0;
         anticipated_debt = 0;
+#endif
     }
 
     bool may_have_work;
@@ -103,7 +115,9 @@ struct RSC_PROJECT_WORK_FETCH {
         backoff_time = 0;
         backoff_interval = 0;
     }
+#ifndef USE_REC
     bool overworked();
+#endif
 };
 
 // estimate the time a resources will be saturated
@@ -181,7 +195,7 @@ struct RSC_WORK_FETCH {
         // used to calculate work request
     double deadline_missed_instances;
         // instance count for jobs that miss deadline
-    std::vector<RESULT*> pending;
+    std::deque<RESULT*> pending;
     BUSY_TIME_ESTIMATOR busy_time_estimator;
 
     void init(int t, int n, double sp) {
@@ -206,16 +220,26 @@ struct RSC_WORK_FETCH {
     void update_saturated_time(double dt);
     void update_busy_time(double dur, double nused);
     PROJECT* choose_project(int);
-    void accumulate_debt();
     RSC_PROJECT_WORK_FETCH& project_state(PROJECT*);
+#ifndef USE_REC
     void update_long_term_debts();
     void update_short_term_debts();
+#endif
     void print_state(const char*);
     void clear_request();
     void set_request(PROJECT*, bool allow_overworked);
     bool may_have_work(PROJECT*);
     RSC_WORK_FETCH() {
-        memset(this, 0, sizeof(*this));
+        rsc_type = 0;
+        ninstances = 0;
+        relative_speed = 0;
+        shortfall = 0;
+        nidle_now = 0;
+        sim_nused = 0;
+        total_fetchable_share = 0;
+        total_runnable_share = 0;
+        saturated_time = 0;
+        deadline_missed_instances = 0;
     }
 };
 
@@ -223,7 +247,16 @@ struct RSC_WORK_FETCH {
 // per project state
 //
 struct PROJECT_WORK_FETCH {
+#ifdef USE_REC
+    double rec;
+        // recent estimated credit
+    double rec_time;
+        // when it was last updated
+    double rec_temp;
+        // temporary copy used during schedule_cpus()
+#else
     double overall_debt;
+#endif
     bool can_fetch_work;
     bool compute_can_fetch_work(PROJECT*);
     bool has_runnable_jobs;
@@ -236,7 +269,10 @@ struct PROJECT_WORK_FETCH {
 // global work fetch state
 //
 struct WORK_FETCH {
+#ifndef USE_REC
     void set_overall_debts();
+    void zero_debts();
+#endif
     PROJECT* choose_project();
         // find a project to ask for work
     PROJECT* non_cpu_intensive_project_needing_work();
@@ -255,7 +291,8 @@ struct WORK_FETCH {
     void rr_init();
     void clear_request();
     void compute_shares();
-    void zero_debts();
+    void clear_backoffs(APP_VERSION&);
+    void request_string(char*);
 };
 
 extern RSC_WORK_FETCH cuda_work_fetch;
@@ -263,4 +300,10 @@ extern RSC_WORK_FETCH ati_work_fetch;
 extern RSC_WORK_FETCH cpu_work_fetch;
 extern WORK_FETCH work_fetch;
 
+#ifdef USE_REC
+void project_priority_init();
+double project_priority(PROJECT*);
+void adjust_rec_temp(RESULT*);
+#endif
+
 #endif
