@@ -133,6 +133,7 @@ enum RPC_SELECTOR {
     RPC_GET_GLOBAL_PREFS_OVERRIDE_STRUCT,
     RPC_SET_GLOBAL_PREFS_OVERRIDE_STRUCT,   // 50
     RPC_SET_DEBTS,
+    RPC_GET_NOTICES,
     NUM_RPC_SELECTORS
 };
 
@@ -147,14 +148,13 @@ enum ASYNC_RPC_TYPE {
     // Periodic RPC as above, but on completion also process a 
     // wxEVT_FRAME_REFRESHVIEW event to refresh the display.
     RPC_TYPE_ASYNC_WITH_REFRESH_AFTER,
-    // Periodic RPC as above, but on completion also update message 
-    // list by calling CMainDocument::CachedMessageUpdate().
-    RPC_TYPE_ASYNC_WITH_UPDATE_MESSAGE_LIST_AFTER,
+    // Periodic RPC as above, but on completion also process a 
+    // wxEVT_FRAME_REFRESHVIEW event to refresh the display.
+    RPC_TYPE_ASYNC_WITH_REFRESH_EVENT_LOG_AFTER,
     // Periodic RPC as above, but on completion also process a 
     // wxEVT_TASKBAR_REFRESH event to refresh the taskbar icon.
     RPC_TYPE_ASYNC_WITH_UPDATE_TASKBAR_ICON_AFTER,
     NUM_RPC_TYPES
-
 };
 
 // Pass the following structure to CMainDocument::RequestRPC()
@@ -191,6 +191,7 @@ struct ASYNC_RPC_REQUEST {
     void *arg4;
     ASYNC_RPC_TYPE rpcType;
     wxDateTime *completionTime;
+    double *RPCExecutionTime;
     int *resultPtr;
     int retval;
     bool isActive;
@@ -264,6 +265,8 @@ public:
             { return RPC_Wait(RPC_GET_PROXY_SETTINGS, (void*)&arg1); }
     int get_messages(int seqno, MESSAGES& arg1)
             { return RPC_Wait(RPC_GET_MESSAGES, (void*)&seqno, (void*)&arg1); }
+    int get_notices(int seqno, NOTICES& arg1)
+            { return RPC_Wait(RPC_GET_NOTICES, (void*)&seqno, (void*)&arg1); }
     int file_transfer_op(FILE_TRANSFER& arg1, const char* op)
             { return RPC_Wait(RPC_FILE_TRANSFER_OP, (void*)&arg1, (void*)op); }
     int result_op(RESULT& arg1, const char* op)
@@ -312,8 +315,8 @@ public:
     int acct_mgr_rpc_poll(ACCT_MGR_RPC_REPLY& arg1)
             { return RPC_Wait(RPC_ACCT_MGR_RPC_POLL, (void*)&arg1); }
 
-    int get_newer_version(std::string& arg1)
-            { return RPC_Wait(RPC_GET_NEWER_VERSION, (void*)&arg1); }
+    int get_newer_version(std::string& version, std::string& version_download_url)
+            { return RPC_Wait(RPC_GET_NEWER_VERSION, (void*)&version, (void*)&version_download_url); }
     int read_global_prefs_override()
             { return RPC_Wait(RPC_READ_GLOBAL_PREFS_OVERRIDE); }
     int read_cc_config()
