@@ -21,17 +21,24 @@
 
 #include "stdwx.h"
 #include "diagnostics.h"
+#include "miofile.h"
 #include "LogBOINC.h"
 #include "BOINCGUIApp.h"
+#include "SkinManager.h"
 #include "MainDocument.h"
 #include "BOINCBaseFrame.h"
 #include "AdvancedFrame.h"
 #include "BOINCClientManager.h"
 #include "error_numbers.h"
 #include "procinfo.h"
-
 #include "filesys.h"
+#include "daemonmgt.h"
 #include "util.h"
+#include "Events.h"
+#include "version.h"
+
+// Alert user if Client crashes 3 times in 30 minutes
+#define CLIENT_3_CRASH_MAX_TIME 30
 
 #ifdef __WXMAC__
 enum {
@@ -45,13 +52,6 @@ enum {
 #include "diagnostics_win.h"
 
 extern int diagnostics_get_process_information(PVOID* ppBuffer, PULONG pcbBuffer);
-EXTERN_C BOOL  IsBOINCServiceInstalled();
-EXTERN_C BOOL  IsBOINCServiceStarting();
-EXTERN_C BOOL  IsBOINCServiceRunning();
-EXTERN_C BOOL  IsBOINCServiceStopping();
-EXTERN_C BOOL  IsBOINCServiceStopped();
-EXTERN_C BOOL  StartBOINCService();
-EXTERN_C BOOL  StopBOINCService();
 
 #else
 #include <sys/wait.h>
@@ -62,6 +62,8 @@ CBOINCClientManager::CBOINCClientManager() {
 
     m_bBOINCStartedByManager = false;
     m_lBOINCCoreProcessId = 0;
+    m_fAutoRestart1Time = 0;
+    m_fAutoRestart2Time = 0;
 
 #ifdef __WXMSW__
     m_hBOINCCoreProcess = NULL;
@@ -79,11 +81,35 @@ CBOINCClientManager::~CBOINCClientManager() {
 
 
 bool CBOINCClientManager::AutoRestart() {
+    double timeNow, timeDiff;
     if (IsBOINCCoreRunning()) return true;
-#ifndef __WXMAC__       // Mac can restart Client as a daemon
-    if (! m_bBOINCStartedByManager) return false;
+#if ! (defined(__WXMAC__) || defined(__WXMSW__)) 
+// Mac and Windows can restart Client as a daemon, but 
+// Linux may not know Client's location if it didn't start the Client
+    if (!m_bBOINCStartedByManager) return false;
 #endif
+    // Alert user if Client crashes 3 times in CLIENT_3_CRASH_MAX_TIME
+    timeNow = dtime();
+    timeDiff = timeNow - m_fAutoRestart1Time;
+    if ((timeDiff) < (CLIENT_3_CRASH_MAX_TIME * 60)) {
+        int                 response;
+        ClientCrashDlg      *dlg = new ClientCrashDlg(timeDiff);
+        if (dlg) {
+            CBOINCBaseFrame* pFrame = wxGetApp().GetFrame();
+            if (!pFrame->IsShown()) {
+                pFrame->Show();
+            }
+            response = dlg->ShowModal();
+            dlg->Destroy();
+            if (response == wxID_CANCEL) return false;
+            timeNow = 0;
+            m_fAutoRestart1Time = 0;
+            m_fAutoRestart2Time = 0;
+        }
+    }
     m_lBOINCCoreProcessId = 0;
+    m_fAutoRestart1Time = m_fAutoRestart2Time;
+    m_fAutoRestart2Time = timeNow;
     StartupBOINCCore();
     return true;
 }
@@ -103,7 +129,7 @@ bool CBOINCClientManager::IsSystemBooting() {
 int CBOINCClientManager::IsBOINCConfiguredAsDaemon() {
     bool bReturnValue = false;
 #if   defined(__WXMSW__)
-    if (IsBOINCServiceInstalled()) bReturnValue = 1;
+    if (is_daemon_installed()) bReturnValue = 1;
 #elif defined(__WXMAC__)
     if ( boinc_file_exists("/Library/LaunchDaemons/edu.berkeley.boinc.plist")) {
         bReturnValue = NewStyleDaemon;                      // New-style daemon uses launchd
@@ -123,8 +149,8 @@ bool CBOINCClientManager::IsBOINCCoreRunning() {
 #ifdef __WXMSW__
     char buf[MAX_PATH] = "";
     
-    if (IsBOINCServiceInstalled()) {
-        running = (FALSE != IsBOINCServiceStarting()) || (FALSE != IsBOINCServiceRunning());
+    if (is_daemon_installed()) {
+        running = (FALSE != is_daemon_starting()) || (FALSE != is_daemon_running());
     } else {
         // Global mutex on Win2k and later
         //
@@ -195,12 +221,12 @@ bool CBOINCClientManager::StartupBOINCCore() {
 
     if (IsBOINCCoreRunning()) return true;
 
-#if   defined(__WXMSW__)
+#if defined(__WXMSW__)
     LPTSTR  szExecute = NULL;
     LPTSTR  szDataDirectory = NULL;
 
     if (IsBOINCConfiguredAsDaemon()) {
-        StartBOINCService();
+        start_daemon_via_daemonctrl();
 
         m_bBOINCStartedByManager = true;
         bReturnValue = IsBOINCCoreRunning();
@@ -431,59 +457,135 @@ void CBOINCClientManager::ShutdownBOINCCore() {
     wxASSERT(pDoc);
     wxASSERT(wxDynamicCast(pDoc, CMainDocument));
 
-    if (m_bBOINCStartedByManager) {
-#ifdef __WXMSW__
-        if (IsBOINCConfiguredAsDaemon()) {
-            StopBOINCService();
-            bClientQuit = true;
-        } else
+#ifdef __WXMAC__
+    // Mac Manager shuts down client only if Manager started client
+    if (!m_bBOINCStartedByManager) return;
 #endif
-        {
-            pDoc->GetConnectedComputerName(strConnectedCompter);
-            if (!pDoc->IsComputerNameLocal(strConnectedCompter)) {
-                RPC_CLIENT rpc;
-                if (!rpc.init("localhost")) {
-                    pDoc->m_pNetworkConnection->GetLocalPassword(strPassword);
-                    rpc.authorize((const char*)strPassword.mb_str());
-                    if (IsBOINCCoreRunning()) {
-                        rpc.quit();
-                        for (iCount = 0; iCount <= 10; iCount++) {
-                            if (!bClientQuit && !IsBOINCCoreRunning()) {
-                                wxLogTrace(wxT("Function Status"), wxT("CBOINCClientManager::ShutdownBOINCCore - (localhost) Application Exit Detected"));
-                                bClientQuit = true;
-                                break;
-                            }
-                            wxLogTrace(wxT("Function Status"), wxT("CBOINCClientManager::ShutdownBOINCCore - (localhost) Application Exit NOT Detected, Sleeping..."));
-                            ::wxSleep(1);
-                        }
-                    } else {
-                        bClientQuit = true;
-                    }
-                }
-                rpc.close();
-            } else {
+
+#ifdef __WXMSW__
+    if (IsBOINCConfiguredAsDaemon()) {
+        stop_daemon_via_daemonctrl();
+        bClientQuit = true;
+    } else
+#endif
+    {
+        pDoc->GetConnectedComputerName(strConnectedCompter);
+        if (!pDoc->IsComputerNameLocal(strConnectedCompter)) {
+            RPC_CLIENT rpc;
+            if (!rpc.init("localhost")) {
+                pDoc->m_pNetworkConnection->GetLocalPassword(strPassword);
+                rpc.authorize((const char*)strPassword.mb_str());
                 if (IsBOINCCoreRunning()) {
-                    pDoc->CoreClientQuit();
+                    rpc.quit();
                     for (iCount = 0; iCount <= 10; iCount++) {
                         if (!bClientQuit && !IsBOINCCoreRunning()) {
-                            wxLogTrace(wxT("Function Status"), wxT("CBOINCClientManager::ShutdownBOINCCore - Application Exit Detected"));
+                            wxLogTrace(wxT("Function Status"), wxT("CBOINCClientManager::ShutdownBOINCCore - (localhost) Application Exit Detected"));
                             bClientQuit = true;
                             break;
                         }
-                        wxLogTrace(wxT("Function Status"), wxT("CBOINCClientManager::ShutdownBOINCCore - Application Exit NOT Detected, Sleeping..."));
+                        wxLogTrace(wxT("Function Status"), wxT("CBOINCClientManager::ShutdownBOINCCore - (localhost) Application Exit NOT Detected, Sleeping..."));
                         ::wxSleep(1);
                     }
                 } else {
                     bClientQuit = true;
                 }
             }
+            rpc.close();
+        } else {
+            if (IsBOINCCoreRunning()) {
+                pDoc->CoreClientQuit();
+                for (iCount = 0; iCount <= 10; iCount++) {
+                    if (!bClientQuit && !IsBOINCCoreRunning()) {
+                        wxLogTrace(wxT("Function Status"), wxT("CBOINCClientManager::ShutdownBOINCCore - Application Exit Detected"));
+                        bClientQuit = true;
+                        break;
+                    }
+                    wxLogTrace(wxT("Function Status"), wxT("CBOINCClientManager::ShutdownBOINCCore - Application Exit NOT Detected, Sleeping..."));
+                    ::wxSleep(1);
+                }
+            } else {
+                bClientQuit = true;
+            }
         }
-
-        if (!bClientQuit) {
-            KillClient();
-        }
-        m_lBOINCCoreProcessId = 0;
     }
 
+    if (!bClientQuit) {
+        KillClient();
+    }
+    m_lBOINCCoreProcessId = 0;
+
     wxLogTrace(wxT("Function Start/End"), wxT("CBOINCClientManager::ShutdownBOINCCore - Function End"));
 }
+
+
+BEGIN_EVENT_TABLE(ClientCrashDlg, wxDialog)
+    EVT_BUTTON(wxID_HELP, ClientCrashDlg::OnHelp)
+END_EVENT_TABLE()
+
+IMPLEMENT_CLASS(ClientCrashDlg, wxDialog)
+
+ClientCrashDlg::ClientCrashDlg(double timeDiff) : wxDialog( NULL, wxID_ANY, wxT(""), wxDefaultPosition ) {
+    wxString            strDialogTitle = wxEmptyString;
+    wxString            strDialogMessage = wxEmptyString;
+    int                 minutes = wxMax((int)((timeDiff + 59.) / 60.), 2);
+    CSkinAdvanced*      pSkinAdvanced = wxGetApp().GetSkinManager()->GetAdvanced();
+    wxASSERT(pSkinAdvanced);
+    
+    // %s is the application name
+    //    i.e. 'BOINC Manager', 'GridRepublic Manager'
+    strDialogTitle.Printf(
+        _("%s - Unexpected Exit"),
+        pSkinAdvanced->GetApplicationName().c_str()
+    );
+    SetTitle(strDialogTitle.c_str());
+
+    // 1st %s is the application name
+    //    i.e. 'BOINC Manager', 'GridRepublic Manager'
+    // 2st %s is the project name
+    //    i.e. 'BOINC', 'GridRepublic'
+    strDialogMessage.Printf(
+        _("The %s client has exited unexpectedly 3 times within the last %d minutes.\nWould you like to restart it again?"),
+        pSkinAdvanced->GetApplicationShortName().c_str(),
+        minutes        
+    );
+            
+    wxBoxSizer *topsizer = new wxBoxSizer( wxVERTICAL );
+    wxBoxSizer *icon_text = new wxBoxSizer( wxHORIZONTAL );
+
+    icon_text->Add( CreateTextSizer( strDialogMessage ), 0, wxALIGN_CENTER | wxLEFT, 10 );
+    topsizer->Add( icon_text, 1, wxCENTER | wxLEFT|wxRIGHT|wxTOP, 10 );
+    
+    wxStdDialogButtonSizer *sizerBtn = CreateStdDialogButtonSizer(wxYES | wxNO | wxHELP);
+    SetEscapeId(wxID_NO);   // Changes return value of NO button to wxID_CANCEL
+    
+    if ( sizerBtn )
+        topsizer->Add(sizerBtn, 0, wxEXPAND | wxALL, 10 );
+
+    SetAutoLayout( true );
+    SetSizer( topsizer );
+
+    topsizer->SetSizeHints( this );
+    topsizer->Fit( this );
+    wxSize size( GetSize() );
+    if (size.x < size.y*3/2)
+    {
+        size.x = size.y*3/2;
+        SetSize( size );
+    }
+
+    Centre( wxBOTH | wxCENTER_FRAME);
+}
+
+
+void ClientCrashDlg::OnHelp(wxCommandEvent& WXUNUSED(eventUnused)) {
+    wxString strURL = wxGetApp().GetSkinManager()->GetAdvanced()->GetOrganizationHelpUrl();
+
+    wxString wxurl;
+    wxurl.Printf(
+        wxT("%s?target=crash_detection&version=%s&controlid=%d"),
+        strURL.c_str(),
+        wxString(BOINC_VERSION_STRING, wxConvUTF8).c_str(),
+        ID_HELPBOINC
+    );
+    wxLaunchDefaultBrowser(wxurl);
+}
