@@ -189,7 +189,7 @@ bool CSkinImage::Validate() {
         }
         if (!m_bmpBitmap.Ok()) {
             if (show_error_msgs) {
-                fprintf(stderr, "Skin Manager: Failed to load '%s' image. Using default.\n", (char *)m_strComponentName.c_str());
+                fprintf(stderr, "Skin Manager: Failed to load '%s' image. Using default.\n", (const char *)m_strComponentName.mb_str());
             }
             m_bmpBitmap = wxBitmap(m_ppDefaultBitmap);
             wxASSERT(m_bmpBitmap.Ok());
@@ -201,7 +201,7 @@ bool CSkinImage::Validate() {
         }
         if (!m_colBackgroundColor.Ok()) {
             if (show_error_msgs) {
-                fprintf(stderr, "Skin Manager: Failed to load '%s' background color. Using default.\n", (char *)m_strComponentName.c_str());
+                fprintf(stderr, "Skin Manager: Failed to load '%s' background color. Using default.\n", (const char *)m_strComponentName.mb_str());
             }
             m_colBackgroundColor = ParseColor(m_strDefaultBackgroundColor);
             wxASSERT(m_colBackgroundColor.Ok());
@@ -283,7 +283,7 @@ bool CSkinIcon::Validate() {
         }
         if (!m_icoIcon.Ok()) {
             if (show_error_msgs) {
-                fprintf(stderr, "Skin Manager: Failed to load '%s' icon. Using default.\n", (char *)m_strComponentName.c_str());
+                fprintf(stderr, "Skin Manager: Failed to load '%s' icon. Using default.\n", (const char *)m_strComponentName.mb_str());
             }
             m_icoIcon = wxIcon(m_ppDefaultIcon);
             wxASSERT(m_icoIcon.Ok());
@@ -371,7 +371,7 @@ bool CSkinSimpleButton::Validate() {
         }
         if (!m_bmpBitmap.Ok()) {
             if (show_error_msgs) {
-                fprintf(stderr, "Skin Manager: Failed to load '%s' image. Using default.\n", (char *)m_strComponentName.c_str());
+                fprintf(stderr, "Skin Manager: Failed to load '%s' image. Using default.\n", (const char *)m_strComponentName.mb_str());
             }
             m_bmpBitmap = wxBitmap(m_ppDefaultBitmap);
             wxASSERT(m_bmpBitmap.Ok());
@@ -383,7 +383,7 @@ bool CSkinSimpleButton::Validate() {
         }
         if (!m_bmpBitmapClicked.Ok()) {
             if (show_error_msgs) {
-                fprintf(stderr, "Skin Manager: Failed to load '%s' clicked image. Using default.\n", (char *)m_strComponentName.c_str());
+                fprintf(stderr, "Skin Manager: Failed to load '%s' clicked image. Using default.\n", (const char *)m_strComponentName.mb_str());
             }
             m_bmpBitmapClicked = wxBitmap(m_ppDefaultBitmapClicked);
             wxASSERT(m_bmpBitmapClicked.Ok());
@@ -500,7 +500,7 @@ bool CSkinSimpleTab::Validate() {
         }
         if (!m_bmpBitmap.Ok()) {
             if (show_error_msgs) {
-                fprintf(stderr, "Skin Manager: Failed to load '%s' tab image. Using default.\n", (char *)m_strComponentName.c_str());
+                fprintf(stderr, "Skin Manager: Failed to load '%s' tab image. Using default.\n", (const char *)m_strComponentName.mb_str());
             }
             m_bmpBitmap = wxBitmap(m_ppDefaultBitmap);
             wxASSERT(m_bmpBitmap.Ok());
@@ -512,7 +512,7 @@ bool CSkinSimpleTab::Validate() {
         }
         if (!m_colBorderColor.Ok()) {
             if (show_error_msgs) {
-                fprintf(stderr, "Skin Manager: Failed to load '%s' tab border color. Using default.\n", (char *)m_strComponentName.c_str());
+                fprintf(stderr, "Skin Manager: Failed to load '%s' tab border color. Using default.\n", (const char *)m_strComponentName.mb_str());
             }
             m_colBorderColor = ParseColor(m_strDefaultBorderColor);
             wxASSERT(m_colBorderColor.Ok());
@@ -524,7 +524,7 @@ bool CSkinSimpleTab::Validate() {
         }
         if (!m_colGradientFromColor.Ok()) {
             if (show_error_msgs) {
-                fprintf(stderr, "Skin Manager: Failed to load '%s' tab gradient from color. Using default.\n", (char *)m_strComponentName.c_str());
+                fprintf(stderr, "Skin Manager: Failed to load '%s' tab gradient from color. Using default.\n", (const char *)m_strComponentName.mb_str());
             }
             m_colGradientFromColor = ParseColor(m_strDefaultGradientFromColor);
             wxASSERT(m_colGradientFromColor.Ok());
@@ -536,7 +536,7 @@ bool CSkinSimpleTab::Validate() {
         }
         if (!m_colGradientToColor.Ok()) {
             if (show_error_msgs) {
-                fprintf(stderr, "Skin Manager: Failed to load '%s' tab gradient to color. Using default.\n", (char *)m_strComponentName.c_str());
+                fprintf(stderr, "Skin Manager: Failed to load '%s' tab gradient to color. Using default.\n", (const char *)m_strComponentName.mb_str());
             }
             m_colGradientToColor = ParseColor(m_strDefaultGradientToColor);
             wxASSERT(m_colGradientToColor.Ok());
@@ -718,14 +718,14 @@ int CSkinSimple::Parse(MIOFILE& in) {
 
 bool CSkinSimple::InitializeDelayedValidation() {
     m_BackgroundImage.SetDefaults(
-        wxT("background"), (const char**)background_image_xpm, wxT("133:181:178")
+        wxT("background"), (const char**)background_image_xpm, wxT("0:0:0")
     );
     m_SpacerImage.SetDefaults(wxT("spacer"), (const char**)spacer_image_xpm);
     if (!m_StaticLineColor.Ok()) {
         if (show_error_msgs) {
             fprintf(stderr, "Skin Manager: Failed to parse static line color. Using default.\n");
         }
-        m_StaticLineColor = ParseColor(wxString(wxT("204:102:51")));
+        m_StaticLineColor = ParseColor(wxString(wxT("5:5:5")));
         wxASSERT(m_StaticLineColor.Ok());
     }
     m_StateIndicatorBackgroundImage.SetDefaults(
@@ -738,19 +738,19 @@ bool CSkinSimple::InitializeDelayedValidation() {
         wxT("error indicator"), (const char**)error_indicator_image_xpm
     );
     m_WorkunitActiveTab.SetDefaults(
-        wxT("active"), (const char**)workunit_active_image_xpm, wxString(wxT("20:82:82")), wxString(wxT("134:179:176")), wxString(wxT("51:102:102"))
+        wxT("active"), (const char**)workunit_active_image_xpm, wxT("102:102:102"), wxT("102:102:102"), wxT("179:179:179")
     );
     m_WorkunitSuspendedTab.SetDefaults(
-        wxT("suspended"), (const char**)workunit_suspended_image_xpm, wxString(wxT("102:153:153")), wxString(wxT("134:179:176")), wxString(wxT("84:175:175"))
+        wxT("suspended"), (const char**)workunit_suspended_image_xpm, wxT("80:80:80"), wxT("50:50:50"), wxT("50:50:50")
     );
     m_WorkunitTabAreaBackgroundImage.SetDefaults(
         wxT("workunit tab area background"), (const char**)workunit_tab_area_background_image_xpm
     );
     m_WorkunitAreaBackgroundImage.SetDefaults(
-        wxT("workunit area background"), (const char**)workunit_area_background_image_xpm
+        wxT("workunit area background"), (const char**)workunit_area_background_image_xpm, wxT("0:0:0")
     );
     m_WorkunitAnimationBackgroundImage.SetDefaults(
-        wxT("workunit animation background"), (const char**)workunit_animation_background_image_xpm
+        wxT("workunit animation background"), (const char**)workunit_animation_background_image_xpm, wxT("0:0:0")
     );
     m_WorkunitAnimationImage.SetDefaults(
         wxT("workunit animation"), (const char**)workunit_animation_image_xpm
@@ -1103,7 +1103,7 @@ bool CSkinWizardATP::InitializeDelayedValidation() {
     }
     if (m_strTitle.IsEmpty()) {
         if (show_error_msgs) {
-            fprintf(stderr, "Skin Manager: Attach to project wizard title was not defined. Using default.\n");
+            fprintf(stderr, "Skin Manager: Add project wizard title was not defined. Using default.\n");
         }
         m_strTitle = wxT("BOINC Manager");
         wxASSERT(!m_strTitle.IsEmpty());
@@ -1172,7 +1172,7 @@ bool CSkinWizardATAM::InitializeDelayedValidation() {
     }
     if (m_strTitle.IsEmpty()) {
         if (show_error_msgs) {
-            fprintf(stderr, "Skin Manager: Attach to project wizard title was not defined. Using default.\n");
+            fprintf(stderr, "Skin Manager: Add project wizard title was not defined. Using default.\n");
         }
         m_strTitle = wxT("BOINC Manager");
         wxASSERT(!m_strTitle.IsEmpty());
@@ -1264,20 +1264,8 @@ bool CSkinManager::ReloadSkin(wxLocale* pLocale, wxString strSkin) {
     m_strSelectedSkin = strSkin;
 
     // Check to see if the skin we want to change to is the default skin
-    if (GetDefaultSkinName() == m_strSelectedSkin) {
-
-        // Disable the error messages since the default images are
-        //   going to be used.
-        show_error_msgs = false;
-
-        // Validate settings
-        InitializeDelayedValidation();
-
-        // Tell whichever UI elements that are loaded to reload the
-        //   skinable resources they use.
-        wxGetApp().FireReloadSkin();
-
-        return true;
+    if (GetDefaultSkinName() == m_strSelectedSkin || GetDefaultBOINCSkinName() == m_strSelectedSkin) {
+        m_strSelectedSkin = GetDefaultSkinName();
     }
 
     // First we try the users canonical locale resources.
@@ -1313,8 +1301,8 @@ bool CSkinManager::ReloadSkin(wxLocale* pLocale, wxString strSkin) {
         }
     }
 
-    if (retval) {
-        fprintf(stderr, "Skin Manager: Failed to load skin '%s'.\n", (char *)ConstructSkinFileName().c_str());
+    if (retval && show_error_msgs) {
+        fprintf(stderr, "Skin Manager: Failed to load skin '%s'.\n", (const char *)ConstructSkinFileName().mb_str(wxConvUTF8));
     }
 
     InitializeDelayedValidation();
@@ -1364,6 +1352,11 @@ wxString CSkinManager::GetDefaultSkinName() {
 }
 
 
+wxString CSkinManager::GetDefaultBOINCSkinName() {
+    return wxString(wxT("BOINC"));
+}
+
+
 wxString CSkinManager::ConstructSkinFileName() {
     return wxString(
         GetSkinsLocation() + 
@@ -1463,4 +1456,3 @@ bool CSkinManager::InitializeDelayedValidation() {
            m_WizardsSkin.InitializeDelayedValidation();
 }
 
-
