@@ -32,11 +32,9 @@
 #include "SkinManager.h"
 #include "MainDocument.h"
 #include "BOINCBaseFrame.h"
-#include "hyperlink.h"
 #include "version.h"
-
 #include "sg_DlgMessages.h"
-#include "sg_SGUIListControl.h"
+#include "NoticeListCtrl.h"
 
 
 
@@ -63,10 +61,9 @@ IMPLEMENT_DYNAMIC_CLASS( CPanelMessages, wxPanel )
 
 BEGIN_EVENT_TABLE( CPanelMessages, wxPanel )
 ////@begin CPanelPreferences event table entries
+    EVT_NOTICELIST_ITEM_DISPLAY( CPanelMessages::OnLinkClicked )
     EVT_ERASE_BACKGROUND( CPanelMessages::OnEraseBackground )
     EVT_BUTTON( wxID_OK, CPanelMessages::OnOK )
-    EVT_BUTTON(ID_COPYAll, CPanelMessages::OnMessagesCopyAll)
-    EVT_BUTTON(ID_COPYSELECTED, CPanelMessages::OnMessagesCopySelected)
     EVT_BUTTON(ID_SIMPLE_HELP, CPanelMessages::OnButtonHelp)
 ////@end CPanelPreferences event table entries
 END_EVENT_TABLE()
@@ -94,8 +91,7 @@ CPanelMessages::CPanelMessages( wxWindow* parent ) :
 bool CPanelMessages::Create()
 {
 ////@begin CPanelMessages member initialisation
-    m_iPreviousDocCount = 0;
-	m_bProcessingRefreshEvent = false;
+    m_bProcessingRefreshEvent = false;
 ////@end CPanelMessages member initialisation
 
     CreateControls();
@@ -103,30 +99,12 @@ bool CPanelMessages::Create()
     GetSizer()->Fit(this);
     GetSizer()->SetSizeHints(this);
 
-	// Create List Pane Items
-    m_pList->InsertColumn(COLUMN_PROJECT, _("Project"), wxLIST_FORMAT_LEFT, 109);
-    m_pList->InsertColumn(COLUMN_TIME, _("Time"), wxLIST_FORMAT_LEFT, 130);
-    m_pList->InsertColumn(COLUMN_MESSAGE, _("Message"), wxLIST_FORMAT_LEFT, 378);
-
-    m_pMessageInfoAttr = new wxListItemAttr(*wxBLACK, *wxWHITE, wxNullFont);
-    m_pMessageErrorAttr = new wxListItemAttr(*wxRED, *wxWHITE, wxNullFont);
-
     return true;
 }
 
 
 CPanelMessages::~CPanelMessages()
 {
-	if (m_pMessageInfoAttr) {
-        delete m_pMessageInfoAttr;
-        m_pMessageInfoAttr = NULL;
-    }
-
-    if (m_pMessageErrorAttr) {
-        delete m_pMessageErrorAttr;
-        m_pMessageErrorAttr = NULL;
-    }
-
 }
 
 
@@ -138,17 +116,20 @@ void CPanelMessages::CreateControls()
 {
     CPanelMessages* itemDialog1 = this;
     CSkinSimple* pSkinSimple = wxGetApp().GetSkinManager()->GetSimple();
+    CSkinAdvanced*     pSkinAdvanced = wxGetApp().GetSkinManager()->GetAdvanced();
 
     wxASSERT(pSkinSimple);
     wxASSERT(wxDynamicCast(pSkinSimple, CSkinSimple));
+    wxASSERT(pSkinAdvanced);
+    wxASSERT(wxDynamicCast(pSkinAdvanced, CSkinAdvanced));
 
     wxFlexGridSizer* itemFlexGridSizer2 = new wxFlexGridSizer(2, 1, 0, 0);
     itemFlexGridSizer2->AddGrowableRow(0);
     itemFlexGridSizer2->AddGrowableCol(0);
     itemDialog1->SetSizer(itemFlexGridSizer2);
 
-    m_pList = new CSGUIListCtrl(this, ID_SIMPLE_MESSAGESVIEW, DEFAULT_LIST_MULTI_SEL_FLAGS);
-    itemFlexGridSizer2->Add(m_pList, 0, wxGROW|wxALL, 5);
+    m_pHtmlListPane = new CNoticeListCtrl(this);
+    itemFlexGridSizer2->Add(m_pHtmlListPane, 0, wxGROW|wxALL, 5);
 
     wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxHORIZONTAL);
     
@@ -158,64 +139,6 @@ void CPanelMessages::CreateControls()
     itemFlexGridSizer2->Add(itemBoxSizer4, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 #endif
 
-#ifdef wxUSE_CLIPBOARD
-    wxBitmapButton* itemButton1 = new wxBitmapButton(
-        this,
-        ID_COPYAll,
-        *pSkinSimple->GetCopyAllButton()->GetBitmap(),
-        wxDefaultPosition,
-        wxSize(
-            (*pSkinSimple->GetCopyAllButton()->GetBitmap()).GetWidth(),
-            (*pSkinSimple->GetCopyAllButton()->GetBitmap()).GetHeight()
-        ),
-        wxBU_AUTODRAW
-    );
-	if ( pSkinSimple->GetCopyAllButton()->GetBitmapClicked() != NULL ) {
-		itemButton1->SetBitmapSelected(*pSkinSimple->GetCopyAllButton()->GetBitmapClicked());
-	}
-    itemButton1->SetHelpText(
-        _("Copy all the messages to the clipboard.")
-    );
-#if wxUSE_TOOLTIPS
-    itemButton1->SetToolTip(
-        _("Copy all the messages to the clipboard.")
-    );
-#endif
-    itemBoxSizer4->Add(itemButton1, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
-
-    wxBitmapButton* itemButton2 = new wxBitmapButton(
-        this,
-        ID_COPYSELECTED,
-        *pSkinSimple->GetCopyButton()->GetBitmap(),
-        wxDefaultPosition,
-        wxSize(
-            (*pSkinSimple->GetCopyButton()->GetBitmap()).GetWidth(),
-            (*pSkinSimple->GetCopyButton()->GetBitmap()).GetHeight()
-        ),
-        wxBU_AUTODRAW
-    );
-	if ( pSkinSimple->GetCopyButton()->GetBitmapClicked() != NULL ) {
-		itemButton2->SetBitmapSelected(*pSkinSimple->GetCopyButton()->GetBitmapClicked());
-	}
-    itemButton2->SetHelpText(
-#ifdef __WXMAC__
-        _("Copy the selected messages to the clipboard. You can select multiple messages by holding down the shift or command key while clicking on messages.")
-#else
-        _("Copy the selected messages to the clipboard. You can select multiple messages by holding down the shift or control key while clicking on messages.")
-#endif
-    );
-#if wxUSE_TOOLTIPS
-    itemButton2->SetToolTip(
-#ifdef __WXMAC__
-        _("Copy the selected messages to the clipboard. You can select multiple messages by holding down the shift or command key while clicking on messages.")
-#else
-        _("Copy the selected messages to the clipboard. You can select multiple messages by holding down the shift or control key while clicking on messages.")
-#endif
-    );
-#endif
-    itemBoxSizer4->Add(itemButton2, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
-#endif
-
     wxBitmapButton* itemBitmapButton44 = new wxBitmapButton(
         this,
         wxID_OK,
@@ -249,7 +172,9 @@ void CPanelMessages::CreateControls()
 		itemButton45->SetBitmapSelected(*pSkinSimple->GetHelpButton()->GetBitmapClicked());
 	}
 #ifdef wxUSE_TOOLTIPS
-	itemButton45->SetToolTip(new wxToolTip(_("Get help with BOINC")));
+    wxString helpTip;
+    helpTip.Printf(_("Get help with %s"), pSkinAdvanced->GetApplicationShortName().c_str());
+    itemButton45->SetToolTip(helpTip);
 #endif
     itemBoxSizer4->Add(itemButton45, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 #else
@@ -325,49 +250,12 @@ void CPanelMessages::OnEraseBackground(wxEraseEvent& event){
  */
 
 void CPanelMessages::OnRefresh() {
-    bool isConnected;
-    static bool was_connected = false;
-    
     if (!m_bProcessingRefreshEvent) {
         m_bProcessingRefreshEvent = true;
 
-        wxASSERT(m_pList);
-
-        wxInt32 iDocCount = wxGetApp().GetDocument()->GetMessageCount();
-        if (0 >= iDocCount) {
-            m_pList->DeleteAllItems();
-        } else {
-            // If connection status changed, adjust color of messages display
-            isConnected = wxGetApp().GetDocument()->IsConnected();
-            if (was_connected != isConnected) {
-                was_connected = isConnected;
-                if (isConnected) {
-                    m_pMessageInfoAttr->SetTextColour(*wxBLACK);
-                    m_pMessageErrorAttr->SetTextColour(*wxRED);
-                } else {
-                    wxColourDatabase colorBase;
-                    m_pMessageInfoAttr->SetTextColour(wxColour(128, 128, 128));
-                    m_pMessageErrorAttr->SetTextColour(wxColour(255, 128, 128));
-                }
-                // Force a complete update
-                m_pList->DeleteAllItems();
-                m_pList->SetItemCount(iDocCount);
-                 m_iPreviousDocCount = 0;    // Force scrolling to bottom
-            } else {
-                // Connection status didn't change
-                if (m_iPreviousDocCount != iDocCount) {
-                    m_pList->SetItemCount(iDocCount);
-                }
-            }
-        }
-
-        if ((iDocCount > 1) && (EnsureLastItemVisible()) && (m_iPreviousDocCount != iDocCount)) {
-            m_pList->EnsureVisible(iDocCount - 1);
-        }
-
-        if (m_iPreviousDocCount != iDocCount) {
-            m_iPreviousDocCount = iDocCount;
-        }
+        m_pHtmlListPane->Freeze();
+        m_pHtmlListPane->UpdateUI();
+        m_pHtmlListPane->Thaw();
 
         m_bProcessingRefreshEvent = false;
     }
@@ -383,58 +271,6 @@ void CPanelMessages::OnOK( wxCommandEvent& event ) {
 }
 
 
-/*!
- * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_COPYAll
- */
-
-void CPanelMessages::OnMessagesCopyAll( wxCommandEvent& WXUNUSED(event) ) {
-    wxLogTrace(wxT("Function Start/End"), wxT("CPanelMessages::OnMessagesCopyAll - Function Begin"));
-
-#ifdef wxUSE_CLIPBOARD
-    wxInt32 iIndex          = -1;
-    wxInt32 iRowCount       = 0;
-    OpenClipboard();
-
-    iRowCount = m_pList->GetItemCount();
-    for (iIndex = 0; iIndex < iRowCount; iIndex++) {
-        CopyToClipboard(iIndex);            
-    }
-
-    CloseClipboard();
-#endif
-
-    wxLogTrace(wxT("Function Start/End"), wxT("CPanelMessages::OnMessagesCopyAll - Function End"));
-}
-
-
-/*!
- * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_COPYSELECTED
- */
-
-void CPanelMessages::OnMessagesCopySelected( wxCommandEvent& WXUNUSED(event) ) {
-    wxLogTrace(wxT("Function Start/End"), wxT("CPanelMessages::OnMessagesCopySelected - Function Begin"));
-
-#ifdef wxUSE_CLIPBOARD
-    wxInt32 iIndex = -1;
-
-    OpenClipboard();
-
-    for (;;) {
-        iIndex = m_pList->GetNextItem(
-            iIndex, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED
-        );
-        if (iIndex == -1) break;
-
-        CopyToClipboard(iIndex);            
-    }
-
-    CloseClipboard();
-#endif
-
-    wxLogTrace(wxT("Function Start/End"), wxT("CPanelMessages::OnMessagesCopySelected - Function End"));
-}
-
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_SIMPLE_HELP
  */
@@ -452,251 +288,34 @@ void CPanelMessages::OnButtonHelp( wxCommandEvent& event ) {
             wxString(BOINC_VERSION_STRING, wxConvUTF8).c_str(),
             event.GetId()
         );
-        wxGetApp().GetFrame()->ExecuteBrowserLink(wxurl);
+        wxLaunchDefaultBrowser(wxurl);
     }
 
     wxLogTrace(wxT("Function Start/End"), wxT("CPanelMessages::OnHelp - Function End"));
 }
 
 
-bool CPanelMessages::OnSaveState(wxConfigBase* pConfig) {
-    wxString    strBaseConfigLocation = wxEmptyString;
-    wxListItem  liColumnInfo;
-    wxInt32     iIndex = 0;
-    wxInt32     iColumnCount = 0;
+/*!
+ * wxEVT_NOTICELIST_ITEM_DISPLAY event handler for ID_LIST_NOTIFICATIONSVIEW
+ */
 
-
-    wxASSERT(pConfig);
-
-
-    // Retrieve the base location to store configuration information
-    // Should be in the following form: "/Projects/"
-    strBaseConfigLocation = pConfig->GetPath() + wxT("/");
-
-    // Convert to a zero based index
-    iColumnCount = m_pList->GetColumnCount() - 1;
-
-    // Which fields are we interested in?
-    liColumnInfo.SetMask(
-        wxLIST_MASK_TEXT |
-        wxLIST_MASK_WIDTH |
-        wxLIST_MASK_FORMAT
-    );
-
-    // Cycle through the columns recording anything interesting
-    for (iIndex = 0; iIndex <= iColumnCount; iIndex++) {
-        m_pList->GetColumn(iIndex, liColumnInfo);
-
-        pConfig->SetPath(strBaseConfigLocation + liColumnInfo.GetText());
-
-        pConfig->Write(wxT("Width"), liColumnInfo.GetWidth());
-        
-#if (defined(__WXMAC__) &&  wxCHECK_VERSION(2,8,0))
-        pConfig->Write(wxT("Width"), m_pList->GetColumnWidth(iIndex)); // Work around bug in wxMac-2.8.0 wxListCtrl::SetColumn()
-#endif
+void CPanelMessages::OnLinkClicked( NoticeListCtrlEvent& event ) {
+    if (event.GetURL().StartsWith(wxT("http://"))) {
+		wxLaunchDefaultBrowser(event.GetURL());
     }
+}
 
 
+bool CPanelMessages::OnSaveState(wxConfigBase* /* pConfig */) {
     return true;
 }
 
 
-bool CPanelMessages::OnRestoreState(wxConfigBase* pConfig) {
-    wxString    strBaseConfigLocation = wxEmptyString;
-    wxListItem  liColumnInfo;
-    wxInt32     iIndex = 0;
-    wxInt32     iColumnCount = 0;
-    wxInt32     iTempValue = 0;
-
-
-    wxASSERT(pConfig);
-
-
-    // Retrieve the base location to store configuration information
-    // Should be in the following form: "/Projects/"
-    strBaseConfigLocation = pConfig->GetPath() + wxT("/");
-
-    // Convert to a zero based index
-    iColumnCount = m_pList->GetColumnCount() - 1;
-
-    // Which fields are we interested in?
-    liColumnInfo.SetMask(
-        wxLIST_MASK_TEXT |
-        wxLIST_MASK_WIDTH |
-        wxLIST_MASK_FORMAT
-    );
-
-    // Cycle through the columns recording anything interesting
-    for (iIndex = 0; iIndex <= iColumnCount; iIndex++) {
-        m_pList->GetColumn(iIndex, liColumnInfo);
-
-        pConfig->SetPath(strBaseConfigLocation + liColumnInfo.GetText());
-
-        pConfig->Read(wxT("Width"), &iTempValue, -1);
-        if (-1 != iTempValue) {
-            liColumnInfo.SetWidth(iTempValue);
-#if (defined(__WXMAC__) &&  wxCHECK_VERSION(2,8,0))
-            m_pList->SetColumnWidth(iIndex,iTempValue); // Work around bug in wxMac-2.8.0 wxListCtrl::SetColumn()
-#endif
-        }
-
-        m_pList->SetColumn(iIndex, liColumnInfo);
-    }
-
+bool CPanelMessages::OnRestoreState(wxConfigBase* /* pConfig */) {
     return true;
 }
 
 
-wxString CPanelMessages::OnListGetItemText(long item, long column) const {
-    wxString        strBuffer   = wxEmptyString;
-
-    switch(column) {
-    case COLUMN_PROJECT:
-        FormatProjectName(item, strBuffer);
-        break;
-    case COLUMN_TIME:
-        FormatTime(item, strBuffer);
-        break;
-    case COLUMN_MESSAGE:
-        FormatMessage(item, strBuffer);
-        break;
-    }
-
-    return strBuffer;
-}
-
-
-wxListItemAttr* CPanelMessages::OnListGetItemAttr(long item) const {
-    wxListItemAttr* pAttribute  = NULL;
-    MESSAGE* message = wxGetApp().GetDocument()->message(item);
-
-    if (message) {
-        switch(message->priority) {
-        case MSG_USER_ERROR:
-            pAttribute = m_pMessageErrorAttr;
-            break;
-        default:
-            pAttribute = m_pMessageInfoAttr;
-            break;
-        }
-    }
-
-    return pAttribute;
-	
-}
-
-
-bool CPanelMessages::EnsureLastItemVisible() {
-    int numVisible = m_pList->GetCountPerPage();
-
-    // Auto-scroll only if already at bottom of list
-    if ((m_iPreviousDocCount > numVisible)
-         && ((m_pList->GetTopItem() + numVisible) < (m_iPreviousDocCount-1)) 
-    ) {
-        return false;
-    }
-    
-    return true;
-}
-
-
-wxInt32 CPanelMessages::FormatProjectName(wxInt32 item, wxString& strBuffer) const {
-    MESSAGE* message = wxGetApp().GetDocument()->message(item);
-
-    if (message) {
-        strBuffer = wxString(message->project.c_str(), wxConvUTF8);
-    }
-
-    return 0;
-}
-
-
-wxInt32 CPanelMessages::FormatTime(wxInt32 item, wxString& strBuffer) const {
-    wxDateTime dtBuffer;
-    MESSAGE*   message = wxGetApp().GetDocument()->message(item);
-
-    if (message) {
-        dtBuffer.Set((time_t)message->timestamp);
-        strBuffer = dtBuffer.Format();
-    }
-
-    return 0;
-}
-
-
-wxInt32 CPanelMessages::FormatMessage(wxInt32 item, wxString& strBuffer) const {
-    MESSAGE*   message = wxGetApp().GetDocument()->message(item);
-
-    if (message) {
-        strBuffer = wxString(message->body.c_str(), wxConvUTF8);
-    }
-
-    strBuffer.Replace(wxT("\n"), wxT(""), true);
-
-    return 0;
-}
-
-
-#ifdef wxUSE_CLIPBOARD
-bool CPanelMessages::OpenClipboard() {
-    bool bRetVal = false;
-
-    bRetVal = wxTheClipboard->Open();
-    if (bRetVal) {
-        m_bClipboardOpen = true;
-        m_strClipboardData = wxEmptyString;
-        wxTheClipboard->Clear();
-    }
-
-    return bRetVal;
-}
-
-
-wxInt32 CPanelMessages::CopyToClipboard(wxInt32 item) {
-    wxInt32        iRetVal = -1;
-
-    if (m_bClipboardOpen) {
-        wxString       strBuffer = wxEmptyString;
-        wxString       strTimeStamp = wxEmptyString;
-        wxString       strProject = wxEmptyString;
-        wxString       strMessage = wxEmptyString;
-
-        FormatTime(item, strTimeStamp);
-        FormatProjectName(item, strProject);
-        FormatMessage(item, strMessage);
-
-#ifdef __WXMSW__
-        strBuffer.Printf(wxT("%s|%s|%s\r\n"), strTimeStamp.c_str(), strProject.c_str(), strMessage.c_str());
-#else
-        strBuffer.Printf(wxT("%s|%s|%s\n"), strTimeStamp.c_str(), strProject.c_str(), strMessage.c_str());
-#endif
-
-        m_strClipboardData += strBuffer;
-
-        iRetVal = 0;
-    }
-
-    return iRetVal;
-}
-
-
-bool CPanelMessages::CloseClipboard() {
-    bool bRetVal = false;
-
-    if (m_bClipboardOpen) {
-        wxTheClipboard->SetData(new wxTextDataObject(m_strClipboardData));
-        wxTheClipboard->Close();
-
-        m_bClipboardOpen = false;
-        m_strClipboardData = wxEmptyString;
-    }
-
-    return bRetVal;
-}
-
-#endif
-
-
 /*!
  * CDlgMessages type definition
  */
@@ -712,6 +331,8 @@ BEGIN_EVENT_TABLE( CDlgMessages, wxDialog )
     EVT_HELP(wxID_ANY, CDlgMessages::OnHelp)
     EVT_SHOW( CDlgMessages::OnShow )
     EVT_BUTTON( wxID_OK, CDlgMessages::OnOK )
+	EVT_SIZE(CDlgMessages::OnSize)
+    EVT_MOVE(CDlgMessages::OnMove)
 ////@end CDlgMessages event table entries
 END_EVENT_TABLE()
 
@@ -745,23 +366,31 @@ CDlgMessages::~CDlgMessages() {
 
 bool CDlgMessages::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
 {
-    wxString strCaption = caption;
-    if (strCaption.IsEmpty()) {
-        CSkinAdvanced*         pSkinAdvanced = wxGetApp().GetSkinManager()->GetAdvanced();
-        wxASSERT(pSkinAdvanced);
-        wxASSERT(wxDynamicCast(pSkinAdvanced, CSkinAdvanced));
-
-        strCaption.Printf(_("%s - Messages"), pSkinAdvanced->GetApplicationName().c_str());
-    }
+    CSkinAdvanced*         pSkinAdvanced = wxGetApp().GetSkinManager()->GetAdvanced();
+    wxASSERT(pSkinAdvanced);
+    wxASSERT(wxDynamicCast(pSkinAdvanced, CSkinAdvanced));
 
+        
     SetExtraStyle(GetExtraStyle()|wxDIALOG_EX_CONTEXTHELP|wxWS_EX_BLOCK_EVENTS);
 
-    wxDialog::Create( parent, id, strCaption, pos, size, style );
+    wxDialog::Create( parent, id, caption, pos, size, style );
 
-    SetBackgroundStyle(wxBG_STYLE_CUSTOM);
+    wxString strCaption = caption;
+    if (strCaption.IsEmpty()) {
+        strCaption.Printf(_("%s - Notices"), pSkinAdvanced->GetApplicationName().c_str());
+    }
+    SetTitle(strCaption);
+
+    // Initialize Application Icon
+    wxIconBundle icons;
+    icons.AddIcon(*pSkinAdvanced->GetApplicationIcon());
+    icons.AddIcon(*pSkinAdvanced->GetApplicationIcon32());
+    SetIcons(icons);
 
     Freeze();
 
+    SetBackgroundStyle(wxBG_STYLE_CUSTOM);
+
 #ifdef __WXDEBUG__
     SetBackgroundColour(wxColour(255, 0, 255));
 #endif
@@ -844,7 +473,7 @@ void CDlgMessages::OnHelp(wxHelpEvent& event) {
             wxString(BOINC_VERSION_STRING, wxConvUTF8).c_str(),
             event.GetId()
         );
-        wxGetApp().GetFrame()->ExecuteBrowserLink(wxurl);
+        wxLaunchDefaultBrowser(wxurl);
     }
 
     wxLogTrace(wxT("Function Start/End"), wxT("CDlgMessages::OnHelp - Function End"));
@@ -899,15 +528,18 @@ void CDlgMessages::SaveWindowDimensions() {
 
     pConfig->SetPath(strBaseConfigLocation);
 
-    pConfig->Write(wxT("WindowIconized"), IsIconized());
+    bool iconized = IsIconized();
+    pConfig->Write(wxT("WindowIconized"), iconized);
     pConfig->Write(wxT("WindowMaximized"), IsMaximized());
-    pConfig->Write(wxT("Width"), GetSize().GetWidth());
-    pConfig->Write(wxT("Height"), GetSize().GetHeight());
+    if (!iconized) {
+        pConfig->Write(wxT("Width"), GetSize().GetWidth());
+        pConfig->Write(wxT("Height"), GetSize().GetHeight());
 
 #ifdef __WXMAC__
-    pConfig->Write(wxT("XPos"), GetPosition().x);
-    pConfig->Write(wxT("YPos"), GetPosition().y);
+        pConfig->Write(wxT("XPos"), GetPosition().x);
+        pConfig->Write(wxT("YPos"), GetPosition().y);
 #endif  // ! __WXMAC__
+    }
 }
     
 
@@ -993,3 +625,17 @@ void CDlgMessages::RestoreWindowDimensions() {
 #endif  // ! __WXMAC__
 }
 
+void CDlgMessages::OnSize(wxSizeEvent& event) {
+    if (IsShown()) {
+        SaveWindowDimensions();
+    }
+    event.Skip();
+}
+
+
+void CDlgMessages::OnMove(wxMoveEvent& event) {
+    if (IsShown()) {
+        SaveWindowDimensions();
+    }
+    event.Skip();
+}
