@@ -21,22 +21,18 @@
 
 #ifdef _WIN32
 
-// Modify the following defines if you have to target a platform prior to the ones specified below.
-// Refer to MSDN for the latest info on corresponding values for different platforms.
-#ifndef WINVER                  // Allow use of features specific to Windows 95 and Windows NT 4 or later.
-#define WINVER 0x0400           // Change this to the appropriate value to target Windows 98 and Windows 2000 or later.
+// Target Windows 2000 or better with Internet Explorer 5.01 or better
+#ifndef WINVER
+#define WINVER 0x0500
 #endif
-
-#ifndef _WIN32_WINNT            // Allow use of features specific to Windows NT 4 or later.
-#define _WIN32_WINNT 0x0400		// Change this to the appropriate value to target Windows 98 and Windows 2000 or later.
+#ifndef _WIN32_WINNT
+#define _WIN32_WINNT 0x0500
 #endif
-
-#ifndef _WIN32_WINDOWS          // Allow use of features specific to Windows 98 or later.
-#define _WIN32_WINDOWS 0x0400   // Change this to the appropriate value to target Windows Me or later.
+#ifndef _WIN32_WINDOWS
+#define _WIN32_WINDOWS 0x0500
 #endif
-
-#ifndef _WIN32_IE               // Allow use of features specific to IE 5.01 or later.
-#define _WIN32_IE 0x0501        // Change this to the appropriate value to target IE 6.0 or later.
+#ifndef _WIN32_IE
+#define _WIN32_IE 0x0501
 #endif
 
 #endif
@@ -115,10 +111,12 @@
 #include <wx/fs_mem.h>
 #include <wx/dnd.h>
 #include <wx/htmllbox.h>
-
+#include <wx/hyperlink.h>
+#include <wx/mstream.h>
+#include <wx/hash.h>
+#include <wx/selstore.h>
 
 #ifdef _WIN32
-
 // Visual Studio 2005 has extended the C Run-Time Library by including "secure"
 // runtime functions and deprecating the previous function prototypes.  Since
 // we need to use the previous prototypes to maintain compatibility with other
