@@ -90,7 +90,7 @@ protected:
     void            setSSMessageText(const char *msg);
     void            updateSSMessageText(char *msg);
     void            strip_cr(char *buf);
-    char            m_gfx_Switcher_Path[MAXPATHLEN];
+    char            m_gfx_Switcher_Path[PATH_MAX];
     bool            m_bErrorMode;        // Whether to draw moving logo and possibly display an error
     unsigned int    m_hrError;           // Error code to display
 
@@ -141,7 +141,8 @@ protected:
     RESULTS         results;
  
     bool            m_bResetCoreState;
-    bool            m_QuitDataManagementProc;
+    bool            m_bQuitDataManagementProc;
+    bool            m_bDataManagementProcStopped;
     bool            m_bV5_GFX_app_is_running;
 
 
