@@ -231,7 +231,8 @@ CScreensaver::CScreensaver() {
     m_CoreClientPID = nil;
     setSSMessageText(0);
     m_CurrentBannerMessage = 0;
-    m_QuitDataManagementProc = false;
+    m_bQuitDataManagementProc = false;
+    m_bDataManagementProcStopped = false;
     m_BrandText = "BOINC";
     
     m_hDataManagementThread = NULL;
@@ -296,7 +297,8 @@ int CScreensaver::Create() {
         if (saverState == SaverState_LaunchingCoreClient)
         {
             SetError(FALSE, 0);
-            m_QuitDataManagementProc = false;
+            m_bQuitDataManagementProc = false;
+            m_bDataManagementProcStopped = false;
             if (rpc == NULL) {
                 rpc = new RPC_CLIENT;
             }
@@ -341,8 +343,13 @@ OSStatus CScreensaver::initBOINCApp() {
     if (++retryCount > 3)   // Limit to 3 relaunches to prevent thrashing
         return -1;
 
+#ifdef _DEBUG
+    err = -1;
+#else
     err = GetpathToBOINCManagerApp(boincPath, sizeof(boincPath));
-    if (err) {   // If we couldn't find BOINCManager.app, try default path
+#endif
+   if (err) 
+    {   // If we couldn't find BOINCManager.app, try default path
         strcpy(boincPath, "/Applications/");
         if (brandId)
             strcat(boincPath, m_BrandText);
@@ -558,7 +565,7 @@ void CScreensaver::ShutdownSaver() {
     m_CoreClientPID = 0;
 //    gQuitCounter = 0;
     m_wasAlreadyRunning = false;
-    m_QuitDataManagementProc = false;
+    m_bQuitDataManagementProc = false;
     saverState = SaverState_Idle;
 }
 
@@ -599,7 +606,7 @@ void CScreensaver::HandleRPCError() {
     // If Core Client is hung, it might cause RPCs to hang, preventing us from 
     // shutting down the Data Management Thread, so don't reinitialize the RPC 
     // client if we have told the Data Management Thread to exit.
-    if (m_QuitDataManagementProc) {
+    if (m_bQuitDataManagementProc) {
         return;
     }
     
@@ -620,7 +627,7 @@ bool CScreensaver::CreateDataManagementThread() {
         m_dwBlankTime = time(0) + (gBlankingTime * 60);
     else
         m_dwBlankTime = 0;
-    
+
     if (m_hDataManagementThread == NULL) {
         retval = pthread_create(&m_hDataManagementThread, NULL, DataManagementProcStub, 0);
         if (retval) {
@@ -634,20 +641,23 @@ bool CScreensaver::CreateDataManagementThread() {
 
 
 bool CScreensaver::DestroyDataManagementThread() {
-    int i;
+    m_bQuitDataManagementProc = true;  // Tell DataManagementProc thread to exit
+    if (!m_hDataManagementThread) return true;
     
-    m_QuitDataManagementProc = true;  // Tell DataManagementProc thread to exit
-    
-    for (i=0; i<10; i++) {  // Wait up to 1 second for DataManagementProc thread to exit
-        if (m_hDataManagementThread == NULL) return true;
+    for (int i=0; i<10; i++) {  // Wait up to 1 second for DataManagementProc thread to exit
+        if (m_bDataManagementProcStopped) return true;
         boinc_sleep(0.1);
     }
+
+    if (rpc) {
     rpc->close();    // In case DataManagementProc is hung waiting for RPC
+    }
     m_hDataManagementThread = NULL; // Don't delay more if this routine is called again.
     if (m_hGraphicsApplication) {
         terminate_screensaver(m_hGraphicsApplication, NULL);
         m_hGraphicsApplication = 0;
     }
+
     return true;
 }
 
@@ -707,20 +717,22 @@ int CScreensaver::GetBrandID()
 
     iBrandId = 0;   // Default value
     
-    err = GetpathToBOINCManagerApp(buf, sizeof(buf));
-    if (err) {     
-        // If we couldn't find our application bundle, look in BOINC Data Directory 
-        // (the installer put a copy there for us)
-        strcpy(buf, "/Library/Application Support/BOINC Data/Branding");
-    } else
-        strcat(buf, "/Contents/Resources/Branding");
-
-    FILE *f = fopen(buf, "r");
+    // The installer put a copy of Branding file in the BOINC Data Directory
+    FILE *f = fopen("/Library/Application Support/BOINC Data/Branding", "r");
+    if (f == NULL) {
+       // If we couldn't find our Branding file in the BOINC Data Directory,  
+       // look in our application bundle
+        err = GetpathToBOINCManagerApp(buf, sizeof(buf));
+        if (err == noErr) {
+            strcat(buf, "/Contents/Resources/Branding");
+            f = fopen(buf, "r");
+        }
+    }
     if (f) {
         fscanf(f, "BrandId=%ld\n", &iBrandId);
         fclose(f);
     }
-    
+        
     return iBrandId;
 }
 
@@ -863,4 +875,3 @@ void PrintBacktrace(void) {
 // Dummy routine to satisfy linker
 }
 
-const char *BOINC_RCSID_7ce0778d35="$Id: mac_saver_module.cpp 19891 2009-12-14 13:45:08Z charlief $";
