@@ -61,25 +61,6 @@ INT WINAPI WinMain(
     WSADATA      wsdata;
 
 
-#ifdef _DEBUG
-    // Initialize Diagnostics
-    retval = diagnostics_init (
-        BOINC_DIAG_DUMPCALLSTACKENABLED | 
-        BOINC_DIAG_HEAPCHECKENABLED |
-        BOINC_DIAG_MEMORYLEAKCHECKENABLED |
-        BOINC_DIAG_ARCHIVESTDOUT |
-        BOINC_DIAG_REDIRECTSTDOUTOVERWRITE |
-        BOINC_DIAG_REDIRECTSTDERROVERWRITE |
-        BOINC_DIAG_TRACETOSTDOUT,
-        "stdoutscr",
-        "stderrscr"
-    );
-    if (retval) {
-        BOINCTRACE("WinMain - BOINC Screensaver Diagnostic Error '%d'\n", retval);
-        MessageBox(NULL, NULL, "BOINC Screensaver Diagnostic Error", MB_OK);
-    }
-#endif
-
     // Initialize the CRT random number generator.
     srand((unsigned int)time(0));
 
@@ -103,7 +84,7 @@ INT WINAPI WinMain(
     }
 
     retval = BOINCSS.Run();
-
+    
     // Cleanup any existing screensaver objects and handles
     BOINCSS.Cleanup();
 
@@ -146,7 +127,8 @@ CScreensaver::CScreensaver() {
     m_hDataManagementThread = NULL;
     m_hGraphicsApplication = NULL;
     m_bResetCoreState = TRUE;
-    m_QuitDataManagementProc = FALSE;
+    m_bQuitDataManagementProc = FALSE;
+    m_bDataManagementProcStopped = FALSE;
     memset(&m_running_result, 0, sizeof(m_running_result));
 
     ZeroMemory(m_Monitors, sizeof(m_Monitors));
@@ -161,10 +143,39 @@ CScreensaver::CScreensaver() {
 //
 HRESULT CScreensaver::Create(HINSTANCE hInstance) {
     HRESULT hr;
-    BOOL    bReturnValue;
-    struct ss_periods periods;
+    int     retval;
+    struct  ss_periods periods;
+
     m_hInstance = hInstance;
 
+
+    // Retrieve the locations of the install directory and data directory
+	UtilGetRegDirectoryStr(_T("DATADIR"), m_strBOINCDataDirectory);
+	UtilGetRegDirectoryStr(_T("INSTALLDIR"), m_strBOINCInstallDirectory);
+
+    if (!m_strBOINCDataDirectory.empty()) {
+        SetCurrentDirectory(m_strBOINCDataDirectory.c_str());
+    }
+
+    // Initialize Diagnostics
+    retval = diagnostics_init (
+#ifdef _DEBUG
+        BOINC_DIAG_HEAPCHECKENABLED |
+        BOINC_DIAG_MEMORYLEAKCHECKENABLED |
+#endif
+        BOINC_DIAG_DUMPCALLSTACKENABLED | 
+        BOINC_DIAG_ARCHIVESTDOUT |
+        BOINC_DIAG_REDIRECTSTDOUTOVERWRITE |
+        BOINC_DIAG_REDIRECTSTDERROVERWRITE |
+        BOINC_DIAG_TRACETOSTDOUT,
+        "stdoutscr",
+        "stderrscr"
+    );
+    if (retval) {
+        BOINCTRACE("WinMain - BOINC Screensaver Diagnostic Error '%d'\n", retval);
+        MessageBox(NULL, NULL, "BOINC Screensaver Diagnostic Error", MB_OK);
+    }
+
     // Parse the command line and do the appropriate thing
     m_SaverMode = ParseCommandLine(GetCommandLine());
 
@@ -183,12 +194,7 @@ HRESULT CScreensaver::Create(HINSTANCE hInstance) {
     // Get project-defined default values for GFXDefaultPeriod, GFXSciencePeriod, GFXChangePeriod
     GetDefaultDisplayPeriods(periods);
     m_bShow_default_ss_first = periods.Show_default_ss_first;
-
-    // Retrieve the locations of the install directory and data directory
-	bReturnValue = UtilGetRegDirectoryStr(_T("DATADIR"), m_strBOINCDataDirectory);
-	bReturnValue = UtilGetRegDirectoryStr(_T("INSTALLDIR"), m_strBOINCInstallDirectory);
-
-
+        
     // Get the last set of saved values, if not set
     // use the configuration file, if not set, use defaults.
     // Normalize on Seconds...
@@ -246,7 +252,7 @@ HRESULT CScreensaver::Create(HINSTANCE hInstance) {
     if (m_dwBlankTime > 0) {
         m_dwBlankTime = (DWORD)time(0) + m_dwBlankTime;
     }
-
+    
     // Create the infrastructure mutexes so we can properly aquire them to report
     //   errors
     if (!CreateInfrastructureMutexes()) {
@@ -255,7 +261,7 @@ HRESULT CScreensaver::Create(HINSTANCE hInstance) {
 
 	if (rpc == NULL) rpc = new RPC_CLIENT;
 
-			// Create the screen saver window(s)
+    // Create the screen saver window(s)
     if (m_SaverMode == sm_preview || 
         m_SaverMode == sm_full
     ) {
@@ -344,9 +350,6 @@ HRESULT CScreensaver::Run() {
             DestroyGraphicsWindowPromotionThread();
             DestroyInputActivityThread();
             break;
-        case sm_passwordchange:
-            ChangePassword();
-            break;
     }
     return S_OK;
 }
@@ -361,8 +364,8 @@ HRESULT CScreensaver::Cleanup() {
         TerminateProcess(m_hGraphicsApplication, 0);
         m_hGraphicsApplication = NULL;
     }
-
     if (rpc) {
+        rpc->close();
         delete rpc;
         rpc = NULL;
     }
@@ -821,7 +824,9 @@ BOOL CScreensaver::GetTextForError(
 //
 BOOL CScreensaver::CreateInputActivityThread() {
     DWORD dwThreadID = 0;
+
     BOINCTRACE(_T("CScreensaver::CreateInputActivityThread Start\n"));
+
     m_hInputActivityThread = CreateThread(
         NULL,                        // default security attributes 
         0,                           // use default stack size  
@@ -845,10 +850,13 @@ BOOL CScreensaver::CreateInputActivityThread() {
 // Terminate the thread that is used to monitor input activity.
 //
 BOOL CScreensaver::DestroyInputActivityThread() {
+   	BOINCTRACE(_T("CScreensaver::DestroyInputActivityThread: Shutting Down...\n"));
+
     if (!TerminateThread(m_hInputActivityThread, 0)) {
     	BOINCTRACE(_T("CScreensaver::DestroyInputActivityThread: Failed to terminate input activity thread '%d'\n"), GetLastError());
         return FALSE;
     }
+
     return TRUE;
 }
 
@@ -879,15 +887,31 @@ DWORD WINAPI CScreensaver::InputActivityProcStub(LPVOID UNUSED(lpParam)) {
 //
 DWORD WINAPI CScreensaver::InputActivityProc() {
     LASTINPUTINFO lii;
+    DWORD         dwCounter = 0;
     lii.cbSize = sizeof(LASTINPUTINFO);
 
+    BOINCTRACE(_T("CScreensaver::InputActivityProc - Last Input Activity '%d'.\n"), m_dwLastInputTimeAtStartup);
+
     while(true) {
-        GetLastInputInfo(&lii);
-        if (m_dwLastInputTimeAtStartup != lii.dwTime) {
-            BOINCTRACE(_T("CScreensaver::InputActivityProc - Activity Detected.\n"));
+        if (GetLastInputInfo(&lii)) {
+            if (dwCounter > 4) {
+                BOINCTRACE(_T("CScreensaver::InputActivityProc - Heartbeat.\n"));
+                dwCounter = 0;
+            }
+            if (m_dwLastInputTimeAtStartup != lii.dwTime) {
+                BOINCTRACE(_T("CScreensaver::InputActivityProc - Activity Detected.\n"));
+                SetError(TRUE, SCRAPPERR_BOINCSHUTDOWNEVENT);
+                FireInterruptSaverEvent();
+            }
+        } else {
+            BOINCTRACE(_T("CScreensaver::InputActivityProc - Failed to detect input activity.\n"));
+            fprintf(stdout, _T("Screen saver shutdown due to not being able to detect input activity.\n"));
+            fprintf(stdout, _T("Try rebooting, if the problem persists contact the BOINC.\n"));
+            fprintf(stdout, _T("development team.\n\n"));
             SetError(TRUE, SCRAPPERR_BOINCSHUTDOWNEVENT);
             FireInterruptSaverEvent();
         }
+        dwCounter++;
         boinc_sleep(0.25);
     }
 }
@@ -921,10 +945,13 @@ BOOL CScreensaver::CreateGraphicsWindowPromotionThread() {
 // Terminate the thread that is used to promote the graphics window.
 //
 BOOL CScreensaver::DestroyGraphicsWindowPromotionThread() {
+   	BOINCTRACE(_T("CScreensaver::DestroyGraphicsWindowPromotionThread: Shutting Down...\n"));
+
     if (!TerminateThread(m_hGraphicsWindowPromotionThread, 0)) {
     	BOINCTRACE(_T("CScreensaver::DestroyGraphicsWindowPromotionThread: Failed to terminate graphics window promotion thread '%d'\n"), GetLastError());
         return FALSE;
     }
+
     return TRUE;
 }
 
@@ -1016,7 +1043,9 @@ DWORD WINAPI CScreensaver::GraphicsWindowPromotionProc() {
 //
 BOOL CScreensaver::CreateDataManagementThread() {
     DWORD dwThreadID = 0;
+
     BOINCTRACE(_T("CScreensaver::CreateDataManagementThread Start\n"));
+
     m_hDataManagementThread = CreateThread(
         NULL,                        // default security attributes 
         0,                           // use default stack size  
@@ -1029,6 +1058,7 @@ BOOL CScreensaver::CreateDataManagementThread() {
     	BOINCTRACE(_T("CScreensaver::CreateDataManagementThread: Failed to create data management thread '%d'\n"), GetLastError());
         return FALSE;
    }
+
    return TRUE;
 }
 
@@ -1038,10 +1068,23 @@ BOOL CScreensaver::CreateDataManagementThread() {
 // Terminate the thread that is used to talk to the daemon.
 //
 BOOL CScreensaver::DestroyDataManagementThread() {
+    BOINCTRACE(_T("CScreensaver::DestoryDataManagementThread: Shutting down... \n"));
+
+    m_bQuitDataManagementProc = true;  
+    for (int i = 0; i < 50; i++) {
+        if (m_bDataManagementProcStopped) {
+            BOINCTRACE(_T("CScreensaver::DestoryDataManagementThread: Thread gracefully shutdown \n"));
+            return TRUE;
+        }
+        boinc_sleep(0.1);
+    }
+
+    BOINCTRACE(_T("CScreensaver::DestoryDataManagementThread: Terminating thread... \n"));
     if (!TerminateThread(m_hDataManagementThread, 0)) {
-    	BOINCTRACE(_T("CScreensaver::DestoryDataManagementThread: Failed to terminate data management thread '%d'\n"), GetLastError());
+    	BOINCTRACE(_T("CScreensaver::DestoryDataManagementThread: Failed to terminate thread '%d'\n"), GetLastError());
         return FALSE;
     }
+
     return TRUE;
 }
 
@@ -1104,7 +1147,7 @@ HRESULT CScreensaver::CreateSaverWindow() {
     cls2.hbrBackground  = (HBRUSH) GetStockObject(BLACK_BRUSH);
     cls2.hInstance      = m_hInstance; 
     cls2.style          = CS_VREDRAW|CS_HREDRAW;
-    cls2.lpfnWndProc    = SaverProcStub;
+    cls2.lpfnWndProc    = GenericSaverProcStub;
     cls2.cbWndExtra     = 0; 
     cls2.cbClsExtra     = 0; 
     RegisterClass(&cls2);
@@ -1218,21 +1261,21 @@ VOID CScreensaver::DoConfig() {
 
 
 
-// Handle window messages for main screensaver windows.
+// Handle window messages for main screensaver window.
 //
 LRESULT CScreensaver::SaverProc(
     HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam
 ) {
     DWORD dwMonitor = 0;
+
 #ifdef _DEBUG
     for(DWORD iIndex = 0; iIndex < m_dwNumMonitors; iIndex++) {
 		if (hWnd == m_Monitors[iIndex].hWnd ) {
             dwMonitor = iIndex;
         }
     }
-#endif
-
     BOINCTRACE(_T("CScreensaver::SaverProc [%d] hWnd '%d' uMsg '%X' wParam '%d' lParam '%d'\n"), dwMonitor, hWnd, uMsg, wParam, lParam);
+#endif
 
     switch (uMsg) {
         case WM_TIMER:
@@ -1297,7 +1340,7 @@ LRESULT CScreensaver::SaverProc(
                     m_dwSaverMouseMoveCount++;
                     if (m_dwSaverMouseMoveCount > 5) {
                         BOINCTRACE(_T("CScreensaver::SaverProc Received WM_MOUSEMOVE and time to InterruptSaver()\n"));
-                        InterruptSaver();
+                        FireInterruptSaverEvent();
                     }
                 }
             }
@@ -1310,18 +1353,21 @@ LRESULT CScreensaver::SaverProc(
         case WM_MBUTTONDOWN:
             BOINCTRACE(_T("CScreensaver::SaverProc Received WM_KEYDOWN | WM_LBUTTONDOWN | WM_RBUTTONDOWN | WM_MBUTTONDOWN\n"));
             if (m_SaverMode != sm_test) {
-                InterruptSaver();
+                FireInterruptSaverEvent();
             }
             return 0;
             break;
 
         case WM_CLOSE:
-        case WM_DESTROY:
-            BOINCTRACE(_T("CScreensaver::SaverProc Received WM_CLOSE or WM_DESTROY\n"));
+            BOINCTRACE(_T("CScreensaver::SaverProc Received WM_CLOSE\n"));
             if (m_SaverMode == sm_preview || m_SaverMode == sm_test) {
-                ShutdownSaver();
+                FireInterruptSaverEvent();
             }
-            return 0;
+            break;
+
+        case WM_DESTROY:
+            BOINCTRACE(_T("CScreensaver::SaverProc Received WM_DESTROY\n"));
+            PostQuitMessage(0);
             break;
 
         case WM_SYSCOMMAND: 
@@ -1349,7 +1395,7 @@ LRESULT CScreensaver::SaverProc(
         case WM_POWERBROADCAST:
             BOINCTRACE(_T("CScreensaver::SaverProc Received WM_POWERBROADCAST\n"));
             if (wParam == PBT_APMQUERYSUSPEND)
-                InterruptSaver();
+                FireInterruptSaverEvent();
             break;
     }
 
@@ -1363,7 +1409,155 @@ LRESULT CScreensaver::SaverProc(
     } else if (WM_INTERRUPTSAVER == uMsg) {
 
         BOINCTRACE(_T("CScreensaver::SaverProc Received WM_INTERRUPTSAVER\n"));
-        InterruptSaver();
+        ShutdownSaver();
+
+    }
+
+    return DefWindowProc(hWnd, uMsg, wParam, lParam);
+}
+
+
+
+
+// Handle window messages for secondary screensaver windows.
+//
+LRESULT CScreensaver::GenericSaverProc(
+    HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam
+) {
+    DWORD dwMonitor = 0;
+
+#ifdef _DEBUG
+    for(DWORD iIndex = 0; iIndex < m_dwNumMonitors; iIndex++) {
+		if (hWnd == m_Monitors[iIndex].hWnd ) {
+            dwMonitor = iIndex;
+        }
+    }
+    BOINCTRACE(_T("CScreensaver::GenericSaverProc [%d] hWnd '%d' uMsg '%X' wParam '%d' lParam '%d'\n"), dwMonitor, hWnd, uMsg, wParam, lParam);
+#endif
+
+    switch (uMsg) {
+        case WM_TIMER:
+            BOINCTRACE(_T("CScreensaver::GenericSaverProc Received WM_TIMER\n"));
+			switch (wParam) { 
+				case 1: 
+					// Initial idle time is done, proceed with initialization.
+					m_bWaitForInputIdle = FALSE;
+					KillTimer(hWnd, 1);
+                    return 0;
+                    break;
+				case 2:
+                    // Create a screen saver window on the primary display if 
+                    //   the boinc client crashes
+	                CreateSaverWindow();
+
+                    // Update the position of the box every second so that it
+                    //   does not end up off the visible area of the screen.
+				    UpdateErrorBox();
+                    return 0;
+                    break;
+            }
+            break;
+        case WM_PAINT:
+            {
+				BOOL    bErrorMode;
+				HRESULT hrError;
+				TCHAR	szError[400];
+				GetError(bErrorMode, hrError, szError, sizeof(szError)/sizeof(TCHAR));
+
+				// Show error message, if there is one
+                PAINTSTRUCT ps;
+                BeginPaint(hWnd, &ps);
+
+                // In preview mode, just fill 
+                // the preview window with black, and the BOINC icon. 
+                if (!bErrorMode && m_SaverMode == sm_preview) {
+                    RECT rc;
+                    GetClientRect(hWnd,&rc);
+				    FillRect(ps.hdc, &rc, (HBRUSH)GetStockObject(BLACK_BRUSH));
+				    DrawIcon(ps.hdc, (rc.right / 2) - 16, (rc.bottom / 2) - 16,
+					    LoadIcon(m_hInstance, MAKEINTRESOURCE(IDI_MAIN_ICON)));
+                } else {
+                    DoPaint(hWnd, ps.hdc, &ps);
+                }
+
+                EndPaint(hWnd, &ps);
+            }
+
+            return 0;
+            break;
+
+        case WM_MOUSEMOVE:
+            if (m_SaverMode != sm_test) {
+                static INT xPrev = -1;
+                static INT yPrev = -1;
+                INT xCur = LOWORD(lParam);
+                INT yCur = HIWORD(lParam);
+                if (xCur != xPrev || yCur != yPrev) {
+                    xPrev = xCur;
+                    yPrev = yCur;
+                    m_dwSaverMouseMoveCount++;
+                    if (m_dwSaverMouseMoveCount > 5) {
+                        BOINCTRACE(_T("CScreensaver::GenericSaverProc Received WM_MOUSEMOVE and time to InterruptSaver()\n"));
+                        FireInterruptSaverEvent();
+                    }
+                }
+            }
+            return 0;
+            break;
+
+        case WM_KEYDOWN:
+        case WM_LBUTTONDOWN:
+        case WM_RBUTTONDOWN:
+        case WM_MBUTTONDOWN:
+            BOINCTRACE(_T("CScreensaver::GenericSaverProc Received WM_KEYDOWN | WM_LBUTTONDOWN | WM_RBUTTONDOWN | WM_MBUTTONDOWN\n"));
+            if (m_SaverMode != sm_test) {
+                FireInterruptSaverEvent();
+            }
+            return 0;
+            break;
+
+        case WM_CLOSE:
+            BOINCTRACE(_T("CScreensaver::GenericSaverProc Received WM_CLOSE\n"));
+            if (m_SaverMode == sm_preview || m_SaverMode == sm_test) {
+                FireInterruptSaverEvent();
+            }
+            break;
+
+        case WM_SYSCOMMAND: 
+            BOINCTRACE(_T("CScreensaver::GenericSaverProc Received WM_SYSCOMMAND\n"));
+            if (m_SaverMode == sm_full) {
+                switch (wParam) {
+                    case SC_NEXTWINDOW:
+                    case SC_PREVWINDOW:
+                    case SC_SCREENSAVE:
+                    case SC_CLOSE:
+                        return 0;
+                }
+            }
+            break;
+
+        case WM_SETCURSOR:
+            BOINCTRACE(_T("CScreensaver::GenericSaverProc Received WM_SETCURSOR\n"));
+            if (m_SaverMode == sm_full) {
+                // Hide cursor
+                SetCursor(NULL);
+                return TRUE;
+            }
+            break;
+
+        case WM_POWERBROADCAST:
+            BOINCTRACE(_T("CScreensaver::GenericSaverProc Received WM_POWERBROADCAST\n"));
+            if (wParam == PBT_APMQUERYSUSPEND)
+                FireInterruptSaverEvent();
+            break;
+    }
+
+    if (WM_SETTIMER == uMsg) {
+
+        BOINCTRACE(_T("CScreensaver::GenericSaverProc Received WM_SETTIMER\n"));
+        // All initialization messages have gone through.  Allow
+        // 500ms of idle time, then proceed with initialization.
+        SetTimer(hWnd, 1, 500, NULL);
 
     }
 
@@ -1521,6 +1715,18 @@ LRESULT CALLBACK CScreensaver::SaverProcStub(
 
 
 
+// This function forwards all window messages to GenericSaverProc, which has
+//       access to the "this" pointer.
+//
+LRESULT CALLBACK CScreensaver::GenericSaverProcStub(
+    HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam
+) {
+    return gspScreensaver->GenericSaverProc(hWnd, uMsg, wParam, lParam);
+}
+
+
+
+
 INT_PTR CALLBACK CScreensaver::ConfigureDialogProcStub(
     HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam
 ) {
@@ -1542,14 +1748,10 @@ VOID CScreensaver::ShutdownSaver() {
     // Kill the currently executing graphics application
     terminate_screensaver(m_hGraphicsApplication, &m_running_result);
 
-    // Post message to drop out of message loop
-    // This can be called from the data management thread, so specifically
-    // lookup and post to the primary window instead of calling PostQuitMessage
-    // since PostQuitMessage posts to the current threads message pump if it
-    // exists.
+    // Close all screensaver windows
     for(DWORD iIndex = 0; iIndex < m_dwNumMonitors; iIndex++) {
 		if ( m_Monitors[iIndex].hWnd ) {
-            PostMessage(m_Monitors[iIndex].hWnd, WM_QUIT, NULL, NULL);
+            DestroyWindow(m_Monitors[iIndex].hWnd);
         }
     }
 
@@ -1574,18 +1776,6 @@ VOID CScreensaver::FireInterruptSaverEvent() {
 
 
 
-// A message was received (mouse move, keydown, etc.) that may mean
-//     the screen saver should show the password dialog and/or shut down.
-//
-VOID CScreensaver::InterruptSaver() {
-    BOINCTRACE(_T("CScreensaver::InterruptSaver Function Begin\n"));
-    ShutdownSaver();
-    BOINCTRACE(_T("CScreensaver::InterruptSaver Function End\n"));
-}
-
-
-
-
 // Update the box that shows the error message
 //
 VOID CScreensaver::UpdateErrorBox() {
@@ -1746,13 +1936,25 @@ VOID CScreensaver::DoPaint(HWND hwnd, HDC hdc, LPPAINTSTRUCT lpps) {
     BITMAP bm;
     GetObject(hbmp, sizeof(BITMAP), (LPSTR)&bm);
 
+    HDC hdcTemp = CreateCompatibleDC(hdc);
+    SelectObject(hdcTemp, hbmp);
+
 	long left = rc.left + (pMonitorInfo->widthError - 4 - bm.bmWidth)/2;
 	long top = rc.top + 2;
-    DrawTransparentBitmap(hdc, hbmp, left, top, RGB(255, 0, 255));
+
+    POINT pt;
+    pt.x = bm.bmWidth;
+    pt.y = bm.bmHeight;
+    DPtoLP(hdcTemp, &pt, 1);
+
+    TransparentBlt(
+        hdc, left, top, pt.x, pt.y, hdcTemp, 0, 0, pt.x, pt.y, RGB(255, 0, 255)
+    );
+    DeleteDC(hdcTemp);
 
 	// Draw text in the center of the frame
 	SetBkColor(hdc, RGB(0,0,0));           // Black
-	SetTextColor(hdc, RGB(255,255,255));   // Red
+	SetTextColor(hdc, RGB(255,255,255));   // White
    
 	// Set font
 	HFONT hFont;
@@ -1786,129 +1988,3 @@ VOID CScreensaver::DoPaint(HWND hwnd, HDC hdc, LPPAINTSTRUCT lpps) {
     if(hFont) DeleteObject(hFont);
 }
 
-
-
-
-// Draws a bitmap on the screen with a transparent background.
-//         Code orginally from Microsoft Knowledge Base Article - 79212
-//
-void CScreensaver::DrawTransparentBitmap(
-    HDC hdc, HBITMAP hBitmap, LONG xStart, LONG yStart,
-    COLORREF cTransparentColor
-){
-    BITMAP     bm;
-    COLORREF   cColor;
-    HBITMAP    bmAndBack, bmAndObject, bmAndMem, bmSave;
-    HBITMAP    bmBackOld, bmObjectOld, bmMemOld, bmSaveOld;
-    HDC        hdcMem, hdcBack, hdcObject, hdcTemp, hdcSave;
-    POINT      ptSize;
-
-    hdcTemp = CreateCompatibleDC(hdc);
-    SelectObject(hdcTemp, hBitmap);   // Select the bitmap
-
-    GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&bm);
-    ptSize.x = bm.bmWidth;            // Get width of bitmap
-    ptSize.y = bm.bmHeight;           // Get height of bitmap
-    DPtoLP(hdcTemp, &ptSize, 1);      // Convert from device
-
-                                     // to logical points
-
-    // Create some DCs to hold temporary data.
-    hdcBack   = CreateCompatibleDC(hdc);
-    hdcObject = CreateCompatibleDC(hdc);
-    hdcMem    = CreateCompatibleDC(hdc);
-    hdcSave   = CreateCompatibleDC(hdc);
-
-    // Create a bitmap for each DC. DCs are required for a number of
-    // GDI functions.
-
-    // Monochrome DC
-    bmAndBack   = CreateBitmap(ptSize.x, ptSize.y, 1, 1, NULL);
-
-    // Monochrome DC
-    bmAndObject = CreateBitmap(ptSize.x, ptSize.y, 1, 1, NULL);
-
-    bmAndMem    = CreateCompatibleBitmap(hdc, ptSize.x, ptSize.y);
-    bmSave      = CreateCompatibleBitmap(hdc, ptSize.x, ptSize.y);
-
-    // Each DC must select a bitmap object to store pixel data.
-    bmBackOld   = (HBITMAP)SelectObject(hdcBack, bmAndBack);
-    bmObjectOld = (HBITMAP)SelectObject(hdcObject, bmAndObject);
-    bmMemOld    = (HBITMAP)SelectObject(hdcMem, bmAndMem);
-    bmSaveOld   = (HBITMAP)SelectObject(hdcSave, bmSave);
-
-    // Set proper mapping mode.
-    SetMapMode(hdcTemp, GetMapMode(hdc));
-
-    // Save the bitmap sent here, because it will be overwritten.
-    BitBlt(hdcSave, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);
-
-    // Set the background color of the source DC to the color.
-    // contained in the parts of the bitmap that should be transparent
-    cColor = SetBkColor(hdcTemp, cTransparentColor);
-
-    // Create the object mask for the bitmap by performing a BitBlt
-    // from the source bitmap to a monochrome bitmap.
-    BitBlt(hdcObject, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);
-
-    // Set the background color of the source DC back to the original
-    // color.
-    SetBkColor(hdcTemp, cColor);
-
-    // Create the inverse of the object mask.
-    BitBlt(hdcBack, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, NOTSRCCOPY);
-
-    // Copy the background of the main DC to the destination.
-    BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdc, xStart, yStart, SRCCOPY);
-
-    // Mask out the places where the bitmap will be placed.
-    BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, SRCAND);
-
-    // Mask out the transparent colored pixels on the bitmap.
-    BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcBack, 0, 0, SRCAND);
-
-    // XOR the bitmap with the background on the destination DC.
-    BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCPAINT);
-
-    // Copy the destination to the screen.
-    BitBlt(hdc, xStart, yStart, ptSize.x, ptSize.y, hdcMem, 0, 0, SRCCOPY);
-
-    // Place the original bitmap back into the bitmap sent here.
-    BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcSave, 0, 0, SRCCOPY);
-
-    // Delete the memory bitmaps.
-    DeleteObject(SelectObject(hdcBack, bmBackOld));
-    DeleteObject(SelectObject(hdcObject, bmObjectOld));
-    DeleteObject(SelectObject(hdcMem, bmMemOld));
-    DeleteObject(SelectObject(hdcSave, bmSaveOld));
-
-    // Delete the memory DCs.
-    DeleteDC(hdcMem);
-    DeleteDC(hdcBack);
-    DeleteDC(hdcObject);
-    DeleteDC(hdcSave);
-    DeleteDC(hdcTemp);
-}
-
-
-
-
-VOID CScreensaver::ChangePassword() {
-    // Load the password change DLL
-    HINSTANCE mpr = LoadLibrary(_T("MPR.DLL"));
-
-    if (mpr != NULL) {
-        // Grab the password change function from it
-        typedef DWORD (PASCAL *PWCHGPROC)(LPCSTR, HWND, DWORD, LPVOID);
-        PWCHGPROC pwd = (PWCHGPROC)GetProcAddress(mpr, "PwdChangePasswordA");
-
-        // Do the password change
-        if (pwd != NULL) {
-            pwd("SCRSAVE", m_hWndParent, 0, NULL);
-        }
-
-        // Free the library
-        FreeLibrary(mpr);
-    }
-}
-
