@@ -194,7 +194,8 @@ protected:
     HANDLE          m_hDataManagementThread;
     HANDLE          m_hGraphicsApplication;
     BOOL            m_bResetCoreState;
-    bool            m_QuitDataManagementProc;
+    bool            m_bQuitDataManagementProc;
+    bool            m_bDataManagementProcStopped;
     bool            m_bV5_GFX_app_is_running;
 	int				m_iLastResultShown;
 	time_t			m_tLastResultChangeTime;
@@ -216,18 +217,14 @@ protected:
 	HRESULT         CreateSaverWindow();
 	VOID            UpdateErrorBox();
     VOID            FireInterruptSaverEvent();
-    VOID            InterruptSaver();
     VOID            ShutdownSaver();
-	VOID            ChangePassword();
 
 
     VOID            DoConfig();
 	HRESULT         DoSaver();
 	VOID            DoPaint( HWND hwnd, HDC hdc, LPPAINTSTRUCT lpps );
-
-	void			DrawTransparentBitmap(HDC hdc, HBITMAP hBitmap, LONG xStart, LONG yStart, COLORREF cTransparentColor);
-
 	LRESULT         SaverProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
+	LRESULT         GenericSaverProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
 
     DWORD           ConvertSliderPositionToTime( DWORD dwPosition );
     DWORD           ConvertTimeToSliderPosition( DWORD dwMinutes );
@@ -237,6 +234,7 @@ protected:
 	INT_PTR         ConfigureDialogProc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam );
 
     static LRESULT CALLBACK SaverProcStub( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
+    static LRESULT CALLBACK GenericSaverProcStub( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
 	static INT_PTR CALLBACK ConfigureDialogProcStub( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
 
 protected:
