@@ -16,12 +16,11 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
-
-
+$cli_only = true;
 require_once("../inc/db.inc");
+require_once("../inc/util_ops.inc");
 
 set_time_limit(0);
-
 db_init();
 
 function clean_user($user) {
