@@ -21,6 +21,19 @@ require_once("../inc/util_ops.inc");
 require_once("../inc/uotd.inc");
 require_once("../project/project.inc");
 
+function svn_revision($path) {
+    $out = array();
+    exec("svn info http://boinc.berkeley.edu/svn/$path", $out);
+    foreach ($out as $line) {
+        $x = strstr($line, "Last Changed Rev: ");
+        if ($x) {
+            $y = substr($x, strlen("Last Changed Rev: "));
+            return (int) $y;
+        }
+    }
+    return null;
+}
+
 $config = get_config();
 $cgi_url = parse_config($config, "<cgi_url>");
 $stripchart_cgi_url = parse_config($config, "<stripchart_cgi_url>");
@@ -38,44 +51,22 @@ if (file_exists("../../local.revision")) {
     $local_rev = file_get_contents("../../local.revision");
 }
 if ($local_rev) {
-    echo "Currently used SVN revision: ".$local_rev."; ";
+    echo "Using BOINC SVN revision: ".$local_rev."; ";
 }
 
-if (file_exists("../cache/remote.revision")
-    && (time() < filemtime("../cache/remote.revision")+(24*60*60))
+if (0
+//if (file_exists("../cache/remote.revision")
+//    && (time() < filemtime("../cache/remote.revision")+(24*60*60))
 ) {
     $remote_rev = file_get_contents("../cache/remote.revision");
 } else {
-    // Get latest revision
-    if (isset($project_http_proxy)) {
-        $context = stream_context_create(
-            array(
-                'http' => array(
-                    'request_fulluri' => true,
-                    'proxy' => $project_http_proxy
-                )
-            )
-        );
-        $handle = fopen("http://boinc.berkeley.edu/svn/", "r", false, $context);
-    } else {
-        $handle = fopen("http://boinc.berkeley.edu/svn/", "r");
-    }
-    if ($handle) {
-        $remote = fread($handle, 255);
-        fclose($handle);
-        preg_match("/Revision (\d+)/", $remote, $remote_rev);
-        $remote_rev = $remote_rev[1];
-
-        $handle = fopen("../cache/remote.revision", "w");
-        fwrite($handle, $remote_rev);
-        fclose($handle);
-    } else {
-        echo "Can't get latest SVN revision";
-    }
+    $remote_rev = svn_revision("branches/server_stable");
 }
 
 if ($remote_rev) {
-    echo "Latest SVN revision: ".$remote_rev."</li>\n";
+    echo "BOINC server_stable SVN revision: $remote_rev";
+} else {
+    echo "Can't get BOINC server_stable SVN revision";
 }
 
 if (!file_exists(".htaccess")) {
@@ -119,44 +110,50 @@ echo "
     <table border=\"0\"><tr valign=\"top\">
     <td><b>Browse database:</b>
     <ul> 
-        <li><a href=\"db_action.php?table=platform\">Platforms</a></li>
-        <li><a href=\"db_action.php?table=app\">Applications</a></li>
-        <li><a href=\"db_form.php?table=app_version\">Application versions</a></li>
+        <li><a href=\"db_form.php?table=result&amp;detail=low\">Results</a></li>
+        <li><a href=\"db_form.php?table=workunit\">Workunits</a></li>
+        <li><a href=\"db_form.php?table=host&amp;detail=low\">Hosts</a></li>
         <li><a href=\"db_form.php?table=user\">Users</a> (<a href=\"list_new_users.php\">recently registered</a>)</li>
         <li><a href=\"db_form.php?table=team\">Teams</a></li>
-        <li><a href=\"db_form.php?table=host&amp;detail=low\">Hosts</a></li>
-        <li><a href=\"db_form.php?table=workunit\">Workunits</a></li>
-        <li><a href=\"db_form.php?table=result&amp;detail=low\">Results</a></li>
+        <li><a href=\"db_action.php?table=app\">Applications</a></li>
+        <li><a href=\"db_form.php?table=app_version\">Application versions</a></li>
+        <li><a href=\"db_action.php?table=platform\">Platforms</a></li>
         <li><a href=dbinfo.php>DB row counts and disk usage</a>
+        <li><a href=\"show_log.php?f=mysql*.log&amp;l=-20\">Tail MySQL logs</a>
     </ul>
     
+
     </td> 
-    <td><b>Regular Operations:</b>
-    <ul>
-        <li><a href=\"profile_screen_form.php\">Screen user profiles </a></li>
-        <li><a href=\"manage_special_users.php\">Manage special users</a></li>
-    </ul>
-    
-    </td> 
-    <td><b>Special Operations:</b>
+    <td><b>Computing</b>
     <ul>
         <li><a href=\"manage_apps.php\">Manage applications</a></li>
         <li><a href=\"manage_app_versions.php\">Manage application versions</a></li>
+        <li><a href=\"cancel_wu_form.php\">Cancel workunits</a></li>
+        <li><a href=\"job_times.php\">FLOP count statistics</a>
+        <li><a href=\"$stripchart_cgi_url/stripchart.cgi\">Stripcharts</a>
+        <li><a href=\"show_log.php\">Show/Grep logs</a>
+        <li><a href=transition_all.php>Transition all WUs</a>
+          <br><span class=note>(this can 'unstick' old WUs)</span>
+        <li>
+            <form method=\"get\" action=\"clear_host.php\">
+            <input type=\"submit\" value=\"Clear RPC seqno\">
+            host ID: 
+            <input type=\"text\" size=\"5\" name=\"hostid\">
+            </form>
+    </ul>
+    
+    </td> 
+    <td><b>User management</b>
+    <ul>
+        <li><a href=\"profile_screen_form.php\">Screen user profiles </a></li>
+        <li><a href=\"manage_special_users.php\">User privileges</a></li>
         <li><a href=\"mass_email.php\">Send mass email to a selected set of users</a></li>
         <li><a href=\"problem_host.php\">Email user with misconfigured host</a></li>
-        <li><a href=\"job_times.php\">FLOP count statistics</a>
-        <li><a href=\"cancel_wu_form.php\">Cancel workunits</a></li>
         <li><form action=\"manage_user.php\">
             <input type=\"submit\" value=\"Manage user\">
             ID: <input name=\"userid\">
             </form>
         </li>
-        <li>
-            <form method=\"get\" action=\"clear_host.php\">
-            Clear Host: 
-            <input type=\"text\" size=\"5\" name=\"hostid\">
-            <input type=\"submit\" value=\"Clear RPC\">
-            </form>
         </li>
     </ul>
     </td>
@@ -215,16 +212,7 @@ echo "<h3>Periodic or special tasks</h3>
    </ul>
     ";
 
-// Stripcharts, logs, etc
-
-echo "<div>
-    <a href=\"$stripchart_cgi_url/stripchart.cgi\">Stripcharts</a>
-    | <a href=\"show_log.php\">Show/Grep all logs</a>
-    | <a href=\"show_log.php?f=mysql*.log&amp;l=-20\">Tail MySQL logs</a>
-    </div>
-";
-
 admin_page_tail();
 
-$cvs_version_tracker[]="\$Id: index.php 16310 2008-10-24 16:18:28Z davea $";  //Generated automatically - do not edit
+$cvs_version_tracker[]="\$Id: index.php 21848 2010-06-30 19:04:36Z boincadm $";  //Generated automatically - do not edit
 ?>
