@@ -16,8 +16,9 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
-function getSingleQuery($query)
-{
+require_once("../inc/util_ops.inc");
+
+function getSingleQuery($query) {
     $result = mysql_query($query);
     if (!$result) return;
     $cnt = mysql_fetch_row($result);
