@@ -23,7 +23,7 @@ require_once("../inc/email.inc");
 db_init();
 $user = get_logged_in_user();
 
-page_head("Change email address");
+page_head(tra("Change email address"));
 
 $email_text = "";
 if (is_valid_email_addr($user->email_addr)) {
@@ -32,10 +32,10 @@ if (is_valid_email_addr($user->email_addr)) {
 
 echo "<form method=post action=edit_email_action.php>\n";
 start_table();
-row1("Change the email address of your account");
-row2("New email address
-    <br><font size=-2>Must be a valid address of the form 'name@domain'</font>",
-    "<input name=email_addr size=50 value='$email_text'>"
+row1(tra("Change the email address of your account"));
+row2(tra("New email address").
+    "<br><span class=note>".tra("Must be a valid address of the form 'name@domain'")."</span>",
+    "<input name=email_addr size=50 type=text value='$email_text'>"
 );
 
 // we need the password here not for verification,
@@ -43,11 +43,11 @@ row2("New email address
 // which is about to change.
 
 row2(
-    "Password
-    <br><a href=edit_passwd_form.php><font size=-2>No password?</font></a>",
+    tra("Password").
+    "<br><a href=".SECURE_URL_BASE."/edit_passwd_form.php><span class=note>".tra("No password?")."</span></a>",
     "<input type=password name=passwd>"
 );
-row2("", "<input type=submit value='Change email address'>");
+row2("", "<input type=submit value='".tra("Change email address")."'>");
 end_table();
 echo "</form>\n";
 page_tail();
