@@ -73,7 +73,7 @@ case 'move':
 case 'title':
     echo "<input type=hidden name=action value=title>";
     row2("New title:",
-        "<input name=\"newtitle\" value=\"".htmlspecialchars($thread->title)."\">"
+        "<input size=80 name=\"newtitle\" value=\"".htmlspecialchars($thread->title)."\">"
     );
     break;
 default:
