@@ -49,29 +49,7 @@ if ($init) {
     page_head(tra("Your account"));
 }
 
-start_table_noborder();
-echo "<tr><td valign=top>";
-start_table();
-show_user_info_private($user);
-if (!no_computing()) {
-    show_user_stats_private($user);
-}
-
-if (file_exists("../project/donations.inc")) {
-    require_once("../project/donations.inc");
-    if (function_exists('show_user_donations_private')) {
-        show_user_donations_private($user);
-    }
-}
-end_table();
-show_other_projects($user, true);
-project_user_page_private($user);
-echo "</td><td valign=top>";
-start_table();
-show_community_private($user);
-end_table();
-
-echo "</td></tr></table>";
+show_account_private($user);
 
 page_tail();
 
