@@ -21,6 +21,8 @@ require_once("../inc/util.inc");
 require_once("../inc/user.inc");
 require_once("../inc/host.inc");
 
+BoincDb::get(true);
+
 $hostid = get_int("hostid");
 $ipprivate = get_str("ipprivate", true);
 $host = BoincHost::lookup_id($hostid);
@@ -34,7 +36,7 @@ if ($user->id != $host->userid) {
     $user = null;
 }
 
-page_head("Computer summary");
+page_head("Computer $hostid");
 show_host($host, $user, $ipprivate);
 page_tail();
 
