@@ -25,7 +25,7 @@ $user = get_logged_in_user();
 $teamid = post_int("teamid");
 $team = BoincTeam::lookup_id($teamid);
 if (!$team) {
-    error_page("No such team");
+    error_page(tra("no such team"));
 }
 require_founder_login($user, $team);
 check_tokens($user->authenticator);
@@ -33,12 +33,12 @@ check_tokens($user->authenticator);
 $userid = post_int("userid");
 $new_founder = BoincUser::lookup_id($userid);
 if (!$new_founder || $new_founder->teamid != $team->id) {
-    error_page( "User is not a member of $team->name");
+    error_page(tra("User is not a member of %1", $team->name));
 }
 
-page_head("Changing founder of $team->name");
+page_head(tra("Changing founder of %1", $team->name));
 $team->update("userid=$userid, ping_user=0");
-echo "$new_founder->name is now founder of $team->name";
+echo tra("%1 is now founder of %2", $new_founder->name, $team->name);
 page_tail();
 
 ?>
