@@ -28,64 +28,53 @@ $user = get_logged_in_user();
 $teamid = get_int("teamid");
 $team = BoincTeam::lookup_id($teamid);
 if (!$team) {
-    error_page("no such team");
+    error_page(tra("no such team"));
 }
 require_founder_login($user, $team);
 
-page_head("Change founder of $team->name");
+page_head(tra("Change founder of %1", $team->name));
 
 if ($team->ping_user != 0) {
     if ($team->ping_user < 0) {
         $ping_user = BoincUser::lookup_id(-$team->ping_user);
         $x = date_str($team->ping_time);
-        echo "<p>Team member ".user_links($ping_user)." requested this
-            team's foundership on $x, but left the team, thus canceling the request.
-        ";
+        echo "<p>".tra("Team member %1 requested this team's foundership on %2, but left the team, thus canceling the request.", user_links($ping_user), $x)
+        ."</p>";
         $team->update("ping_user=0, ping_time=0");
     } else {
         $ping_user = BoincUser::lookup_id($team->ping_user);
         $x = date_str(transfer_ok_time($team));
-        echo "<p>Team member ".user_links($ping_user)." has requested this
-            team's foundership.
-            This may be because you left the team or haven't had contact
-            with the team for a long time.
-        ";
+        echo "<p>".tra("Team member %1 has requested this team's foundership. This may be because you left the team or haven't had contact with the team for a long time.", user_links($ping_user))
+        ."</p>";
         echo "<p>
             <form method=\"post\" action=\"team_founder_transfer_action.php\">
             <input type=\"hidden\" name=\"action\" value=\"decline\">
             <input type=\"hidden\" name=\"teamid\" value=\"".$team->id."\">
-            To decline the request, <input type=\"submit\" value=\"click here\">
+            <input type=\"submit\" value=\"".tra("decline request")."\">
             </form>
             <p>
-            If you don't decline the request by $x,
-            $ping_user->name will have the option of
-            assuming team foundership.
-            <p>
-            To accept the request, assign foundership to $ping_user->name
-            using the form below.
-            <p>
+            ".tra("If you don't decline the request by %1, %2 will have the option of assuming team foundership.<br /><br />
+                  To accept the request, assign foundership to %3 using the form below.", $x, $ping_user->name, $ping_user->name)."
+            </p>
             <hr>
             <p>
         ";
     }
 } else {
-    echo "No transfer request is pending.<p>";
+    echo tra("No transfer request is pending.")."<br /><br />";
 }
 
-echo "
-    To assign foundership of this team to another member,
-    check the box next to member name and click <b>Change founder</b> below.
-    <form method=post action=team_change_founder_action.php>
-    <input type=hidden name=teamid value=$team->id>
-";
+echo tra("To assign foundership of this team to another member, check the box next to member name and click <strong>Change founder</strong> below.")
+    ."<form method=post action=team_change_founder_action.php>
+    <input type=hidden name=teamid value=$team->id>";
 echo form_tokens($user->authenticator);
 
 start_table();
 echo "<tr>
-    <th>New founder?</th>
-    <th>Name</th>
-    <th>Total credit</th>
-    <th>Recent average credit</th>
+    <th>".tra("New founder?")."</th>
+    <th>".tra("Name")."</th>
+    <th>".tra("Total credit")."</th>
+    <th>".tra("Recent average credit")."</th>
     </tr>
 ";
 
@@ -111,10 +100,10 @@ foreach ($users as $user) {
 if ($navailable_users > 0) {
     echo "<input type=hidden name=navailable_users value=$navailable_users>";
     end_table();
-    echo "<input type=submit value=\"Change founder\">";
+    echo "<input type=submit value=\"".tra("Change founder")."\">";
 } else {
     echo '<tr>
-        <td colspan="4">There are no users to transfer team to.</td>
+        <td colspan="4">".tra("There are no users to transfer team to.")."</td>
         </tr>
     ';
     end_table();
@@ -122,5 +111,5 @@ if ($navailable_users > 0) {
 echo "</form>";
 page_tail();
 
-$cvs_version_tracker[]="\$Id: team_change_founder_form.php 16665 2008-12-09 22:42:28Z boincadm $";  //Generated automatically - do not edit
+$cvs_version_tracker[]="\$Id: team_change_founder_form.php 20582 2010-02-16 01:06:03Z davea $";  //Generated automatically - do not edit
 ?>
