@@ -23,20 +23,19 @@ require_once("../inc/team.inc");
 require_once("../inc/forum_db.inc");
 
 function create_confirm($user, $team) {
-    page_head("Create Message Board");
-    echo "
-        You may create a Message Board for use by $team->name.
+    page_head(tra("Create Message Board"));
+    echo tra("You may create a message board for use by %1.", $team->name)."
         <ul>
-        <li> Only team members will be able to post.
-        <li> At your option, only members will be able to read.
-        <li> You and your Team Admins will have moderator privileges.
+        <li>".tra("Only team members will be able to post.")."
+        <li>".tra("At your option, only members will be able to read.")."
+        <li>".tra("You and your Team Admins will have moderator privileges.")."
         </ul>
     ";
     $tokens = url_tokens($user->authenticator);
     show_button(
         "team_forum.php?teamid=$team->id&cmd=create$tokens",
-        "Create Message Board",
-        "Create a Message Board for $team->name"
+        tra("Create Message Board"),
+        tra("Create a message board for %1", $team->name)
     );
     page_tail();
 }
@@ -44,18 +43,18 @@ function create_confirm($user, $team) {
 function create_forum($user, $team) {
     $f = BoincForum::lookup("parent_type=1 and category=$team->id");
     if ($f) {
-        error_page("Team already has a forum");
+        error_page(tra("Team already has a message board"));
     }
     $id = BoincForum::insert("(category, parent_type) values ($team->id, 1)");
     $forum = BoincForum::lookup_id($id);
     if (!$forum) {
-        error_page("couldn't create forum");
+        error_page(tra("couldn't create message board"));
     }
     edit_form($user, $team, $forum, true);
 }
 
 function edit_form($user, $team, $forum, $first) {
-    page_head("Team forum");
+    page_head(tra("Team Message Board"));
     echo "
         <form action=team_forum.php method=post>
         <input type=hidden name=teamid value=$team->id>
@@ -64,19 +63,19 @@ function edit_form($user, $team, $forum, $first) {
     echo form_tokens($user->authenticator);
     start_table();
     if (!strlen($forum->title)) $forum->title = $team->name;
-    if (!strlen($forum->description)) $forum->description = "Discussion among members of $team->name";
-    row2("Title", "<input name=title value=\"$forum->title\">");
-    row2("Description", "<textarea name=description>$forum->description</textarea>");
-    row2("Minimum time between posts (seconds)",
+    if (!strlen($forum->description)) $forum->description = tra("Discussion among members of %1", $team->name);
+    row2(tra("Title"), "<input name=title value=\"$forum->title\">");
+    row2(tra("Description"), "<textarea name=description>$forum->description</textarea>");
+    row2(tra("Minimum time between posts (seconds)"),
         "<input name=post_min_interval value=$forum->post_min_interval>"
     );
-    row2("Minimum total credit to post",
+    row2(tra("Minimum total credit to post"),
         "<input name=post_min_total_credit value=$forum->post_min_total_credit>"
     );
-    row2("Minimum average credit to post",
+    row2(tra("Minimum average credit to post"),
         "<input name=post_min_expavg_credit value=$forum->post_min_expavg_credit>"
     );
-    row2("", "<input type=submit value=OK>");
+    row2("", "<input type=submit value=".tra("Submit").">");
     end_table();
     echo "
         </form>
@@ -85,7 +84,7 @@ function edit_form($user, $team, $forum, $first) {
         echo "
             <p>
             <a href=team_forum.php?teamid=$team->id&cmd=remove_confirm$tokens>
-            Remove your team's message board.</a>
+            ".tra("Remove your team's message board.")."</a>
         ";
     }
     page_tail();
@@ -93,20 +92,17 @@ function edit_form($user, $team, $forum, $first) {
 
 function remove_confirm($user, $team) {
     $tokens = url_tokens($user->authenticator);
-    page_head("Really remove message board?");
-    echo "
-        Are you sure you want to remove your team's message board?
-        All threads and posts will be permanently removed.
-        (You may, however, create a new message board later).
-        <p>
-        <a href=team_forum.php?teamid=$team->id&cmd=remove>Yes - remove message board</a>
+    page_head(tra("Really remove message board?"));
+    echo tra("Are you sure you want to remove your team's message board? All threads and posts will be permanently removed. (You may, however, create a new message board later).")
+        ."<br /><br />
+        <a href=team_forum.php?teamid=$team->id&cmd=remove>".tra("Yes - remove message board")."</a>
     ";
     page_tail();
 }
 
 function remove($team) {
     $forum = BoincForum::lookup("parent_type=1 and category=$team->id");
-    if (!$forum) error_page("not found");
+    if (!$forum) error_page(tra("message board not found"));
     
     // delete threads and posts
     //
@@ -120,7 +116,15 @@ function remove($team) {
     }
     $forum->delete();
 
-    page_head("Message board removed");
+    page_head(tra("Message board removed"));
+    echo "<p>"
+        .tra(
+            "Your team's message board has been removed. You may now %1create a new one%2.",
+            "<a href=team_forum.php?teamid=$team->id&cmd=manage>",
+            "</a>"
+        )
+        ."</p>"
+    ;
     page_tail();
 }
 
@@ -134,18 +138,18 @@ function edit_action($forum) {
     $post_min_expavg_credit = post_int('post_min_expavg_credit');
     $ret = $forum->update("title='$title', description='$description', post_min_interval=$post_min_interval, post_min_total_credit=$post_min_total_credit, post_min_expavg_credit=$post_min_expavg_credit");
     if ($ret) {
-        page_head("Team Message Board Updated");
-        echo "Update successful";
+        page_head(tra("Team Message Board Updated"));
+        echo tra("Update successful");
         page_tail();
     } else {
-        error_page("update failed");
+        error_page(tra("Update failed"));
     }
 }
 
 function show_forum($team) {
     $forum = BoincForum::lookup("parent_type=1 and category=$team->id");
     if (!$forum) {
-        error_page("team has no forum");
+        error_page(tra("team has no forum"));
     }
     Header("Location: forum_forum.php?id=$forum->id");
 }
@@ -155,7 +159,7 @@ if (!$teamid) $teamid = post_int('teamid');
 
 $team = BoincTeam::lookup_id($teamid);
 if (!$team) {
-    error_page("no such team");
+    error_page(tra("no such team"));
 }
 
 $cmd = get_str('cmd', true);
@@ -180,7 +184,7 @@ if ($cmd == 'manage') {
     require_founder_login($user, $team);
     check_tokens($user->authenticator);
     $forum = BoincForum::lookup("parent_type=1 and category=$teamid");
-    if (!$forum) error_page("no forum");
+    if (!$forum) error_page(tra("no such forum"));
     edit_action($forum);
 } else if ($cmd == "remove_confirm") {
     $user = get_logged_in_user();
@@ -191,7 +195,7 @@ if ($cmd == 'manage') {
     require_founder_login($user, $team);
     remove($team);
 } else if ($cmd != "") {
-    error_page("unknown command $cmd");
+    error_page(tra("unknown command %1", $cmd));
 } else {
     show_forum($team);
 }
