@@ -71,7 +71,7 @@
 #endif
 
 #include "miofile.h"
-#include "cal.h"
+#include "cal_boinc.h"
 
 #define MAX_COPROC_INSTANCES 64
 
@@ -127,6 +127,7 @@ struct COPROC {
 
 #ifndef _USING_FCGI_
     virtual void write_xml(MIOFILE&);
+    void write_request(MIOFILE&);
 #endif
     inline void clear() {
         // can't just memcpy() - trashes vtable
@@ -144,6 +145,12 @@ struct COPROC {
             available_ram_unknown[i] = true;
         }
     }
+    inline void clear_usage() {
+        for (int i=0; i<count; i++) {
+            usage[i] = 0;
+            pending_usage[i] = 0;
+        }
+    }
     COPROC(const char* t){
         clear();
         strcpy(type, t);
@@ -156,62 +163,10 @@ struct COPROC {
     void print_available_ram();
 };
 
-struct COPROCS {
-    std::vector<COPROC*> coprocs;   // not deleted in destructor
-        // so any structure that includes this needs to do it manually
-
-    COPROCS(){}
-    ~COPROCS(){}    // don't delete coprocs; else crash in APP_INIT_DATA logic
-    void write_xml(MIOFILE& out);
-    void get(
-        bool use_all, std::vector<std::string> &descs,
-        std::vector<std::string> &warnings,
-        std::vector<int>& ignore_cuda_dev,
-        std::vector<int>& ignore_ati_dev
-    );
-    int parse(MIOFILE&);
-    void summary_string(char*, int);
-    COPROC* lookup(const char*);
-    bool fully_used() {
-        for (unsigned int i=0; i<coprocs.size(); i++) {
-            COPROC* cp = coprocs[i];
-            if (cp->used < cp->count) return false;
-        }
-        return true;
-    }
-
-    // Copy a coproc set, possibly setting usage to zero.
-    // used in round-robin simulator and CPU scheduler,
-    // to avoid messing w/ master copy
-    //
-    void clone(COPROCS& c, bool copy_used) {
-        for (unsigned int i=0; i<c.coprocs.size(); i++) {
-            COPROC* cp = c.coprocs[i];
-            COPROC* cp2 = new COPROC(cp->type);
-            cp2->count = cp->count;
-			if (copy_used) cp2->used = cp->used;
-            coprocs.push_back(cp2);
-        }
-    }
-    inline void clear_usage() {
-        for (unsigned int i=0; i<coprocs.size(); i++) {
-            COPROC* cp = coprocs[i];
-            for (int j=0; j<cp->count; j++) {
-                cp->usage[j] = 0;
-                cp->pending_usage[j] = 0;
-            }
-        }
-    }
-    inline void delete_coprocs() {
-        for (unsigned int i=0; i<coprocs.size(); i++) {
-            delete coprocs[i];
-        }
-    }
-};
-
-// the following copied from /usr/local/cuda/include/driver_types.h
+// based on cudaDeviceProp from /usr/local/cuda/include/driver_types.h
+// doesn't have to match exactly since we get the attributes one at a time.
 //
-struct cudaDeviceProp {
+struct CUDA_DEVICE_PROP {
   char   name[256];
   unsigned int totalGlobalMem;
     // not used on the server; dtotalGlobalMem is used instead
@@ -236,15 +191,15 @@ struct cudaDeviceProp {
 struct COPROC_CUDA : public COPROC {
     int cuda_version;  // CUDA runtime version
     int display_driver_version;
-    cudaDeviceProp prop;
+    CUDA_DEVICE_PROP prop;
 
 #ifndef _USING_FCGI_
-    virtual void write_xml(MIOFILE&);
+    virtual void write_xml(MIOFILE&, bool include_request);
 #endif
     COPROC_CUDA(): COPROC("CUDA"){}
     virtual ~COPROC_CUDA(){}
-    static void get(
-        COPROCS&, bool use_all,
+    void get(
+        bool use_all,
         std::vector<std::string>&, std::vector<std::string>&,
         std::vector<int>& ignore_devs
     );
@@ -268,44 +223,27 @@ struct COPROC_CUDA : public COPROC {
     void get_available_ram();
 
     bool check_running_graphics_app();
-};
+    void fake(int driver_version, double ram, int count);
 
-enum CUdevice_attribute_enum {
-  CU_DEVICE_ATTRIBUTE_MAX_THREADS_PER_BLOCK = 1,
-  CU_DEVICE_ATTRIBUTE_MAX_BLOCK_DIM_X = 2,
-  CU_DEVICE_ATTRIBUTE_MAX_BLOCK_DIM_Y = 3,
-  CU_DEVICE_ATTRIBUTE_MAX_BLOCK_DIM_Z = 4,
-  CU_DEVICE_ATTRIBUTE_MAX_GRID_DIM_X = 5,
-  CU_DEVICE_ATTRIBUTE_MAX_GRID_DIM_Y = 6,
-  CU_DEVICE_ATTRIBUTE_MAX_GRID_DIM_Z = 7,
-  CU_DEVICE_ATTRIBUTE_SHARED_MEMORY_PER_BLOCK = 8,
-  CU_DEVICE_ATTRIBUTE_TOTAL_CONSTANT_MEMORY = 9,
-  CU_DEVICE_ATTRIBUTE_WARP_SIZE = 10,
-  CU_DEVICE_ATTRIBUTE_MAX_PITCH = 11,
-  CU_DEVICE_ATTRIBUTE_REGISTERS_PER_BLOCK = 12,
-  CU_DEVICE_ATTRIBUTE_CLOCK_RATE = 13,
-  CU_DEVICE_ATTRIBUTE_TEXTURE_ALIGNMENT = 14,
-  CU_DEVICE_ATTRIBUTE_GPU_OVERLAP = 15,
-  CU_DEVICE_ATTRIBUTE_MULTIPROCESSOR_COUNT = 16,
-  CU_DEVICE_ATTRIBUTE_KERNEL_EXEC_TIMEOUT = 17,
-  CU_DEVICE_ATTRIBUTE_INTEGRATED = 18,
-  CU_DEVICE_ATTRIBUTE_CAN_MAP_HOST_MEMORY = 19,
-  CU_DEVICE_ATTRIBUTE_COMPUTE_MODE = 20
 };
 
 struct COPROC_ATI : public COPROC {
     char name[256];
     char version[50];
+    int version_num;
+        // based on CAL version (not driver version)
+        // encoded as 1000000*major + 1000*minor + release
     bool atirt_detected;
     bool amdrt_detected;
     CALdeviceattribs attribs; 
     CALdeviceinfo info;
 #ifndef _USING_FCGI_
-    virtual void write_xml(MIOFILE&);
+    virtual void write_xml(MIOFILE&, bool include_request);
 #endif
     COPROC_ATI(): COPROC("ATI"){}
     virtual ~COPROC_ATI(){}
-    static void get(COPROCS&,
+    void get(
+        bool use_all,
         std::vector<std::string>&, std::vector<std::string>&,
         std::vector<int>& ignore_devs
     );
@@ -318,9 +256,51 @@ struct COPROC_ATI : public COPROC {
         return x?x:5e10;
     }
     void get_available_ram();
+    void fake(double, int);
 };
 
-extern COPROC_CUDA* fake_cuda(COPROCS&, double, int);
-extern COPROC_ATI* fake_ati(COPROCS&, double, int);
+struct COPROCS {
+    COPROC_CUDA cuda;
+    COPROC_ATI ati;
+
+    COPROCS(){}
+    ~COPROCS(){}    // don't delete coprocs; else crash in APP_INIT_DATA logic
+    void write_xml(MIOFILE& out, bool include_request);
+    void get(
+        bool use_all, std::vector<std::string> &descs,
+        std::vector<std::string> &warnings,
+        std::vector<int>& ignore_cuda_dev,
+        std::vector<int>& ignore_ati_dev
+    );
+    int parse(MIOFILE&);
+    void summary_string(char*, int);
+
+    // Copy a coproc set, possibly setting usage to zero.
+    // used in round-robin simulator and CPU scheduler,
+    // to avoid messing w/ master copy
+    //
+    void clone(COPROCS& c, bool copy_used) {
+        cuda = c.cuda;
+        ati = c.ati;
+        if (!copy_used) {
+            cuda.used = 0;
+            ati.used = 0;
+        }
+    }
+    inline void clear() {
+        cuda.count = 0;
+        ati.count = 0;
+    }
+    inline void clear_usage() {
+        cuda.clear_usage();
+        ati.clear_usage();
+    }
+    inline bool none() {
+        return (cuda.count==0) && (ati.count==0);
+    }
+    inline int ndevs() {
+        return cuda.count + ati.count;
+    }
+};
 
 #endif
