@@ -15,8 +15,18 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
-#if defined(_WIN32) && !defined(__STDWX_H__) && !defined(_BOINC_WIN_) && !defined(_AFX_STDAFX_H_)
+#if   defined(_WIN32) && !defined(__STDWX_H__)
 #include "boinc_win.h"
+#elif defined(_WIN32) && defined(__STDWX_H__)
+#include "stdwx.h"
+#endif
+
+#if defined(__MINGW32__)
+#include <fcntl.h>
+#endif
+
+#ifdef _MSC_VER
+#define getcwd  _getcwd
 #endif
 
 #if !defined(_WIN32) || defined(__CYGWIN32__)
@@ -77,13 +87,21 @@ char boinc_failed_file[256];
 
 int is_file(const char* path) {
     struct stat sbuf;
+#ifdef _WIN32
     int retval = stat(path, &sbuf);
+#else
+    int retval = lstat(path, &sbuf);
+#endif
     return (!retval && (((sbuf.st_mode) & S_IFMT) == S_IFREG));
 }
 
 int is_dir(const char* path) {
     struct stat sbuf;
+#ifdef _WIN32
     int retval = stat(path, &sbuf);
+#else
+    int retval = lstat(path, &sbuf);
+#endif
     return (!retval && (((sbuf.st_mode) & S_IFMT) == S_IFDIR));
 }
 
@@ -180,6 +198,20 @@ void dir_close(DIRREF dirp) {
 #endif
 }
 
+bool is_dir_empty(const char *p) {
+    char file[256];
+
+    DIRREF dir = dir_open(p);
+    if (!dir) return true;
+
+    if (!dir_scan(file, dir, sizeof(file))) {
+        dir_close(dir);
+        return false;
+    }
+
+    return true;
+}
+
 DirScanner::DirScanner(string const& path) {
 #ifdef _WIN32
     first = true;
@@ -544,8 +576,7 @@ int boinc_copy(const char* orig, const char* newf) {
 
 static int boinc_rename_aux(const char* old, const char* newf) {
 #ifdef _WIN32
-    boinc_delete_file(newf);
-    if (MoveFileA(old, newf)) return 0;
+    if (MoveFileExA(old, newf, MOVEFILE_REPLACE_EXISTING|MOVEFILE_WRITE_THROUGH)) return 0;
     return GetLastError();
 #else
     int retval = rename(old, newf);
@@ -619,6 +650,7 @@ int boinc_make_dirs(const char* dirpath, const char* filepath) {
 
     if (strlen(filepath) + strlen(dirpath) > 1023) return ERR_BUFFER_OVERFLOW;
     strcpy(buf, filepath);
+    strcpy(oldpath, dirpath);
 
     q = buf;
     while(*q) {
@@ -679,15 +711,10 @@ int FILE_LOCK::lock(const char* filename) {
 }
 
 int FILE_LOCK::unlock(const char* filename) {
-    int retval = 0;
 #if defined(_WIN32) && !defined(__CYGWIN32__)
-    if (!CloseHandle(handle)) {
-        retval = GetLastError();
-    }
+    CloseHandle(handle);
 #else
-    if (close(fd)) {
-        retval = -1;
-    }
+    close(fd);
     fd = -1;
 #endif
     boinc_delete_file(filename);
