@@ -28,10 +28,15 @@
 #include <dirent.h>
 #include <grp.h>
 #include <stdio.h>
+#include <sys/types.h>
 #ifdef __cplusplus
 #include <string>
 #endif
 
+#ifndef MAXPATHLEN
+#define MAXPATHLEN 1024
+#endif
+
 #endif /* !WIN32 */
 
 #ifdef __cplusplus
@@ -49,7 +54,7 @@ extern "C" {
   extern int boinc_rmdir(const char*);
   extern void boinc_getcwd(char*);
   extern void relative_to_absolute(const char* relname, char* path);
-  extern int boinc_make_dirs(char*, char*);
+  extern int boinc_make_dirs(const char*, const char*);
   extern char boinc_failed_file[256];
   extern int is_file(const char* path);
   extern int is_dir(const char* path);
@@ -87,9 +92,10 @@ typedef DIR *DIRREF;
 
 extern DIRREF dir_open(const char*);
 extern int dir_scan(char*, DIRREF, int);
-int dir_scan(std::string&, DIRREF);
+extern int dir_scan(std::string&, DIRREF);
 extern void dir_close(DIRREF);
 
+extern bool is_dir_empty(const char*);
 
 class DirScanner {
 #if defined(_WIN32) && !defined(__CYGWIN32__)
