@@ -1,34 +1,33 @@
-// Berkeley Open Infrastructure for Network Computing
+// This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2005 University of California
+// Copyright (C) 2008 University of California
 //
-// This is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation;
-// either version 2.1 of the License, or (at your option) any later version.
+// BOINC is free software; you can redistribute it and/or modify it
+// under the terms of the GNU Lesser General Public License
+// as published by the Free Software Foundation,
+// either version 3 of the License, or (at your option) any later version.
 //
-// This software is distributed in the hope that it will be useful,
+// BOINC is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 // See the GNU Lesser General Public License for more details.
 //
-// To view the GNU Lesser General Public License visit
-// http://www.gnu.org/copyleft/lesser.html
-// or write to the Free Software Foundation, Inc.,
-// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+// You should have received a copy of the GNU Lesser General Public License
+// along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
 #ifndef _MIOFILE_
 #define _MIOFILE_
 
+#ifdef _USING_FCGI_
+#include "boinc_fcgi.h"
+#endif
+
 #ifndef _WIN32
 #include <string>
 #endif
 
 #include "mfile.h"
 
-#ifdef _USING_FCGI_
-#include "fcgi_stdio.h"
-#endif
 
 // MIOFILE lets you do formatted I/O to either a FILE or a memory buffer,
 // depending on how you initialize it.
@@ -55,19 +54,20 @@ public:
     MIOFILE();
     ~MIOFILE();
     void init_mfile(MFILE*);
+#ifndef _USING_FCGI_
     void init_file(FILE*);
+#else
+    void init_file(FCGI_FILE *);
+#endif
     void init_buf_read(const char*);
 	void init_buf_write(char*, int len);
     int printf(const char* format, ...);
     char* fgets(char*, int);
     int _ungetc(int);
+    bool eof();
     inline int _getc() {
         if (f) {
-#ifdef _USING_FCGI_
-            return FCGI_fgetc(f);
-#else
-            return getc(f);
-#endif
+            return fgetc(f);
         }
         return (*buf)?(*buf++):EOF;
     }
