@@ -15,11 +15,10 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
-
-#ifdef _BOINC_DLL
-#include "stdafx.h"
-#else
+#if   defined(_WIN32) && !defined(__STDWX_H__)
 #include "boinc_win.h"
+#elif defined(_WIN32) && defined(__STDWX_H__)
+#include "stdwx.h"
 #endif
 
 #include "diagnostics.h"
@@ -742,24 +741,35 @@ GetAccountSid(
     return bSuccess;
 }
 
-// Suspend or resume the threads in a given process.
+// signature of OpenThread()
+//
+typedef HANDLE (WINAPI *tOT)(
+    DWORD dwDesiredAccess, BOOL bInheritHandle, DWORD dwThreadId
+);
+
+// Suspend or resume the threads in a given process,
+// but don't suspend 'calling_thread'.
+//
 // The only way to do this on Windows is to enumerate
 // all the threads in the entire system,
 // and find those belonging to the process (ugh!!)
 //
 
-// OpenThread
-typedef HANDLE (WINAPI *tOT)(DWORD dwDesiredAccess, BOOL bInheritHandle, DWORD dwThreadId);
-
-int suspend_or_resume_threads(DWORD pid, bool resume) { 
+int suspend_or_resume_threads(
+    DWORD pid, DWORD calling_thread_id, bool resume
+) { 
     HANDLE threads, thread;
-    HMODULE hKernel32Lib = NULL;
+    static HMODULE hKernel32Lib = NULL;
     THREADENTRY32 te = {0}; 
-    tOT pOT = NULL;
+    static tOT pOT = NULL;
  
     // Dynamically link to the proper function pointers.
-    hKernel32Lib = GetModuleHandleA("kernel32.dll");
-    pOT = (tOT) GetProcAddress( hKernel32Lib, "OpenThread" );
+    if (!hKernel32Lib) {
+        hKernel32Lib = GetModuleHandleA("kernel32.dll");
+    }
+    if (!pOT) {
+        pOT = (tOT) GetProcAddress( hKernel32Lib, "OpenThread" );
+    }
 
     if (!pOT) {
         return -1;
@@ -776,6 +786,7 @@ int suspend_or_resume_threads(DWORD pid, bool resume) {
 
     do { 
         if (!diagnostics_is_thread_exempt_suspend(te.th32ThreadID)) continue;
+        if (te.th32ThreadID == calling_thread_id) continue;
         if (te.th32OwnerProcessID == pid) {
             thread = pOT(THREAD_SUSPEND_RESUME, FALSE, te.th32ThreadID);
             resume ?  ResumeThread(thread) : SuspendThread(thread);
@@ -837,7 +848,7 @@ void chdir_to_data_dir() {
 
 
 // return true if running under remote desktop
-// (in which case CUDA apps don't work)
+// (in which case CUDA and Stream apps don't work)
 //
 typedef BOOL (__stdcall *tWTSQSI)( IN HANDLE, IN DWORD, IN DWORD, OUT LPTSTR*, OUT DWORD* );
 typedef VOID (__stdcall *tWTSFM)( IN PVOID );
@@ -848,6 +859,7 @@ bool is_remote_desktop() {
     static tWTSFM pWTSFM = NULL;
     LPTSTR pBuf = NULL;
     DWORD dwLength;
+    USHORT usProtocol=-1, usConnectionState=-1;
 
     if (!wtsapi32lib) {
         wtsapi32lib = LoadLibrary(_T("wtsapi32.dll"));
@@ -857,14 +869,15 @@ bool is_remote_desktop() {
         }
     }
 
-    // WTSQuerySessionInformation(
-    //   WTS_CURRENT_SERVER_HANDLE,
-    //   WTS_CURRENT_SESSION,
-    //   WTSClientProtocolType,
-    //   &pBuf,
-    //   &dwLength
-    // );
     if (pWTSQSI) {
+
+        // WTSQuerySessionInformation(
+        //   WTS_CURRENT_SERVER_HANDLE,
+        //   WTS_CURRENT_SESSION,
+        //   WTSClientProtocolType,
+        //   &pBuf,
+        //   &dwLength
+        // );
         if (pWTSQSI(
             (HANDLE)NULL,
             (DWORD)-1,
@@ -872,10 +885,35 @@ bool is_remote_desktop() {
             &pBuf,
             &dwLength
         )) {
-            USHORT prot = *(USHORT*)pBuf;
+            usProtocol = *(USHORT*)pBuf;
             pWTSFM(pBuf);
-            if (prot == 2) return true;
         }
+
+        // WTSQuerySessionInformation(
+        //   WTS_CURRENT_SERVER_HANDLE,
+        //   WTS_CURRENT_SESSION,
+        //   WTSConnectState,
+        //   &pBuf,
+        //   &dwLength
+        // );
+        if (pWTSQSI(
+            (HANDLE)NULL,
+            (DWORD)-1,
+            (DWORD)8,
+            &pBuf,
+            &dwLength
+        )) {
+            usConnectionState = *(USHORT*)pBuf;
+            pWTSFM(pBuf);
+        }
+
+        // RDP Session implies Remote Desktop
+        if (usProtocol == 2) return true;
+
+        // Fast User Switching keeps the protocol set to the console but changes
+        // the connected state to disconnected.
+        if ((usProtocol == 0) && (usConnectionState == 4)) return true;
+
     }
 
     return false;
