@@ -16,37 +16,20 @@
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
 // Example multi-thread BOINC application.
-// It does 64 "units" of computation, where each units is about 1 GFLOP.
+// This app defines its own classes (THREAD, THREAD_SET) for managing threads.
+// You can also use libraries such as OpenMP.
+// Just make sure you call boinc_init_parallel().
+//
+// This app does 64 "units" of computation, where each units is about 1 GFLOP.
 // It divides this among N "worker" threads.
 // N is passed in the command line, and defaults to 1.
 //
-// The main issue is how to suspend/resume the threads.
-// The standard BOINC API doesn't work - it assumes that
-// the initial thread is the only one.
-// On Linux, there's no API to suspend/resume threads.
-// All you can do is SIGSTOP/SIGCONT, which affects the whole process.
-// So we use the following process/thread structure:
-//
-// Windows:
-// Initial thread:
-//  - launches worker threads,
-//  - in polling loop, checks for suspend/resume messages
-//    from the BOINC client, and handles them itself.
-// Unix:
-//  Initial process
-//    - forks worker process
-//    - in polling loop, checks for worker process completion
-//    - doesn't send status msgs
-//  Worker process
-//    Initial thread:
-//    - forks worker threads, wait for them to finish, exit
-//    - uses BOINC runtime to send status messages (frac done, CPU time)
-//
 // Doesn't do checkpointing.
 
 #include <stdio.h>
 #include <vector>
 #ifdef _WIN32
+#include "boinc_win.h"
 #else
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -60,8 +43,8 @@
 
 using std::vector;
 
-#define DEFAULT_NTHREADS 1
-#define TOTAL_UNITS 64
+#define DEFAULT_NTHREADS 4
+#define TOTAL_UNITS 16
 
 int units_per_thread;
 
@@ -85,6 +68,8 @@ struct THREAD {
     int units_done;
 
     THREAD(THREAD_FUNC func, int i) {
+        char buf[256];
+
         index = i;
         units_done = 0;
 #ifdef _WIN32
@@ -97,41 +82,26 @@ struct THREAD {
             NULL
         );
         if (!id) {
-            fprintf(stderr, "%s Can't start thread\n", boinc_msg_prefix());
+            fprintf(stderr, "%s Can't start thread\n",
+                boinc_msg_prefix(buf, sizeof(buf))
+            );
             exit(1);
         }
 #else
         int retval;
         retval = pthread_create(&id, 0, func, (void*)this);
         if (retval) {
-            fprintf(stderr, "%s can't start thread\n", boinc_msg_prefix());
+            fprintf(stderr, "%s can't start thread\n",
+                boinc_msg_prefix(buf, sizeof(buf))
+            );
             exit(1);
         }
 #endif
     }
-
-#ifdef _WIN32
-    void suspend(bool if_susp) {
-        if (if_susp) {
-            SuspendThread(id);
-        } else {
-            ResumeThread(id);
-        }
-    }
-#endif
 };
 
 struct THREAD_SET {
     vector<THREAD*> threads;
-#ifdef _WIN32
-    void suspend(bool if_susp) {
-        for (unsigned int i=0; i<threads.size(); i++) {
-            THREAD* t = threads[i];
-            if (t->id != THREAD_ID_NULL) t->suspend(if_susp);
-        }
-        fprintf(stderr, "%s suspended all\n", boinc_msg_prefix());
-    }
-#endif
     bool all_done() {
         for (unsigned int i=0; i<threads.size(); i++) {
             if (threads[i]->id != THREAD_ID_NULL) return false;
@@ -167,12 +137,13 @@ UINT WINAPI worker(void* p) {
 #else
 void* worker(void* p) {
 #endif
+    char buf[256];
     THREAD* t = (THREAD*)p;
     for (int i=0; i<units_per_thread; i++) {
         double x = do_a_giga_flop(i);
         t->units_done++;
         fprintf(stderr, "%s thread %d finished %d: %f\n",
-            boinc_msg_prefix(), t->index, i, x
+            boinc_msg_prefix(buf, sizeof(buf)), t->index, i, x
         );
     }
     t->id = THREAD_ID_NULL;
@@ -181,11 +152,25 @@ void* worker(void* p) {
 #endif
 }
 
-void main_thread(int nthreads) {
-    int i;
-#ifdef _WIN32
-    static BOINC_STATUS status;
-#endif
+int main(int argc, char** argv) {
+    int i, nthreads = DEFAULT_NTHREADS;
+    double start_time = dtime();
+    char buf[256];
+
+    boinc_init_parallel();
+
+    for (i=1; i<argc; i++) {
+        if (!strcmp(argv[i], "--nthreads")) {
+            nthreads = atoi(argv[++i]);
+        } else {
+            fprintf(stderr, "%s unrecognized arg: %s\n",
+                boinc_msg_prefix(buf, sizeof(buf)), argv[i]
+            );
+        }
+    }
+
+    units_per_thread = TOTAL_UNITS/nthreads;
+
     THREAD_SET thread_set;
     for (i=0; i<nthreads; i++) {
         thread_set.threads.push_back(new THREAD(worker, i));
@@ -194,80 +179,13 @@ void main_thread(int nthreads) {
         double f = thread_set.units_done()/((double)TOTAL_UNITS);
         boinc_fraction_done(f);
         if (thread_set.all_done()) break;
-#ifdef _WIN32
-        int old_susp = status.suspended;
-        boinc_get_status(&status);
-        if (status.quit_request || status.abort_request || status.no_heartbeat) {
-            exit(0);
-        }
-        if (status.suspended != old_susp) {
-            thread_set.suspend(status.suspended != 0);
-        }
-        boinc_sleep(0.1);
-#else
         boinc_sleep(1.0);
-#endif
     }
-}
-
-int main(int argc, char** argv) {
-    BOINC_OPTIONS options;
-    int nthreads = DEFAULT_NTHREADS;
-    double start_time = dtime();
-
-    boinc_options_defaults(options);
-    options.direct_process_action = 0;
-
-    for (int i=1; i<argc; i++) {
-        if (!strcmp(argv[i], "--nthreads")) {
-            nthreads = atoi(argv[++i]);
-        } else {
-            fprintf(stderr, "%s unrecognized arg: %s\n",
-                boinc_msg_prefix(), argv[i]
-            );
-        }
-    }
-
-    units_per_thread = TOTAL_UNITS/nthreads;
-
-#ifdef _WIN32
-    boinc_init_options(&options);
-    main_thread(nthreads);
-#else
-    options.send_status_msgs = 0;
-    boinc_init_options(&options);
-    int pid = fork();
-    if (pid) {          // parent
-        BOINC_STATUS status;
-        boinc_get_status(&status);
-        int exit_status;
-        while (1) {
-            bool old_susp = status.suspended;
-            boinc_get_status(&status);
-            if (status.quit_request || status.abort_request || status.no_heartbeat) {
-                kill(pid, SIGKILL);
-                exit(0);
-            }
-            if (status.suspended != old_susp) {
-                kill(pid, status.suspended?SIGSTOP:SIGCONT);
-            }
-            if (waitpid(pid, &exit_status, WNOHANG) == pid) {
-                break;
-            }
-            boinc_sleep(0.1);
-        }
-    } else {            // child (worker)
-        memset(&options, 0, sizeof(options));
-        options.send_status_msgs = 1;
-        boinc_init_options(&options);
-        main_thread(nthreads);
-    }
-#endif
 
     double elapsed_time = dtime()-start_time;
     fprintf(stderr,
         "%s All done.  Used %d threads.  Elapsed time %f\n",
-        boinc_msg_prefix(), nthreads, elapsed_time
+        boinc_msg_prefix(buf, sizeof(buf)), nthreads, elapsed_time
     );
     boinc_finish(0);
 }
