@@ -47,11 +47,12 @@
 #include "filesys.h"
 #include "parse.h"
 #include "str_util.h"
+#include "str_replace.h"
 #include "util.h"
 #include "error_numbers.h"
 
 #define JOB_FILENAME "job.xml"
-#define CHECKPOINT_FILENAME "checkpoint.txt"
+#define CHECKPOINT_FILENAME "wrapper_checkpoint.txt"
 
 #define POLL_PERIOD 1.0
 
@@ -65,6 +66,8 @@ struct TASK {
     string stderr_filename;
     string checkpoint_filename;
         // name of task's checkpoint file, if any
+    string fraction_done_filename;
+        // name of file where app will write its fraction done
     string command_line;
     double weight;
         // contribution of this task to overall fraction done
@@ -104,6 +107,18 @@ struct TASK {
         last_stat.st_mtime = new_stat.st_mtime;
         return changed;
     }
+    inline double fraction_done() {
+        if (fraction_done_filename.size() == 0) return 0;
+        FILE* f = fopen(fraction_done_filename.c_str(), "r");
+        if (!f) return 0;
+        double frac;
+        int n = fscanf(f, "%lf", &frac);
+        fclose(f);
+        if (n != 1) return 0;
+        if (frac < 0) return 0;
+        if (frac > 1) return 1;
+        return frac;
+    }
 };
 
 vector<TASK> tasks;
@@ -120,7 +135,7 @@ int TASK::parse(XML_PARSER& xp) {
     while (!xp.get(tag, sizeof(tag), is_tag)) {
         if (!is_tag) {
             fprintf(stderr, "%s TASK::parse(): unexpected text %s\n",
-                boinc_msg_prefix(), tag
+                boinc_msg_prefix(buf, sizeof(buf)), tag
             );
             continue;
         }
@@ -143,6 +158,7 @@ int TASK::parse(XML_PARSER& xp) {
             continue;
         }
         else if (xp.parse_string(tag, "checkpoint_filename", checkpoint_filename)) continue;
+        else if (xp.parse_string(tag, "fraction_done_filename", fraction_done_filename)) continue;
         else if (xp.parse_double(tag, "weight", weight)) continue;
     }
     return ERR_XML_PARSE;
@@ -150,13 +166,16 @@ int TASK::parse(XML_PARSER& xp) {
 
 int parse_job_file() {
     MIOFILE mf;
-    char tag[1024], buf[256];
+    char tag[1024], buf[256], buf2[256];
     bool is_tag;
 
     boinc_resolve_filename(JOB_FILENAME, buf, 1024);
     FILE* f = boinc_fopen(buf, "r");
     if (!f) {
-        fprintf(stderr, "%s can't open job file %s\n", boinc_msg_prefix(), buf);
+        fprintf(stderr,
+            "%s can't open job file %s\n",
+            boinc_msg_prefix(buf2, sizeof(buf2)), buf
+        );
         return ERR_FOPEN;
     }
     mf.init_file(f);
@@ -165,8 +184,9 @@ int parse_job_file() {
     if (!xp.parse_start("job_desc")) return ERR_XML_PARSE;
     while (!xp.get(tag, sizeof(tag), is_tag)) {
         if (!is_tag) {
-            fprintf(stderr, "%s SCHED_CONFIG::parse(): unexpected text %s\n",
-                boinc_msg_prefix(), tag
+            fprintf(stderr,
+                "%s SCHED_CONFIG::parse(): unexpected text %s\n",
+                boinc_msg_prefix(buf2, sizeof(buf2)), tag
             );
             continue;
         }
@@ -243,6 +263,13 @@ int TASK::run(int argct, char** argvt) {
     string stdout_path, stdin_path, stderr_path;
     char app_path[1024], buf[256];
 
+    if (checkpoint_filename.size()) {
+        boinc_delete_file(checkpoint_filename.c_str());
+    }
+    if (fraction_done_filename.size()) {
+        boinc_delete_file(fraction_done_filename.c_str());
+    }
+
     strcpy(buf, application.c_str());
     char* p = strstr(buf, "$PROJECT_DIR");
     if (p) {
@@ -260,7 +287,7 @@ int TASK::run(int argct, char** argvt) {
     }
 
     fprintf(stderr, "%s wrapper: running %s (%s)\n",
-        boinc_msg_prefix(), app_path, command_line.c_str()
+        boinc_msg_prefix(buf, sizeof(buf)), app_path, command_line.c_str()
     );
 
 #ifdef _WIN32
@@ -303,6 +330,9 @@ int TASK::run(int argct, char** argvt) {
         &startup_info,
         &process_info
     )) {
+        char error_msg[1024];
+        windows_error_string(error_msg, sizeof(error_msg));
+        fprintf(stderr, "can't run app: %s\n", error_msg);
         return ERR_EXEC;
     }
     pid_handle = process_info.hProcess;
@@ -320,7 +350,8 @@ int TASK::run(int argct, char** argvt) {
 
     pid = fork();
     if (pid == -1) {
-        boinc_finish(ERR_FORK);
+        perror("fork(): ");
+        return ERR_FORK;
     }
     if (pid == 0) {
 		// we're in the child process here
@@ -353,6 +384,7 @@ int TASK::run(int argct, char** argvt) {
         argc = parse_command_line(arglist, argv+1);
         setpriority(PRIO_PROCESS, 0, PROCESS_IDLE_PRIORITY);
         retval = execv(app_path, argv);
+        perror("execv() failed: ");
         exit(ERR_EXEC);
     }
 #endif
@@ -395,7 +427,7 @@ void TASK::kill() {
 
 void TASK::stop() {
 #ifdef _WIN32
-    suspend_or_resume_threads(pid, false);
+    suspend_or_resume_threads(pid, 0, false);
 #else
     ::kill(pid, SIGSTOP);
 #endif
@@ -404,7 +436,7 @@ void TASK::stop() {
 
 void TASK::resume() {
 #ifdef _WIN32
-    suspend_or_resume_threads(pid, true);
+    suspend_or_resume_threads(pid, 0, true);
 #else
     ::kill(pid, SIGCONT);
 #endif
@@ -467,33 +499,33 @@ void send_status_message(
 // We keep a checkpoint file that says how many tasks we've completed
 // and how much CPU time has been used so far
 //
-void write_checkpoint(int ntasks, double cpu) {
+void write_checkpoint(int ntasks_completed, double cpu) {
     FILE* f = fopen(CHECKPOINT_FILENAME, "w");
     if (!f) return;
-    fprintf(f, "%d %f\n", ntasks, cpu);
+    fprintf(f, "%d %f\n", ntasks_completed, cpu);
     fclose(f);
 }
 
-void read_checkpoint(int& ntasks, double& cpu) {
+void read_checkpoint(int& ntasks_completed, double& cpu) {
     int nt;
     double c;
 
-    ntasks = 0;
+    ntasks_completed = 0;
     cpu = 0;
     FILE* f = fopen(CHECKPOINT_FILENAME, "r");
     if (!f) return;
     int n = fscanf(f, "%d %lf", &nt, &c);
     fclose(f);
     if (n != 2) return;
-    ntasks = nt;
+    ntasks_completed = nt;
     cpu = c;
 }
 
 int main(int argc, char** argv) {
     BOINC_OPTIONS options;
-    int retval, ntasks;
+    int retval, ntasks_completed;
     unsigned int i;
-    double total_weight=0, w=0;
+    double total_weight=0, weight_completed=0;
     double checkpoint_cpu_time;
         // overall CPU time at last checkpoint
 
@@ -522,9 +554,12 @@ int main(int argc, char** argv) {
         boinc_finish(retval);
     }
 
-    read_checkpoint(ntasks, checkpoint_cpu_time);
-    if (ntasks > (int)tasks.size()) {
-        fprintf(stderr, "Checkpoint file: ntasks %d too large\n", ntasks);
+    read_checkpoint(ntasks_completed, checkpoint_cpu_time);
+    if (ntasks_completed > (int)tasks.size()) {
+        fprintf(stderr,
+            "Checkpoint file: ntasks_completed too large: %d > %d\n",
+            ntasks_completed, (int)tasks.size()
+        );
         boinc_finish(1);
     }
     for (i=0; i<tasks.size(); i++) {
@@ -532,17 +567,18 @@ int main(int argc, char** argv) {
     }
     for (i=0; i<tasks.size(); i++) {
         TASK& task = tasks[i];
-        w += task.weight;
-        if ((int)i<ntasks) continue;
-        double frac_done = w/total_weight;
+        if ((int)i<ntasks_completed) {
+            weight_completed += task.weight;
+            continue;
+        }
+        double frac_done = weight_completed/total_weight;
 
         task.starting_cpu = checkpoint_cpu_time;
         retval = task.run(argc, argv);
         if (retval) {
-            fprintf(stderr, "can't run app: %d\n", retval);
             boinc_finish(retval);
         }
-        while(1) {
+        while (1) {
             int status;
             if (task.poll(status)) {
                 if (status) {
@@ -558,7 +594,9 @@ int main(int argc, char** argv) {
                 break;
             }
             poll_boinc_messages(task);
-            send_status_message(task, frac_done, checkpoint_cpu_time);
+            double task_fraction_done = task.fraction_done();
+            double delta = task_fraction_done*task.weight/total_weight;
+            send_status_message(task, frac_done+delta, checkpoint_cpu_time);
             if (task.has_checkpointed()) {
                 checkpoint_cpu_time = task.starting_cpu + task.cpu_time();
                 write_checkpoint(i, checkpoint_cpu_time);
@@ -567,6 +605,7 @@ int main(int argc, char** argv) {
         }
         checkpoint_cpu_time = task.starting_cpu + task.final_cpu_time;
         write_checkpoint(i+1, checkpoint_cpu_time);
+        weight_completed += task.weight;
     }
     boinc_finish(0);
 }
