@@ -29,6 +29,7 @@
 //
 #define ANTIQUE_DELAY       3600
 #define ANTIQUE_INTERVAL    86400
+#define ANTIQUE_LIMIT       50000
 
 // how often to retry errors
 //
@@ -57,6 +58,7 @@
 #include "str_replace.h"
 #include "filesys.h"
 #include "strings.h"
+#include "svn_version.h"
 
 #include "sched_config.h"
 #include "sched_util.h"
@@ -73,57 +75,48 @@ bool dont_retry_errors = false;
 bool dont_delete_antiques = false;
 bool dont_delete_batches = false;
 int antique_delay = ANTIQUE_DELAY;
+int antique_interval = ANTIQUE_INTERVAL;
+int antique_limit = ANTIQUE_LIMIT;
 bool do_input_files = true;
 bool do_output_files = true;
 int sleep_interval = DEFAULT_SLEEP_INTERVAL;
 
-void usage() {
-    fprintf(stderr,
-"file_deleter: deletes files that are no longer needed.\n"
-"\n"
-"default operation:\n"
-"1) enumerate N WUs and M results (N,M compile params)\n"
-"   that are ready to file-delete, and try to delete their files\n"
-"2) if the enums didn't yield anything, sleep for K seconds\n"
-"3) repeat from 1)\n"
-"4) every 1 hour, enumerate everything in state FILE_DELETE_ERROR\n"
-"   and try to delete it.\n"
-"5) after 1 hour, and every 24 hours thereafter,\n"
-"   scan for and delete all files in the upload directories\n"
-"   that are older than any WU in the database,\n"
-"   and were created at least one month ago.\n"
-"   This deletes files uploaded by hosts after the WU was deleted.\n"
-"\n"
-"options:\n"
-"\n"
-"-d N\n"
-"     set debug output level (1/2/3)\n"
-"-mod M R\n"
-"     handle only WUs with ID mod M == R\n"
-"-appid ID\n"
-"     handle only WUs of a particular app\n"
-"-one_pass\n"
-"     instead of sleeping in 2), exit\n"
-"-delete_antiques_now\n"
-"     do 5) immediately\n"
-"-dont_retry_error\n"
-"     don't do 4)\n"
-"-dont_delete_antiques\n"
-"     don't do 5)\n"
-"-preserve_result_files\n"
-"     update the DB, but don't delete output files.\n"
-"     For debugging.\n"
-"-preserve_wu_files\n"
-"     update the DB, but don't delete input files.\n"
-"     For debugging.\n"
-"-dont_delete_batches\n"
-"     don't delete anything with positive batch number\n"
-"-input_files_only\n"
-"     delete only input (download) files\n"
-"-output_files_only\n"
-"     delete only output (upload) files\n"
+void usage(char *name) {
+    fprintf(stderr, "Deletes files that are no longer needed.\n\n"
+        "Default operation:\n"
+        "1) enumerate N WUs and M results (N,M compile params)\n"
+        "   that are ready to file-delete, and try to delete their files\n"
+        "2) if the enums didn't yield anything, sleep for K seconds\n"
+        "3) repeat from 1)\n"
+        "4) every 1 hour, enumerate everything in state FILE_DELETE_ERROR\n"
+        "   and try to delete it.\n"
+        "5) after 1 hour, and every 24 hours thereafter,\n"
+        "   scan for and delete all files in the upload directories\n"
+        "   that are older than any WU in the database,\n"
+        "   and were created at least one month ago.\n"
+        "   This deletes files uploaded by hosts after the WU was deleted.\n\n"
+        "Usage: %s [OPTION]...\n\n"
+        "Options:\n"
+        "  -d N | --debug_level N          set debug output level (1 to 4)\n"
+        "  --mod M R                       handle only WUs with ID mod M == R\n"
+        "  --appid ID                      handle only WUs of a particular app\n"
+        "  --one_pass                      instead of sleeping in 2), exit\n"
+        "  --delete_antiques_now           do 5) immediately\n"
+        "  --dont_retry_error              don't do 4)\n"
+        "  --dont_delete_antiques          don't do 5)\n"
+        "  --delete_antiques_interval      change the interval between delete antique passes (in seconds, defaults to 24h)\n"
+        "  --delete_antiques_limit         change the maximum number of files deleted in one delete antique pass (defaults to 50000)\n"
+        "  --preserve_result_files         update the DB, but don't delete output files.\n"
+        "                                  For debugging.\n"
+        "  --preserve_wu_files             update the DB, but don't delete input files.\n"
+        "                                 For debugging.\n"
+        "  --dont_delete_batches           don't delete anything with positive batch number\n"
+        "  --input_files_only              delete only input (download) files\n"
+        "  --output_files_only             delete only output (upload) files\n"
+        "  [ -h | --help ]                 shows this help text\n"
+        "  [ -v | --version ]              shows version information\n",
+        name
     );
-    exit(1);
 }
 
 // Given a filename, find its full path in the upload directory hierarchy
@@ -133,7 +126,7 @@ void usage() {
 int get_file_path(
     const char *filename, char* upload_dir, int fanout, char* path
 ) {
-    dir_hier_path(filename, upload_dir, fanout, path);
+    dir_hier_path(filename, upload_dir, fanout, path, true);
     if (boinc_file_exists(path)) {
         return 0;
     }
@@ -196,6 +189,21 @@ int wu_delete_files(WORKUNIT& wu) {
                     } else {
                         count_deleted++;
                     }
+                    // delete the cached MD5 file if needed
+                    //
+                    if (config.cache_md5_info) {
+                        strcat(pathname,".md5");
+                        log_messages.printf(MSG_NORMAL,
+                            "[WU#%d] deleting %s\n", wu.id, filename
+                        );
+                        retval = unlink(pathname);
+                        if (retval) {
+                            log_messages.printf(MSG_CRITICAL,
+                                "[WU#%d] unlink %s failed: %d\n",
+                                wu.id, filename, retval
+                            );
+                        }
+                    }
                 }
             }
         }
@@ -281,7 +289,7 @@ int result_delete_files(RESULT& result) {
 static bool preserve_wu_files=false;
 static bool preserve_result_files=false;
 
-// return nonzero if did anything
+// return true if we changed the file_delete_state of a WU or a result
 //
 bool do_pass(bool retry_error) {
     DB_WORKUNIT wu;
@@ -289,7 +297,7 @@ bool do_pass(bool retry_error) {
     bool did_something = false;
     char buf[256];
     char clause[256];
-    int retval;
+    int retval, new_state;
 
     check_stop_daemons();
 
@@ -319,30 +327,33 @@ bool do_pass(bool retry_error) {
             }
             break;
         }
-        did_something = true;
 
-        retval = 0;
-        if (!preserve_wu_files) {
+        if (preserve_wu_files) {
+            retval = 0;
+        } else {
             retval = wu_delete_files(wu);
         }
         if (retval) {
-            wu.file_delete_state = FILE_DELETE_ERROR;
+            new_state = FILE_DELETE_ERROR;
             log_messages.printf(MSG_CRITICAL,
                 "[WU#%d] file deletion failed: %d\n", wu.id, retval
             );
         } else {
-            wu.file_delete_state = FILE_DELETE_DONE;
+            new_state = FILE_DELETE_DONE;
         }
-        sprintf(buf, "file_delete_state=%d", wu.file_delete_state);
-        retval = wu.update_field(buf);
-        if (retval) {
-            log_messages.printf(MSG_CRITICAL,
-                "[WU#%d] update failed: %d\n", wu.id, retval
-            );
-        } else {
-            log_messages.printf(MSG_DEBUG,
-                "[WU#%d] file_delete_state updated\n", wu.id
-            );
+        if (new_state != wu.file_delete_state) {
+            sprintf(buf, "file_delete_state=%d", new_state);
+            retval = wu.update_field(buf);
+            if (retval) {
+                log_messages.printf(MSG_CRITICAL,
+                    "[WU#%d] update failed: %d\n", wu.id, retval
+                );
+            } else {
+                log_messages.printf(MSG_DEBUG,
+                    "[WU#%d] file_delete_state updated\n", wu.id
+                );
+                did_something = true;
+            }
         } 
     }
 
@@ -362,30 +373,32 @@ bool do_pass(bool retry_error) {
             break;
         }
 
-        did_something = true;
-        retval = 0;
-        if (!preserve_result_files) {
+        if (preserve_result_files) {
+            retval = 0;
+        } else {
             retval = result_delete_files(result);
         }
         if (retval) {
-            result.file_delete_state = FILE_DELETE_ERROR;
+            new_state = FILE_DELETE_ERROR;
             log_messages.printf(MSG_CRITICAL,
                 "[RESULT#%d] file deletion failed: %d\n", result.id, retval
             );
         } else {
-            result.file_delete_state = FILE_DELETE_DONE;
+            new_state = FILE_DELETE_DONE;
         }
-        sprintf(buf, "file_delete_state=%d", result.file_delete_state); 
-        retval = result.update_field(buf);
-        retval = result.update_field(buf);
-        if (retval) {
-            log_messages.printf(MSG_CRITICAL,
-                "[RESULT#%d] update failed: %d\n", result.id, retval
-            );
-        } else {
-            log_messages.printf(MSG_DEBUG,
-                "[RESULT#%d] file_delete_state updated\n", result.id
-            );
+        if (new_state != result.file_delete_state) {
+            sprintf(buf, "file_delete_state=%d", new_state); 
+            retval = result.update_field(buf);
+            if (retval) {
+                log_messages.printf(MSG_CRITICAL,
+                    "[RESULT#%d] update failed: %d\n", result.id, retval
+                );
+            } else {
+                log_messages.printf(MSG_DEBUG,
+                    "[RESULT#%d] file_delete_state updated\n", result.id
+                );
+                did_something = true;
+            }
         } 
     } 
 
@@ -423,7 +436,8 @@ int delete_antique_files() {
     int nfiles=0;
 
     log_messages.printf(MSG_DEBUG,
-        "delete_antique_files(): start (%d files)\n", files_to_delete.size()
+        "delete_antique_files(): start (%d files)\n",
+        (int)files_to_delete.size()
     );
     while (!files_to_delete.empty()) {
         char timestamp[128];
@@ -482,7 +496,7 @@ int add_antiques_to_list(int days) {
 
     if (!apache_info) {
         log_messages.printf(MSG_CRITICAL,
-            "httpd_user'%s' found - add <httpd_user> entry in config.xml\n",
+            "default httpd_user '%s' found - add <httpd_user> entry in config.xml\n",
             config.httpd_user
         );
         return -1;
@@ -491,7 +505,7 @@ int add_antiques_to_list(int days) {
         "Searching for antique files older than %d days\n", days
     );
 
-    sprintf(command,  "find %s -type f -mtime +%d -follow", config.upload_dir, days);
+    sprintf(command, "find %s -type f -mtime +%d -follow | head -%d", config.upload_dir, days, antique_limit);
     
     // Now execute the command, read output on a stream.  We could use
     // find to also exec a 'delete' command.  But we want to log all
@@ -628,44 +642,78 @@ int main(int argc, char** argv) {
     int retval;
     bool one_pass = false;
     int i;
-
+    DB_APP app;
+    
     check_stop_daemons();
+
+    *app.name='\0';
     for (i=1; i<argc; i++) {
-        if (!strcmp(argv[i], "-one_pass")) {
+        if (is_arg(argv[i], "one_pass")) {
             one_pass = true;
-        } else if (!strcmp(argv[i], "-dont_retry_errors")) {
+        } else if (is_arg(argv[i], "dont_retry_errors")) {
             dont_retry_errors = true;
-        } else if (!strcmp(argv[i], "-preserve_wu_files")) {
+        } else if (is_arg(argv[i], "preserve_wu_files")) {
             preserve_wu_files = true;
-        } else if (!strcmp(argv[i], "-preserve_result_files")) {
+        } else if (is_arg(argv[i], "preserve_result_files")) {
             preserve_result_files = true;
-        } else if (!strcmp(argv[i], "-appid")) {
-            appid = atoi(argv[++i]);
-        } else if (!strcmp(argv[i], "-d")) {
-            log_messages.set_debug_level(atoi(argv[++i]));
-        } else if (!strcmp(argv[i], "-mod")) {
+        } else if (is_arg(argv[i], "app")) {
+            strcpy(app.name, argv[++i]);
+        } else if (is_arg(argv[i], "appid")) {
+            if (!argv[++i]) {
+                log_messages.printf(MSG_CRITICAL, "%s requires an argument\n\n", argv[--i]);
+                usage(argv[0]);
+                exit(1);
+            }
+            appid = atoi(argv[i]);
+        } else if (is_arg(argv[i], "d") || is_arg(argv[i], "debug_level")) {
+            if (!argv[++i]) {
+                log_messages.printf(MSG_CRITICAL, "%s requires an argument\n\n", argv[--i]);
+                usage(argv[0]);
+                exit(1);
+            }
+            int dl = atoi(argv[i]);
+            log_messages.set_debug_level(dl);
+            if (dl == 4) g_print_queries = true;
+        } else if (is_arg(argv[i], "mod")) {
+            if (!argv[i+1] || !argv[i+2]) {
+                log_messages.printf(MSG_CRITICAL, "%s requires two arguments\n\n", argv[i]);
+                usage(argv[0]);
+                exit(1);
+            }
             id_modulus   = atoi(argv[++i]);
             id_remainder = atoi(argv[++i]);
-        } else if (!strcmp(argv[i], "-dont_delete_antiques")) {
+        } else if (is_arg(argv[i], "dont_delete_antiques")) {
             dont_delete_antiques = true;
-        } else if (!strcmp(argv[i], "-dont_delete_batches")) {
+        } else if (is_arg(argv[i], "delete_antiques_interval")) {
+            antique_interval = atoi(argv[++i]);
+        } else if (is_arg(argv[i], "delete_antiques_limit")) {
+            antique_limit = atoi(argv[++i]);
+        } else if (is_arg(argv[i], "dont_delete_batches")) {
             dont_delete_batches = true;
-        } else if (!strcmp(argv[i], "-delete_antiques_now")) {
+        } else if (is_arg(argv[i], "delete_antiques_now")) {
             antique_delay = 0;
-        } else if (!strcmp(argv[i], "-input_files_only")) {
+        } else if (is_arg(argv[i], "input_files_only")) {
             do_output_files = false;
             dont_delete_antiques = true;
-        } else if (!strcmp(argv[i], "-output_files_only")) {
+        } else if (is_arg(argv[i], "output_files_only")) {
             do_input_files = false;
-        } else if (!strcmp(argv[i], "-sleep_interval")) {
-            sleep_interval = atoi(argv[++i]);
-        } else if (!strcmp(argv[i], "-help")) {
-            usage();
+        } else if (is_arg(argv[i], "sleep_interval")) {
+            if (!argv[++i]) {
+                log_messages.printf(MSG_CRITICAL, "%s requires an argument\n\n", argv[--i]);
+                usage(argv[0]);
+                exit(1);
+            }
+            sleep_interval = atoi(argv[i]);
+        } else if (is_arg(argv[i], "h") || is_arg(argv[i], "help")) {
+            usage(argv[0]);
+            exit(0);
+        } else if (is_arg(argv[i], "v") || is_arg(argv[i], "version")) {
+            printf("%s\n", SVN_VERSION);
+            exit(0);
         } else {
-            log_messages.printf(MSG_CRITICAL,
-                "Unrecognized arg: %s\n", argv[i]
-            );
-            usage();
+            log_messages.printf(MSG_CRITICAL, "unknown command line argument: %s\n\n", argv[i]);
+            usage(argv[0]);
+            exit(1);
         }
     }
 
@@ -697,6 +745,19 @@ int main(int argc, char** argv) {
             "boinc_db.set_isolation_level: %d; %s\n", retval, boinc_db.error_string()
         );
     }
+
+    if (*app.name && !appid) {
+      char buf[256];      
+      sprintf(buf, "where name='%s'", app.name);
+      retval = app.lookup(buf);
+      if (retval) {
+        log_messages.printf(MSG_CRITICAL, "Can't find app\n");
+        exit(1);
+      }
+      appid=app.id;
+      log_messages.printf(MSG_DEBUG, "Deleting files of appid %d\n",appid);
+    }
+
     install_stop_signal_handler();
 
     bool retry_errors_now = !dont_retry_errors;
@@ -725,7 +786,7 @@ int main(int argc, char** argv) {
                 "Doing antique deletion pass\n"
             );
             do_antique_pass();
-            next_antique_time = dtime() + ANTIQUE_INTERVAL;
+            next_antique_time = dtime() + antique_interval;
         }
         if (!dont_retry_errors && !retry_errors_now && (dtime() > next_error_time)) {
             retry_errors_now = true;
@@ -736,4 +797,4 @@ int main(int argc, char** argv) {
     }
 }
 
-const char *BOINC_RCSID_bd0d4938a6 = "$Id: file_deleter.cpp 18952 2009-08-31 20:16:46Z romw $";
+const char *BOINC_RCSID_bd0d4938a6 = "$Id: file_deleter.cpp 22083 2010-07-31 04:08:14Z davea $";
