@@ -29,6 +29,7 @@
 #include <unistd.h>
 #include <csignal>
 #include <fcntl.h>
+#include <string>
 
 #ifdef _USING_FCGI_
 #include "boinc_fcgi.h"
@@ -42,6 +43,7 @@
 #include "error_numbers.h"
 #include "str_util.h"
 #include "filesys.h"
+#include "svn_version.h"
 
 #include "sched_config.h"
 #include "sched_util.h"
@@ -611,7 +613,21 @@ void installer() {
     signal(SIGTERM, boinc_catch_signal); // terminate process
 }
 
-int main() {
+void usage(char *name) {
+    fprintf(stderr,
+        "This is the BOINC file upload handler.\n"
+        "It receives the results from the clients\n"
+        "and puts them on the file server.\n\n"
+        "Normally this is run as a CGI program.\n\n"
+        "Usage: %s [OPTION]...\n\n"
+        "Options:\n"
+        "  [ -h | --help ]        Show this help text.\n"
+        "  [ -v | --version ]     Show version information.\n",
+        name
+    );
+}
+
+int main(int argc, char *argv[]) {
     int retval;
     R_RSA_PUBLIC_KEY key;
     char log_path[256];
@@ -619,6 +635,21 @@ int main() {
     unsigned int counter=0;
 #endif
 
+    for(int c = 1; c < argc; c++) {
+        std::string option(argv[c]);
+        if(option == "-v" || option == "--version") {
+            printf("%s\n", SVN_VERSION);
+            exit(0);
+        } else if(option == "-h" || option == "--help") {
+            usage(argv[0]);
+            exit(0);
+        } else if (option.length()){
+            fprintf(stderr, "unknown command line argument: %s\n\n", argv[c]);
+            usage(argv[0]);
+            exit(1);
+        }
+    }
+
     installer();
 
     get_log_path(log_path, "file_upload_handler.log");
@@ -686,4 +717,4 @@ int main() {
     return 0;
 }
 
-const char *BOINC_RCSID_470a0d4d11 = "$Id: file_upload_handler.cpp 18786 2009-07-31 19:46:47Z davea $";
+const char *BOINC_RCSID_470a0d4d11 = "$Id: file_upload_handler.cpp 21181 2010-04-15 03:13:56Z davea $";
