@@ -21,6 +21,6 @@
     // actually ntypes+1 (0 is reserved for "no HR")
 
 extern int hr_class(HOST&, int hr_type);
-extern bool hr_unknown_platform_type(HOST&, int hr_type);
+extern bool hr_unknown_class(HOST&, int hr_type);
 extern const char* hr_names[HR_NTYPES];
 extern int hr_nclasses[HR_NTYPES];
