@@ -18,16 +18,29 @@
 #ifndef _SCHED_CONFIG_
 #define _SCHED_CONFIG_
 
-#include "regex.h"
+#include <regex.h>
 #include <vector>
 #include <cstdio>
 
+#include "sched_limit.h"
+
 using std::vector;
 
-// parsed version of server configuration file
+// constants related to consecutive_valid.
+// These could be made configurable.
+
+#define CONS_VALID_RELIABLE 10
+    // host is eligible to be considered "reliable"
+#define CONS_VALID_HOST_SCALE 10
+    // host is eligible for host scaling of credit
+#define CONS_VALID_UNREPLICATED 10
+    // host is eligible for single replication
+
+// server configuration
+// MISNAMED - should be SERVER_CONFIG,
+// and should factor out scheduler-specific stuff into SCHED_CONFIG
 //
-class SCHED_CONFIG {
-public:
+struct SCHED_CONFIG {
     char master_url[256];
     char long_name[256];
     char db_name[256];
@@ -41,116 +54,127 @@ public:
     char download_dir[256];
     char upload_url[256];
     char upload_dir[256];
-    char sched_lockfile_dir[256];
     int grace_period_hours;
     int delete_delay_hours;
-    bool one_result_per_user_per_wu;
-    bool one_result_per_host_per_wu;
     bool msg_to_host;
-    int min_sendwork_interval;
-    int max_wus_in_progress;
-    int max_wus_in_progress_gpu;
     bool non_cpu_intensive;
     bool verify_files_on_app_start;
     int homogeneous_redundancy;
-    bool locality_scheduling;
-    bool locality_scheduling_sorted_order;
     bool ignore_upload_certificates;
     bool dont_generate_upload_certificates;
-    bool ignore_delay_bound;
-    int gpu_multiplier;             // mult is NCPUS + this*NGPUS
-    int daily_result_quota;         // max results per day is this * mult
-    int max_wus_to_send;            // max results per RPC is this * mult
     int uldl_dir_fanout;        // fanout of ul/dl dirs; 0 if none
     int uldl_dir_levels;
-    int locality_scheduling_wait_period;
-    int locality_scheduling_send_timeout;
-    vector<regex_t> *locality_scheduling_workunit_file;
-    vector<regex_t> *locality_scheduling_sticky_file;
-    double locality_scheduler_fraction;
-    int min_core_client_version;
-    int min_core_client_version_announced;
-    int min_core_client_upgrade_deadline;
-    char replace_download_url_by_timezone[256];
     bool cache_md5_info;
-    bool nowork_skip;
-    bool resend_lost_results;
-    bool send_result_abort;
     bool use_benchmark_weights;
     double fp_benchmark_weight;
-    double default_disk_max_used_gb;
-    double default_disk_max_used_pct;
-    double default_disk_min_free_gb;
-    char symstore[256];
-    double next_rpc_delay;
-    int sched_debug_level;
     int fuh_debug_level;
-    double reliable_max_error_rate;  // max error rate for a host to be declared reliable
-    int reliable_max_avg_turnaround;
-        // max average turnaround for a host to be declared reliable
     int reliable_priority_on_over;
         // additional results generated after at least one result
         // is over will have their priority boosted by this amount    
     int reliable_priority_on_over_except_error;
         // additional results generated after at least one result is over
-        // (unless their is an error) will have their priority boosted
+        // (unless there is an error) will have their priority boosted
         // by this amount
     int reliable_on_priority;
         // results with a priority equal or greater than this value
         // will be sent to reliable hosts
-    double reliable_reduced_delay_bound;
-        // Reduce the delay bounds for reliable hosts by this percent
-    int granted_credit_ramp_up; 
-    double granted_credit_weight;
     bool distinct_beta_apps;
         // allow users to select beta apps independently
-    bool workload_sim;
-        // Do workload simulation in deciding whether to send a result
     bool ended;
         // Project has ended - tell clients to detach
     int shmem_work_items;
         // number of work items in shared memory
     int feeder_query_size;
         // number of work items to request in each feeder query
-    bool no_amd_k6;
-        // don't allow AMD K6 CPUs
     char httpd_user[256];
         // user name under which web server runs (default: apache)
+    bool enable_assignment;
+    bool job_size_matching;
+
+    //////////// STUFF RELEVANT ONLY TO SCHEDULER FOLLOWS ///////////
+
+    vector<regex_t> *ban_cpu;
+    vector<regex_t> *ban_os;
+    int daily_result_quota;         // max results per day is this * mult
+    double default_disk_max_used_gb;
+    double default_disk_max_used_pct;
+    double default_disk_min_free_gb;
+    bool dont_store_success_stderr;
     int file_deletion_strategy;
         // select method of automatically deleting files from host
-    bool request_time_stats_log;
-    bool enable_assignment;
-    int max_ncpus;
-    vector<regex_t> *ban_os;
-    vector<regex_t> *ban_cpu;
+    int gpu_multiplier;             // mult is NCPUS + this*NGPUS
+    bool ignore_delay_bound;
+    bool locality_scheduling;
+    double locality_scheduler_fraction;
+    bool locality_scheduling_sorted_order;
+    int locality_scheduling_wait_period;
+    int locality_scheduling_send_timeout;
+    vector<regex_t> *locality_scheduling_workunit_file;
+    vector<regex_t> *locality_scheduling_sticky_file;
     bool matchmaker;
+    int max_ncpus;
+    JOB_LIMITS max_jobs_in_progress;
+    int max_wus_to_send;            // max results per RPC is this * mult
+    int min_core_client_version;
+    int min_core_client_version_announced;
+    int min_core_client_upgrade_deadline;
+    int min_sendwork_interval;
     int mm_min_slots;
     int mm_max_slots;
-    bool job_size_matching;
-    bool use_credit_multiplier;
-    bool multiple_clients_per_host;
+    double next_rpc_delay;
+    bool no_amd_k6;
+        // don't allow AMD K6 CPUs
     bool no_vista_sandbox;
-    bool ignore_dcf;
+    bool nowork_skip;
+    bool one_result_per_host_per_wu;
+    bool one_result_per_user_per_wu;
+    int reliable_max_avg_turnaround;
+        // max average turnaround for a host to be declared reliable
+    double reliable_max_error_rate;
+        // DEPRECATED
+        // max error rate for a host to be declared reliable
+    double reliable_reduced_delay_bound;
+        // Reduce the delay bounds for reliable hosts by this percent
+    char replace_download_url_by_timezone[256];
+    int max_download_urls_per_file;
     int report_max;
-    bool dont_store_success_stderr;
+    bool request_time_stats_log;
+    bool resend_lost_results;
+    int sched_debug_level;
+    char sched_lockfile_dir[256];
+    bool send_result_abort;
+    char symstore[256];
+    bool workload_sim;
+        // Do workload simulation in deciding whether to send a result
+    bool prefer_primary_platform;
+        // When selecting app versions,
+        // use the client's primary platform if a version exists.
+        // e.g. send 64-bit versions to 64-bit clients,
+        // rather than trying the 32-bit version to see if it's faster.
+        // Do this only if you're sure that your 64-bit versions are
+        // always faster than the corresponding 32-bit versions
 
-    // log flags
+    // scheduler log flags
     //
-    bool debug_version_select;
+    bool debug_array;               // debug job-cache scheduling
     bool debug_assignment;
-    bool debug_prefs;
-    bool debug_send;
-    bool debug_resend;
-    bool debug_request_headers;
-    bool debug_user_messages;
-    bool debug_request_details;
-    bool debug_handle_results;
-    bool debug_edf_sim_workload;    // show workload for EDF sim
+    bool debug_credit;
     bool debug_edf_sim_detail;      // show details of EDF sim
+    bool debug_edf_sim_workload;    // show workload for EDF sim
+    bool debug_fcgi;
+    bool debug_handle_results;
     bool debug_locality;            // locality scheduling
-    bool debug_array;               // debug old-style array scheduling
+    bool debug_prefs;
+    bool debug_quota;
+    bool debug_request_details;
+    bool debug_request_headers;
+    bool debug_resend;
+    bool debug_send;
+    bool debug_user_messages;
+    bool debug_version_select;
 
     int parse(FILE*);
+    int parse_aux(FILE*);
     int parse_file(const char *dir = 0);
 
     int upload_path(const char*, char*);
