@@ -27,12 +27,39 @@
 #include "sched_config.h"
 #include "sched_shmem.h"
 #include "str_util.h"
+#include "svn_version.h"
 
-int main() {
+void usage(char *name) {
+    fprintf(stderr,
+        "Displays the work_item part of shared-memory structure.\n\n"
+        "Usage: %s [OPTION]\n\n"
+        "Options:\n"
+        "  [ -h | --help ]        Show this help text.\n"
+        "  [ -v | --version ]     Shows version information.\n",
+        name
+    );
+}
+
+int main(int argc, char *argv[]) {
     SCHED_SHMEM* ssp;
     int retval;
     void* p;
 
+    for (int c = 1; c < argc; c++) {
+        std::string option(argv[c]);
+        if(option == "-h" || option == "--help") {
+            usage(argv[0]);
+            exit(0);
+        } else if(option == "-v" || option == "--version") {
+            printf("%s\n", SVN_VERSION);
+            exit(0);
+        } else {
+            fprintf(stderr, "unknown command line argument: %s\n\n", argv[c]);
+            usage(argv[0]);
+            exit(1);
+        }
+    }
+
     retval = config.parse_file();
     if (retval) {
         printf("Can't parse config.xml: %s\n", boincerror(retval));
@@ -48,4 +75,4 @@ int main() {
     ssp->show(stdout);
 }
 
-const char *BOINC_RCSID_a370415aab = "$Id: show_shmem.cpp 18042 2009-05-07 13:54:51Z davea $";
+const char *BOINC_RCSID_a370415aab = "$Id: show_shmem.cpp 21181 2010-04-15 03:13:56Z davea $";
