@@ -24,7 +24,7 @@
 //
 // Also updates the nusers field of teams
 //
-// usage: update_stats [-update_teams] [-update_users] [-update_hosts]
+// usage: update_stats [--update_teams] [--update_users] [--update_hosts]
 #include "config.h"
 #include <cstdio>
 #include <cstring>
@@ -36,6 +36,7 @@
 #include "util.h"
 #include "str_util.h"
 #include "error_numbers.h"
+#include "svn_version.h"
 
 #include "sched_config.h"
 #include "sched_util.h"
@@ -53,6 +54,7 @@ int update_users() {
     DB_USER user;
     int retval;
     char buf[256];
+    double now = dtime();
 
     while (1) {
         retval = user.enumerate("where expavg_credit>0.1");
@@ -65,7 +67,9 @@ int update_users() {
         }
 
         if (user.expavg_time > update_time_cutoff) continue;
-        update_average(0, 0, CREDIT_HALF_LIFE, user.expavg_credit, user.expavg_time);
+        update_average(
+            now, 0, 0, CREDIT_HALF_LIFE, user.expavg_credit, user.expavg_time
+        );
         sprintf( buf, "expavg_credit=%f, expavg_time=%f",
             user.expavg_credit, user.expavg_time
         );
@@ -83,6 +87,7 @@ int update_hosts() {
     DB_HOST host;
     int retval;
     char buf[256];
+    double now = dtime();
 
     while (1) {
         retval = host.enumerate("where expavg_credit>0.1");
@@ -95,7 +100,9 @@ int update_hosts() {
         }
 
         if (host.expavg_time > update_time_cutoff) continue;
-        update_average(0, 0, CREDIT_HALF_LIFE, host.expavg_credit, host.expavg_time);
+        update_average(
+            now, 0, 0, CREDIT_HALF_LIFE, host.expavg_credit, host.expavg_time
+        );
         sprintf(
             buf,"expavg_credit=%f, expavg_time=%f",
             host.expavg_credit, host.expavg_time
@@ -141,6 +148,7 @@ int update_teams() {
     DB_TEAM team;
     int retval;
     char buf[256];
+    double now = dtime();
 
     while (1) {
         retval = team.enumerate("where expavg_credit>0.1");
@@ -162,7 +170,10 @@ int update_teams() {
             continue;
         }
         if (team.expavg_time < update_time_cutoff) {
-            update_average(0, 0, CREDIT_HALF_LIFE, team.expavg_credit, team.expavg_time);
+            update_average(
+                now, 0, 0, CREDIT_HALF_LIFE, team.expavg_credit,
+                team.expavg_time
+            );
         }
         sprintf(
             buf, "expavg_credit=%f, expavg_time=%f, nusers=%d",
@@ -177,6 +188,26 @@ int update_teams() {
     return 0;
 }
 
+void usage(char *name) {
+    fprintf(stderr,
+        "Update average credit for idle users, hosts and teams.\n"
+        "These fields are updates as new credit is granted;\n"
+        "the purpose of this program is to decay credit of entities\n"
+        "that are inactive for long periods.\n"
+        "Hence it should be run about once a day at most.\n\n"
+        "Also updates the nusers field of teams\n\n"
+        "Usage: %s [OPTION]...\n\n"
+        "Options:\n"
+        "  [ -d X ]                        Set debug level to X\n"
+        "  [ --update_teams ]              Updates teams.\n"
+        "  [ --update_users ]              Updates users.\n"
+        "  [ --update_hosts ]              Updates hosts.\n"
+        "  [ -h | --help ]                 Shows this help text\n"
+        "  [ -v | --version ]              Shows version information\n",
+        name
+    );
+}
+
 int main(int argc, char** argv) {
     int retval, i;
     bool do_update_teams = false, do_update_users = false;
@@ -187,16 +218,31 @@ int main(int argc, char** argv) {
     check_stop_daemons();
 
     for (i=1; i<argc; i++) {
-        if (!strcmp(argv[i], "-update_teams")) {
+        if (is_arg(argv[i], "update_teams")) {
             do_update_teams = true;
-        } else if (!strcmp(argv[i], "-update_users")) {
+        } else if (is_arg(argv[i], "update_users")) {
             do_update_users = true;
-        } else if (!strcmp(argv[i], "-update_hosts")) {
+        } else if (is_arg(argv[i], "update_hosts")) {
             do_update_hosts = true;
         } else if (!strcmp(argv[i], "-d")) {
-            log_messages.set_debug_level(atoi(argv[++i]));
+            if (!argv[++i]) {
+                log_messages.printf(MSG_CRITICAL, "%s requires an argument\n\n", argv[--i]);
+                usage(argv[0]);
+                exit(1);
+            }
+            int dl = atoi(argv[i]);
+            log_messages.set_debug_level(dl);
+            if (dl == 4) g_print_queries = true;
+        } else if (!strcmp(argv[i], "-v") || !strcmp(argv[i], "--version")) {
+            printf("%s\n", SVN_VERSION);
+            exit(0);
+        } else if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help")) {
+            usage(argv[0]);
+            exit(0);
         } else {
-            log_messages.printf(MSG_CRITICAL, "Unrecognized arg: %s\n", argv[i]);
+            log_messages.printf(MSG_CRITICAL, "unknown command line argument: %s\n\n", argv[i]);
+            usage(argv[0]);
+            exit(1);
         }
     }
 
@@ -249,4 +295,4 @@ int main(int argc, char** argv) {
     return 0;
 }
 
-const char *BOINC_RCSID_6b05e9ecce = "$Id: update_stats.cpp 18042 2009-05-07 13:54:51Z davea $";
+const char *BOINC_RCSID_6b05e9ecce = "$Id: update_stats.cpp 22642 2010-11-08 17:40:52Z romw $";
